@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -35,7 +36,7 @@
 /* Bucket size = BUCKET_TIME * Bandwidth-Max radius attribute */
 /* Not used if BUCKET_SIZE is defined */
 #define BUCKET_TIME                        1 /* 1 seconds */
-#define BUCKET_SIZE_MIN                 5000 /* Minimum size of leaky bucket (~10 packets) */
+#define BUCKET_SIZE_MIN                 7000 /* Minimum size of leaky bucket (~10 packets) */
 
 #define CHECK_INTERVAL                     3 /* Time between checking connections */
 
@@ -46,14 +47,14 @@
 /* redir */
 #define REDIR_MAXLISTEN                   32
 #define REDIR_MAXTIME                    120 /* Seconds */
-#define REDIR_HTTP_MAX_TIME               10 /* Seconds */
-#define REDIR_HTTP_SELECT_TIME             2 /* Seconds */
+#define REDIR_HTTP_MAX_TIME               20 /* Seconds */
+#define REDIR_HTTP_SELECT_TIME             3 /* Seconds */
 #define REDIR_RADIUS_MAX_TIME             60 /* Seconds */
 #define REDIR_RADIUS_SELECT_TIME      500000 /* microseconds = 0.5 seconds */
 #define REDIR_CHALLEN                     16
 #define REDIR_MD5LEN                      16
 #define REDIR_MACSTRLEN                   17
-#define REDIR_MAXBUFFER                 5125
+#define REDIR_MAXBUFFER                65535
 
 #define REDIR_USERNAMESIZE               256 /* Max length of username */
 #define REDIR_MAXQUERYSTRING            2048
@@ -88,7 +89,6 @@
 #define RADIUS_CHAPSIZE                   24 
 #define RADIUS_QUEUESIZE                 256 /* Same size as id address space */
 #define RADIUS_ATTR_VLEN                 253
-#define RADIUS_PACKSIZE                 4096
 #define RADIUS_HDRSIZE                    20
 #define RADIUS_MPPEKEYSSIZE               32 /* Length of MS_CHAP_MPPE_KEYS attribute */ 
 
@@ -111,24 +111,35 @@
 #define MAX_RAWIF 1
 #endif
 
+#define MAX_EAP_LEN                     3000 /* Size of buffer to hold EAP message */
+
 #ifdef ENABLE_LARGELIMITS
 #define PKT_MAX_LEN                    65535 /* Maximum packet size we receive */
-#define MAX_EAP_LEN                     3000 /* Size of buffer to hold EAP message */
 #define SESSION_PASS_THROUGH_MAX          16
 #define MAX_PASS_THROUGHS               1024 /* Max number of allowed UAM pass-throughs */
 #define MAX_REGEX_PASS_THROUGHS          512 /* Max number of allowed UAM pass-throughs */
 #define MAX_UAM_DOMAINS                  128 /* Max number of allowed UAM domains */
 #define MACOK_MAX                         56
 #define MAX_SELECT                        56
+#define RADIUS_PACKSIZE                 4096
 #else
-#define PKT_MAX_LEN                     5000 /* Maximum packet size we receive */
-#define MAX_EAP_LEN                     1500 /* Size of buffer to hold EAP message */
+#define PKT_MAX_LEN                     9000 /* Maximum packet size we receive */
 #define SESSION_PASS_THROUGH_MAX           8
 #define MAX_PASS_THROUGHS                128 /* Max number of allowed UAM pass-throughs */
 #define MAX_REGEX_PASS_THROUGHS            8 /* Max number of allowed UAM pass-throughs */
 #define MAX_UAM_DOMAINS                   32 /* Max number of allowed UAM domains */
 #define MACOK_MAX                         16
 #define MAX_SELECT                        16
+#define RADIUS_PACKSIZE                 1600
+#define RADIUS_QUEUE_PACKET_PTR 1
+#endif
+
+#ifdef ENABLE_LAYER3
+#ifdef ENABLE_LARGELIMITS
+#define MAX_IPSRC_PASS_THROUGHS               512 /* Max number of allowed IP source addresses, for Layer3 */
+#else
+#define MAX_IPSRC_PASS_THROUGHS               8 /* Max number of allowed IP source addresses, for Layer3 */
+#endif
 #endif
 
 #endif
