@@ -1,9 +1,10 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -23,7 +24,7 @@
 
 extern struct dhcp_t *dhcp;
 
-static ssize_t
+ssize_t
 dns_fullname(char *data, size_t dlen,      /* buffer to store name */
 	     uint8_t *res, size_t reslen,  /* current resource */
 	     uint8_t *opkt, size_t olen,   /* original packet */
@@ -35,8 +36,8 @@ dns_fullname(char *data, size_t dlen,      /* buffer to store name */
   if (lvl >= 15) return -1;
 
 #if(_debug_ > 1)
-  log_dbg("dlen=%d reslen=%d olen=%d lvl=%d", 
-	  dlen, reslen, olen, lvl);
+  log_dbg("%s dlen=%d reslen=%d olen=%d lvl=%d", 
+	  __FUNCTION__, dlen, reslen, olen, lvl);
 #endif
 
   /* only capture the first name in query */
@@ -115,8 +116,11 @@ add_A_to_garden(uint8_t *p) {
   if (pass_through_add(dhcp->pass_throughs,
 		       MAX_PASS_THROUGHS,
 		       &dhcp->num_pass_throughs,
-		       &dhcp->next_pass_through,
-		       &pt))
+		       &pt, 1
+#ifdef HAVE_PATRICIA
+		       , dhcp->ptree_dyn
+#endif
+		       ))
     ;
 }
 
@@ -134,12 +138,14 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
   
   uint8_t name[PKT_IP_PLEN];
   ssize_t namelen = 0;
+  char required = 0;
   
   uint16_t type;
   uint16_t class;
   uint32_t ttl;
   uint16_t rdlen;
 
+  uint8_t *pkt_type=0;
   uint8_t *pkt_ttl=0;
 
   uint32_t ul;
@@ -151,8 +157,10 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
 #endif
 
   memset(name, 0, sizeof(name));
-  namelen = dns_fullname((char*)name, sizeof(name)-1, p_pkt, len, opkt, olen, 0);
-  if (namelen < 0) return_error;
+  namelen = dns_fullname((char*)name, sizeof(name)-1, 
+			 p_pkt, len, opkt, olen, 0);
+
+  if (namelen < 0 || namelen > len) return_error;
 
   p_pkt += namelen;
   len -= namelen;
@@ -164,6 +172,7 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
 
   if (len < 4) return_error;
 
+  pkt_type = p_pkt;
   memcpy(&us, p_pkt, sizeof(us));
   type = ntohs(us);
   p_pkt += 2;
@@ -178,28 +187,6 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
   log_dbg("It was a dns record type: %d class: %d", type, class);
 #endif
 
-  /* if dnsparanoia, checks here */
-
-  if (antidnstunnel) {
-    switch (type) {
-    case 1:/* A */ 
-    case 5:/* CNAME */ 
-      break;
-    default:
-#if(_debug_ > 1)
-      if (_options.debug) switch(type) {
-	case 6:  log_dbg("SOA record"); break;
-	case 12: log_dbg("PTR record"); break;
-	case 15: log_dbg("MX record");  break;
-	case 16: log_dbg("TXT record"); break;
-	default: log_dbg("Record type %d", type); break;
-	}
-#endif
-      log_warn(0, "dropping dns for anti-dnstunnel (type %d: length %d)", type, (int) namelen);
-      return -1;
-    }
-  }
-  
   if (q) {
     if (dns_fullname((char *)question, qsize, *pktp, *left, opkt, olen, 0) < 0)
       return_error;
@@ -209,7 +196,8 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
     *pktp = p_pkt;
     *left = len;
 
-    if (!isReq && *qmatch == -1 && _options.uamdomains && _options.uamdomains[0]) {
+    if (!isReq && *qmatch == -1 && 
+	_options.uamdomains && _options.uamdomains[0]) {
       int id;
 
       for (id=0; _options.uamdomains[id] && id < MAX_UAM_DOMAINS; id++) {
@@ -254,6 +242,24 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
     }
 #endif
 
+#ifdef ENABLE_IPV6
+    if (_options.ipv6) {
+      if (isReq && type == 28) {
+	log_dbg("changing AAAA to A request");
+	us = 1;
+	us = htons(us);
+	memcpy(pkt_type, &us, sizeof(us));
+	*modified = 1;
+      } else if (!isReq && type == 1) {
+	log_dbg("changing A to AAAA response");
+	us = 28;
+	us = htons(us);
+	memcpy(pkt_type, &us, sizeof(us));
+	*modified = 1;
+      }
+    }
+#endif
+
     return 0;
   } 
 
@@ -284,7 +290,7 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
     memcpy(pkt_ttl, &ul, sizeof(ul));
     *modified = 1;
   }
-      
+
   if (len < rdlen) return_error;
   
   /*
@@ -292,11 +298,17 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
    */  
   
   switch (type) {
+
+  default: 
+    log_dbg("Record type %d", type);
+    return_error;
+    break;
     
-  case 1:  /* A */
+  case 1:  
 #if(_debug_ > 1)
     log_dbg("A record");
 #endif
+    required = 1;
 
 #ifdef ENABLE_MDNS
     if (mode == DNS_MDNS_MODE) {
@@ -320,10 +332,13 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
     }
     break;
 
-  case 5:  /* CNAME */
-    log_dbg("CNAME record %s", name);
-    break;
+  case 2: log_dbg("NS record"); required = 1; break;
+  case 5: log_dbg("CNAME record %s", name); required = 1; break;
+  case 6: log_dbg("SOA record"); break;
     
+  case 12: log_dbg("PTR record"); break;
+  case 15: log_dbg("MX record"); required = 1; break;
+
   case 16:/* TXT */
     log_dbg("TXT record %d", rdlen);
     if (_options.debug) {
@@ -339,22 +354,20 @@ dns_copy_res(struct dhcp_conn_t *conn, int q,
     }
     break;
 
-  default:
-
-    if (_options.debug) switch(type) {
-      case 6:  log_dbg("SOA record"); break;
-      case 12: log_dbg("PTR record"); break;
-      case 15: log_dbg("MX record");  break;
-      case 47: log_dbg("NSEC record"); break;
-      default: log_dbg("Record type %d", type); break;
-    }
-
-    if (antidnstunnel) {
-      log_warn(0, "dropping dns for anti-dnstunnel (type %d: length %d)", type, rdlen);
-      return -1;
-    }
-
+  case 28: 
+    log_dbg("AAAA record"); 
+    required = 1; 
     break;
+  case 29: log_dbg("LOC record"); break;
+  case 33: log_dbg("SRV record"); break;
+  case 41: log_dbg("EDNS OPT pseudorecord"); break;
+  case 47: log_dbg("NSEC record"); break;
+  }
+
+  if (antidnstunnel && !required) {
+    log_warn(0, "dropping dns for anti-dnstunnel (type %d: length %d)", 
+	     type, rdlen);
+    return -1;
   }
   
   p_pkt += rdlen;
