@@ -1,9 +1,10 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -28,6 +29,12 @@ int
 dns_getname(uint8_t **pktp, size_t *left,
 	    char *name, size_t namesz, size_t *nameln);
 
+ssize_t
+dns_fullname(char *data, size_t dlen,
+	     uint8_t *res, size_t reslen,
+	     uint8_t *opkt, size_t olen, 
+	     int lvl);
+
 int 
 dns_copy_res(struct dhcp_conn_t *conn, int q, 
 	     uint8_t **pktp, size_t *left, 
