@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -99,7 +100,8 @@ int iphash_get(struct iphash_t *this, struct iphashm_t **member,  struct in_addr
 
   hash = iphash_hash4(addr) & this->hashmask;
   for (p = this->hash[hash]; p; p = p->nexthash) {
-    if ( p->addr.s_addr == addr->s_addr && p->port == port &&
+    if ( p->addr.s_addr == addr->s_addr && 
+	 (p->port == 0 || p->port == port) &&
 	 p->flags & IPHASH_INUSE ) {
       if (member) *member = p;
       return 0;
