@@ -1,6 +1,7 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -207,10 +208,10 @@ int ippool_new(struct ippool_t **this,
   struct in_addr stataddr;
   struct in_addr statmask;
   struct in_addr naddr;
-  unsigned int m;
-  unsigned int listsize;
-  unsigned int dynsize;
-  unsigned int statsize;
+  uint32_t m;
+  uint32_t listsize;
+  uint32_t dynsize;
+  uint32_t statsize;
 
   if (!allowdyn) {
     dynsize = 0;
@@ -221,15 +222,24 @@ int ippool_new(struct ippool_t **this,
       return -1;
     }
 
+    /* auto-dhcpstart if not already set */
+    if (!start) 
+      start = ntohl(addr.s_addr & ~(mask.s_addr));
+
+    /* ensure we have the true network space */
+    addr.s_addr = addr.s_addr & mask.s_addr;
+
     m = ntohl(mask.s_addr);
     dynsize = ((~m)+1); 
 
-    if ( ((ntohl(addr.s_addr) + start)&m) != (ntohl(addr.s_addr)&m) ) {
-      log_err(0, "Invalid dhcpstart (outside of subnet)!");
+    if ( ((ntohl(addr.s_addr) + start) & m) != (ntohl(addr.s_addr) & m) ) {
+      addr.s_addr = htonl(ntohl(addr.s_addr) + start);
+      log_err(0, "Invalid dhcpstart=%d (%s) (outside of subnet)!",
+	      start, inet_ntoa(addr));
       return -1;
     }
 
-    if ( ((ntohl(addr.s_addr) + end)&m) != (ntohl(addr.s_addr)&m) ) {
+    if ( ((ntohl(addr.s_addr) + end) & m) != (ntohl(addr.s_addr) & m) ) {
       log_err(0, "Invalid dhcpend (outside of subnet)!");
       return -1;
     }
@@ -242,7 +252,8 @@ int ippool_new(struct ippool_t **this,
       }
 
       if ((end - start) > dynsize) {
-	log_err(0, "Too many IPs between dhcpstart=%d and dhcpend=%d for network", start, end);
+	log_err(0, "Too many IPs between dhcpstart=%d and dhcpend=%d",
+		start, end);
 	return -1;
       }
 
@@ -288,6 +299,9 @@ int ippool_new(struct ippool_t **this,
       return -1;
     }
 
+    /* ensure we have the true network space */
+    stataddr.s_addr = stataddr.s_addr & statmask.s_addr;
+
     m = ntohl(statmask.s_addr);
     statsize = ((~m)+1);
 
@@ -320,14 +334,16 @@ int ippool_new(struct ippool_t **this,
        ((1 << (*this)->hashlog) < listsize);
        (*this)->hashlog++);
 
-  log_dbg("Hashlog %d %d %d\n", (*this)->hashlog, listsize, (1 << (*this)->hashlog));
+  log_dbg("Hashlog %d %d %d", (*this)->hashlog, listsize, 
+	  (1 << (*this)->hashlog));
 
   /* Determine hashsize */
   (*this)->hashsize = 1 << (*this)->hashlog; /* Fails if mask=0: All Internet*/
   (*this)->hashmask = (*this)->hashsize -1;
   
   /* Allocate hash table */
-  if (!((*this)->hash = calloc(sizeof(struct ippoolm_t *), (*this)->hashsize))){
+  if (!((*this)->hash = 
+	calloc(sizeof(struct ippoolm_t *), (*this)->hashsize))){
     log_err(0, "Failed to allocate memory for hash members in ippool");
     return -1;
   }
@@ -341,7 +357,8 @@ int ippool_new(struct ippool_t **this,
   for (i = 0; i < dynsize; i++) {
 
     naddr.s_addr = htonl(ntohl(addr.s_addr) + i + start);
-    if (naddr.s_addr == _options.uamlisten.s_addr) {
+    if (naddr.s_addr == _options.uamlisten.s_addr ||
+	naddr.s_addr == _options.dhcplisten.s_addr) {
       start++; /* skip the uamlisten address! */
       naddr.s_addr = htonl(ntohl(addr.s_addr) + i + start);
     }
@@ -392,7 +409,6 @@ int ippool_new(struct ippool_t **this,
 }
 
 
-
 /* Delete existing address pool */
 int ippool_free(struct ippool_t *this) {
   free(this->hash);
@@ -436,7 +452,8 @@ int ippool_newip(struct ippool_t *this,
   struct ippoolm_t *p2 = NULL;
   uint32_t hash;
   
-  log_dbg("Requesting new %s ip: %s", statip ? "static" : "dynamic", inet_ntoa(*addr));
+  log_dbg("Requesting new %s ip: %s", 
+	  statip ? "static" : "dynamic", inet_ntoa(*addr));
 
   /* If static:
    *   Look in dynaddr. 
@@ -459,6 +476,7 @@ int ippool_newip(struct ippool_t *this,
   if ((addr) && (addr->s_addr) && statip) { /* IP address given */
 #ifdef ENABLE_UAMANYIP
     if (!_options.uamanyip) {
+      log_dbg("HERE!!! -> %d", _options.uamanyip);
 #endif
       if (!this->allowstat) {
 	log_dbg("Static IP address not allowed");
@@ -631,7 +649,7 @@ int ippool_freeip(struct ippool_t *this, struct ippoolm_t *member) {
   }
 
   if (member->is_static) {
-
+    
     if (ippool_hashdel(this, member))
       return -1;
 
@@ -643,7 +661,7 @@ int ippool_freeip(struct ippool_t *this, struct ippoolm_t *member) {
     else {
       this->firststat = member;
     }
-
+    
     this->laststat = member;
     
     member->in_use = 0;
