@@ -1,9 +1,10 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -19,8 +20,9 @@
 
 #define MAIN_FILE
 
-#include "chilli.h"
 #include "cmdline.h"
+#include "system.h"
+#include "chilli.h"
 #include "debug.h"
 
 struct options_t _options;
@@ -31,13 +33,16 @@ static const char *description =
   "  http://www.coova.org/\n";
 
 static const char *copyright = 
-  "2006-2011 Coova Technologies LLC., Copyright (c) 2003-2005 Mondru AB.\n"
+  "2006-2013 David Bird (Coova Technologies), Copyright (c) 2003-2005 Mondru AB.\n"
   "Licensed under the GNU General Public License (GPL).\n";
 
 static const char *usage =
   "Usage: chilli [OPTIONS]...\n";
 
 static const char *compile_options = "Compiled with "
+#ifdef ENABLE_AUTHEDALLOWED
+  "ENABLE_AUTHEDALLOWED "
+#endif
 #ifdef ENABLE_BINSTATFILE
   "ENABLE_BINSTATFILE "
 #endif
@@ -83,6 +88,9 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_EWTAPI
   "ENABLE_EWTAPI "
 #endif
+#ifdef ENABLE_GARDENACCOUNTING
+  "ENABLE_GARDENACCOUNTING "
+#endif
 #ifdef ENABLE_IEEE8021Q
   "ENABLE_IEEE8021Q "
 #endif
@@ -104,9 +112,15 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_LEAKYBUCKET
   "ENABLE_LEAKYBUCKET "
 #endif
+#ifdef ENABLE_LOCATION
+  "ENABLE_LOCATION "
+#endif
 #ifdef ENABLE_MINIPORTAL
   "ENABLE_MINIPORTAL "
 #endif
+#ifdef ENABLE_MULTILAN
+  "ENABLE_MULTILAN "
+#endif
 #ifdef ENABLE_MULTIROUTE
   "ENABLE_MULTIROUTE "
 #endif
@@ -119,12 +133,21 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_PROXYVSA
   "ENABLE_PROXYVSA "
 #endif
+#ifdef ENABLE_EXTADMVSA
+  "ENABLE_EXTADMVSA "
+#endif
 #ifdef ENABLE_REDIRDNSREQ
   "ENABLE_REDIRDNSREQ "
 #endif
 #ifdef ENABLE_SESSGARDEN
   "ENABLE_SESSGARDEN "
 #endif
+#ifdef ENABLE_SESSIONID
+  "ENABLE_SESSIONID "
+#endif
+#ifdef ENABLE_SESSIONSTATE
+  "ENABLE_SESSIONSTATE "
+#endif
 #ifdef ENABLE_SSDP
   "ENABLE_SSDP "
 #endif
@@ -137,15 +160,27 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_UAMUIPORT
   "ENABLE_UAMUIPORT "
 #endif
+#ifdef ENABLE_USERAGENT
+  "ENABLE_USERAGENT "
+#endif
 #ifdef HAVE_MATRIXSSL
   "HAVE_MATRIXSSL "
 #endif
+#ifdef HAVE_NETFILTER_COOVA
+  "HAVE_NETFILTER_COOVA "
+#endif
 #ifdef HAVE_OPENSSL
   "HAVE_OPENSSL "
 #endif
+#ifdef USING_MMAP
+  "USING_MMAP "
+#endif
 #ifdef USING_POLL
   "USING_POLL "
 #endif
+#ifdef EX_OPT_FEATURES
+#include EX_OPT_FEATURES
+#endif
 ;
 
 char *STRDUP(char *s) {
@@ -217,6 +252,7 @@ int main(int argc, char **argv) {
   char hostname[USERURLSIZE];
   int numargs;
   int ret = -1;
+  int i;
 
   options_init();
 
@@ -226,7 +262,7 @@ int main(int argc, char **argv) {
     log_err(0, "Failed to parse command line options");
     goto end_processing;
   }
-
+  
   if (args_info.version_given) {
     options_print_version();
     exit(2);
@@ -243,13 +279,16 @@ int main(int argc, char **argv) {
 				&args_info, 0, 0, 0)) {
     log_err(0, "Failed to parse configuration file: %s!", 
 	    args_info.conf_arg);
-    goto end_processing;
+    if (!args_info.forgiving_flag)
+      goto end_processing;
   }
 
   /* Get the system default DNS entries */
-  if (res_init()) {
-    log_err(0, "Failed to update system DNS settings (res_init()!");
-    goto end_processing;
+  if (!args_info.nosystemdns_flag) {
+    if (res_init()) {
+      log_err(0, "Failed to update system DNS settings (res_init()!");
+      goto end_processing;
+    }
   }
 
   /* Handle each option */
@@ -260,6 +299,10 @@ int main(int argc, char **argv) {
   else 
     _options.debug = 0;
 
+  /* pass-throughs */
+  memset(_options.pass_throughs, 0, sizeof(_options.pass_throughs));
+  _options.num_pass_throughs = 0;
+
   /** simple configuration parameters **/
   _options.layer3 = args_info.layer3_flag;
 #if(_debug_ && !defined(ENABLE_LAYER3))
@@ -270,6 +313,11 @@ int main(int argc, char **argv) {
   _options.gid = args_info.gid_arg;
   _options.mtu = args_info.mtu_arg;
   _options.usetap = args_info.usetap_flag;
+  _options.noarpentries = args_info.noarpentries_flag;
+#if(_debug_ && !defined(ENABLE_TAP))
+  if (_options.noarpentries) 
+    log_warn(0, "tap not implemented. build with --enable-tap");
+#endif
 #if(_debug_ && !defined(ENABLE_TAP))
   if (_options.usetap) 
     log_warn(0, "tap not implemented. build with --enable-tap");
@@ -317,31 +365,44 @@ int main(int argc, char **argv) {
   _options.radiustimeout = args_info.radiustimeout_arg;
   _options.radiusretry = args_info.radiusretry_arg;
   _options.radiusretrysec = args_info.radiusretrysec_arg;
+#ifdef ENABLE_RADPROXY
   _options.proxyport = args_info.proxyport_arg;
+  _options.proxymacaccept = args_info.proxymacaccept_flag;
+  _options.proxyonacct = args_info.proxyonacct_flag;
+#endif
 #if(_debug_ && !defined(ENABLE_RADPROXY))
-  if (_options.proxyport)
+  if (args_info.proxyport_arg)
     log_err(0,"radproxy not implemented. build with --enable-radproxy");
 #endif
   _options.txqlen = args_info.txqlen_arg;
+#ifdef USING_MMAP
   _options.ringsize = args_info.ringsize_arg;
+  _options.mmapring = args_info.mmapring_flag;
+#endif
   _options.sndbuf = args_info.sndbuf_arg;
   _options.rcvbuf = args_info.rcvbuf_arg;
+  _options.childmax = args_info.childmax_arg;
   _options.postauth_proxyport = args_info.postauthproxyport_arg;
+  _options.postauth_proxyssl = args_info.postauthproxyssl_flag;
   _options.pap_always_ok = args_info.papalwaysok_flag;
   _options.mschapv2 = args_info.mschapv2_flag;
   _options.acct_update = args_info.acctupdate_flag;
   _options.dhcpradius = args_info.dhcpradius_flag;
-  _options.ieee8021q = args_info.ieee8021q_flag;
   _options.dhcp_broadcast = args_info.dhcpbroadcast_flag;
   _options.dhcpgwport = args_info.dhcpgatewayport_arg;
   _options.noc2c = args_info.noc2c_flag;
   _options.tcpwin = args_info.tcpwin_arg;
   _options.tcpmss = args_info.tcpmss_arg;
   _options.max_clients = args_info.maxclients_arg;
+  _options.radiusqsize = args_info.radiusqsize_arg;
+  _options.dhcphashsize = args_info.dhcphashsize_arg;
   _options.uamdomain_ttl = args_info.uamdomainttl_arg;
   _options.seskeepalive = args_info.seskeepalive_flag;
   _options.uamallowpost = args_info.uamallowpost_flag;
   _options.redir = args_info.redir_flag;
+  _options.redirurl = args_info.redirurl_flag;
+  _options.statusfilesave = args_info.statusfilesave_flag;
+  _options.dhcpnotidle = args_info.dhcpnotidle_flag;
 #if(_debug_ && !defined(ENABLE_CHILLIREDIR))
   if (_options.redir) 
     log_err(0, "chilli_redir not implemented. build with --enable-chilliredir");
@@ -355,7 +416,6 @@ int main(int argc, char **argv) {
   if (_options.radsec) 
     log_err(0, "chilli_radsec not implemented. build with --enable-chilliradsec");
 #endif
-  _options.proxymacaccept = args_info.proxymacaccept_flag;
   _options.noradallow = args_info.noradallow_flag;
   _options.peerid = args_info.peerid_arg;
 #if(_debug_ && !defined(ENABLE_CLUSTER))
@@ -368,13 +428,44 @@ int main(int argc, char **argv) {
     log_err(0, "redirdnsreq not implemented. build with --enable-redirdnsreq");
 #endif
 
+#ifdef ENABLE_IPV6
+  _options.ipv6 = args_info.ipv6_flag;
+  _options.ipv6only = args_info.ipv6only_flag;
+  if (args_info.ipv6mode_arg) {
+    if (!strcmp(args_info.ipv6mode_arg, "4to6")) {
+      _options.ipv6 = 1;
+      _options.ipv4to6 = 1;
+      _options.ipv6only = 0;
+    } else if (!strcmp(args_info.ipv6mode_arg, "6to4")) {
+      _options.ipv6 = 1;
+      _options.ipv6to4 = 1;
+    } else if (!strcmp(args_info.ipv6mode_arg, "6and4") ||
+	       !strcmp(args_info.ipv6mode_arg, "4and6")) {
+      _options.ipv6 = 1;
+      _options.ipv6only = 0;
+    } else {
+      log_warn(0, "unknown ipv6mode %s", args_info.ipv6mode_arg);
+      _options.ipv6 = 0;
+    }
+  }
+  log_dbg("IPv6 %sabled %s", 
+	  _options.ipv6 ? "en" : "dis", 
+	  args_info.ipv6mode_arg ? args_info.ipv6mode_arg : "");
+#endif
+
 #ifdef ENABLE_LEAKYBUCKET
+  _options.scalewin = args_info.scalewin_flag;
   _options.bwbucketupsize = args_info.bwbucketupsize_arg;
   _options.bwbucketdnsize = args_info.bwbucketdnsize_arg;
   _options.bwbucketminsize = args_info.bwbucketminsize_arg;
 #endif
 
 #ifdef ENABLE_PROXYVSA
+  _options.vlanlocation = args_info.vlanlocation_flag;
+  _options.location_stop_start = args_info.locationstopstart_flag;
+  _options.location_copy_called = args_info.locationcopycalled_flag;
+  _options.location_immediate_update = args_info.locationimmediateupdate_flag;
+  _options.location_option_82 = args_info.locationopt82_flag;
   if (args_info.proxylocattr_given) {
     for (numargs = 0; numargs < args_info.proxylocattr_given 
 	   && numargs < PROXYVSA_ATTR_CNT; ++numargs)  {
@@ -404,20 +495,41 @@ int main(int argc, char **argv) {
   if (args_info.dhcpgateway_arg &&
       !inet_aton(args_info.dhcpgateway_arg, &_options.dhcpgwip)) {
     log_err(0, "Invalid DHCP gateway IP address: %s!", args_info.dhcpgateway_arg);
-    goto end_processing;
+    if (!args_info.forgiving_flag)
+      goto end_processing;
   }
 
   if (args_info.dhcprelayagent_arg &&
       !inet_aton(args_info.dhcprelayagent_arg, &_options.dhcprelayip)) {
     log_err(0, "Invalid DHCP gateway relay IP address: %s!", args_info.dhcprelayagent_arg);
-    goto end_processing;
+    if (!args_info.forgiving_flag)
+      goto end_processing;
   }
 
   _options.dhcpif = STRDUP(args_info.dhcpif_arg);
 
+#ifdef ENABLE_MULTILAN
+  for (numargs = 0; numargs < args_info.moreif_given &&
+	 numargs < MAX_MOREIF; ++numargs) {
+    char *nif = STRDUP(args_info.moreif_arg[numargs]);
+    char *vln = strchr(nif, '/');
+    _options.moreif[numargs].dhcpif = nif;
+    if (vln) {
+      if (strlen(vln) > 1) 
+	_options.moreif[numargs].vlan = vln + 1;
+      *vln = 0;
+    } else {
+      vln = strchr(nif, '.');
+      if (vln && strlen(vln) > 1) 
+	_options.moreif[numargs].vlan = vln + 1;
+    }
+  }
+#endif
+
   if (!args_info.radiussecret_arg) {
     log_err(0, "radiussecret must be specified!");
-    goto end_processing;
+    if (!args_info.forgiving_flag)
+      goto end_processing;
   }
 
   if (!args_info.nexthop_arg) {
@@ -432,7 +544,8 @@ int main(int argc, char **argv) {
 
     if ((macstrlen = strlen(args_info.nexthop_arg)) >= (RADIUS_ATTR_VLEN-1)) {
       log_err(0, "MAC address too long");
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
 
     memcpy(macstr, args_info.nexthop_arg, macstrlen);
@@ -469,7 +582,8 @@ int main(int argc, char **argv) {
 
     if ((macstrlen = strlen(args_info.dhcpmac_arg)) >= (RADIUS_ATTR_VLEN-1)) {
       log_err(0, "MAC address too long");
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
 
     memcpy(macstr, args_info.dhcpmac_arg, macstrlen);
@@ -497,29 +611,32 @@ int main(int argc, char **argv) {
   if (args_info.net_arg) {
     if (option_aton(&_options.net, &_options.mask, args_info.net_arg, 0)) {
       log_err(0, "Invalid network address: %s!", args_info.net_arg);
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
     if (!args_info.uamlisten_arg) {
       _options.uamlisten.s_addr = htonl(ntohl(_options.net.s_addr)+1);
     }
     else if (!inet_aton(args_info.uamlisten_arg, &_options.uamlisten)) {
       log_err(0, "Invalid UAM IP address: %s!", args_info.uamlisten_arg);
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
     if (!args_info.dhcplisten_arg) {
       _options.dhcplisten.s_addr = _options.uamlisten.s_addr;
     }
     else if (!inet_aton(args_info.dhcplisten_arg, &_options.dhcplisten)) {
       log_err(0, "Invalid DHCP IP address: %s!", args_info.dhcplisten_arg);
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
   }
   else {
     log_err(0, "Network address must be specified ('net' parameter)!");
-    goto end_processing;
+    if (!args_info.forgiving_flag)
+      goto end_processing;
   }
 
-
   log_dbg("DHCP Listen: %s", inet_ntoa(_options.dhcplisten));
   log_dbg("UAM Listen: %s", inet_ntoa(_options.uamlisten));
 
@@ -528,78 +645,79 @@ int main(int argc, char **argv) {
   }
 
   if (args_info.uamserver_arg) {
+    int uamserverport=80;
+
     if (_options.debug & DEBUG_CONF) {
       log_dbg("Uamserver: %s\n", args_info.uamserver_arg);
     }
-    memset(_options.uamserver, 0, sizeof(_options.uamserver));
-    _options.uamserverlen = 0;
+
     if (get_urlparts(args_info.uamserver_arg, hostname, USERURLSIZE, 
-		     &_options.uamserverport, 0)) {
+		     &uamserverport, 0)) {
       log_err(0, "Failed to parse uamserver: %s!", args_info.uamserver_arg);
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
 
     if (!args_info.uamaliasname_arg ||
-	strncmp(args_info.uamaliasname_arg, hostname, strlen(args_info.uamaliasname_arg))) {
+	strncmp(args_info.uamaliasname_arg, hostname, 
+		strlen(args_info.uamaliasname_arg))) {
       if (!(host = gethostbyname(hostname))) {
 	log_err(0, "Could not resolve IP address of uamserver: %s!", 
 		args_info.uamserver_arg);
-	goto end_processing;
       }
       else {
 	int j = 0;
+	pass_through pt;
+
+	memset(&pt, 0, sizeof(pt));
+	pt.port = uamserverport;
+	pt.mask.s_addr = ~0;
+
 	while (host->h_addr_list[j] != NULL) {
 	  if (_options.debug & DEBUG_CONF) {
 	    log_dbg("Uamserver IP address #%d: %s\n", j,
 		    inet_ntoa(*(struct in_addr*) host->h_addr_list[j]));
 	  }
-	  if (_options.uamserverlen>=UAMSERVER_MAX) {
-	    log_err(0,
-		    "Too many IPs in uamserver %s!",
-		    args_info.uamserver_arg);
-	    goto end_processing;
-	  }
-	  else {
-	    _options.uamserver[_options.uamserverlen++] = 
-	      *((struct in_addr*) host->h_addr_list[j++]);
-	  }
+
+	  pt.host.s_addr = ((struct in_addr*) host->h_addr_list[j++])->s_addr;
+
+	  if (pass_through_add(_options.pass_throughs,
+			       MAX_PASS_THROUGHS,
+			       &_options.num_pass_throughs, &pt, 0
+#ifdef HAVE_PATRICIA
+			       , 0
+#endif
+			       ))
+	    log_err(0, "Too many pass-throughs! skipped %s:%d",
+		    inet_ntoa(pt.host), pt.port);
 	}
       }
     }
   }
 
-  if (args_info.uamhomepage_arg && *args_info.uamhomepage_arg) {
-    if (get_urlparts(args_info.uamhomepage_arg, hostname, USERURLSIZE, 
-		     &_options.uamhomepageport, 0)) {
-      log_err(0,"Failed to parse uamhomepage: %s!", 
-	      args_info.uamhomepage_arg);
-      goto end_processing;
-    }
-
-    if (!args_info.uamaliasname_arg ||
-	strncmp(args_info.uamaliasname_arg, hostname, strlen(args_info.uamaliasname_arg))) {
-      if (!(host = gethostbyname(hostname))) {
-	log_err(0,"Could not resolve uamhomepage server IP %s!", 
-		args_info.uamhomepage_arg);
-	/*goto end_processing;*/
-      }
-      else {
-	int j = 0;
-	while (host->h_addr_list[j] != NULL) {
-	  if (_options.uamserverlen>=UAMSERVER_MAX) {
-	    log_err(0,"Too many IPs in uamhomepage %s!",
-		    args_info.uamhomepage_arg);
-	    goto end_processing;
-	  }
-	  else {
-	    _options.uamserver[_options.uamserverlen++] = 
-	      *((struct in_addr*) host->h_addr_list[j++]);
-	  }
-	}
-      }
+#ifdef ENABLE_FORCEDNS
+  _options.forcedns1_port = args_info.forcedns1port_arg;
+  if (args_info.forcedns1port_arg && !args_info.forcedns1_arg) {
+    _options.forcedns1_addr.s_addr = _options.uamlisten.s_addr;
+  } else if (args_info.forcedns1_arg) {
+    if (!inet_aton(args_info.forcedns1_arg, &_options.forcedns1_addr)) {
+      log_err(0, "Invalid DNS IP address: %s!", args_info.forcedns1_arg);
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
   }
-
+  _options.forcedns2_port = args_info.forcedns2port_arg;
+  if (args_info.forcedns2port_arg && !args_info.forcedns2_arg) {
+    _options.forcedns2_addr.s_addr = _options.uamlisten.s_addr;
+  } else if (args_info.forcedns2_arg) {
+    if (!inet_aton(args_info.forcedns2_arg, &_options.forcedns2_addr)) {
+      log_err(0, "Invalid DNS IP address: %s!", args_info.forcedns2_arg);
+      if (!args_info.forgiving_flag)
+	goto end_processing;
+    }
+  }
+#endif
+  
   _options.uamanydns = args_info.uamanydns_flag;
 #ifdef ENABLE_UAMANYIP
   _options.uamanyip = args_info.uamanyip_flag;
@@ -609,16 +727,52 @@ int main(int argc, char **argv) {
   _options.radiusoriginalurl = args_info.radiusoriginalurl_flag;
   _options.routeonetone = args_info.routeonetone_flag;
 
-  /* pass-throughs */
-  memset(_options.pass_throughs, 0, sizeof(_options.pass_throughs));
-  _options.num_pass_throughs = 0;
+#ifdef HAVE_PATRICIA
+  _options.patricia = args_info.patricia_flag;
+#endif
+
+#ifdef ENABLE_GARDENACCOUNTING
+  _options.nousergardendata = args_info.nousergardendata_flag;
+  _options.uamgardendata = args_info.uamgardendata_flag;
+  _options.uamotherdata = args_info.uamotherdata_flag;
+#endif
 
   for (numargs = 0; numargs < args_info.uamallowed_given; ++numargs) {
     pass_throughs_from_string(_options.pass_throughs,
 			      MAX_PASS_THROUGHS,
-			      &_options.num_pass_throughs, 0, 
-			      args_info.uamallowed_arg[numargs]);
+			      &_options.num_pass_throughs,  
+			      args_info.uamallowed_arg[numargs], 0, 0
+#ifdef HAVE_PATRICIA
+			      , 0
+#endif
+      );
   }
+#ifdef ENABLE_LAYER3
+  for (numargs = 0; numargs < args_info.ipsrcallowed_given; ++numargs) {
+    pass_throughs_from_string(_options.ipsrc_pass_throughs,
+			      MAX_IPSRC_PASS_THROUGHS,
+			      &_options.ipsrc_num_pass_throughs,  
+			      args_info.ipsrcallowed_arg[numargs], 0, 0
+#ifdef HAVE_PATRICIA
+			      , 0
+#endif
+      );
+  }
+#endif
+
+  _options.uamauthedallowed = args_info.uamauthedallowed_flag;
+#ifdef ENABLE_AUTHEDALLOWED
+  for (numargs = 0; numargs < args_info.authedallowed_given; ++numargs) {
+    pass_throughs_from_string(_options.authed_pass_throughs,
+			      MAX_PASS_THROUGHS,
+			      &_options.num_authed_pass_throughs,  
+			      args_info.authedallowed_arg[numargs], 0, 0
+#ifdef HAVE_PATRICIA
+			      , 0
+#endif
+      );
+  }
+#endif
 
 #ifdef ENABLE_DHCPOPT
   _options.dhcp_options_len = 0;
@@ -692,8 +846,8 @@ int main(int argc, char **argv) {
   for (numargs = 0; numargs < args_info.uamregex_given; ++numargs) {
     regex_pass_throughs_from_string(_options.regex_pass_throughs,
 				    MAX_REGEX_PASS_THROUGHS,
-				    &_options.regex_num_pass_throughs, 0, 
-				    args_info.uamregex_arg[numargs]);
+				    &_options.regex_num_pass_throughs,  
+				    args_info.uamregex_arg[numargs], 0);
   }
 #endif
 
@@ -704,8 +858,18 @@ int main(int argc, char **argv) {
   }
 
   if (args_info.uamdomain_given) {
-    for (numargs = 0; numargs < args_info.uamdomain_given && numargs < MAX_UAM_DOMAINS; ++numargs) 
-      _options.uamdomains[numargs] = STRDUP(args_info.uamdomain_arg[numargs]);
+    for (numargs = 0, i=0; 
+	 numargs < args_info.uamdomain_given && i < MAX_UAM_DOMAINS; 
+	 ++numargs) {
+      char *tb = args_info.uamdomain_arg[numargs];
+      char *tok, *str, *ptr=0;
+      for (str = tb ; i < MAX_UAM_DOMAINS; str = NULL) {
+	tok = strtok_r(str, ",", &ptr);
+	if (!tok) break;
+	log_dbg("uamdomain %s", tok);
+	_options.uamdomains[i++] = STRDUP(tok);
+      }
+    }
   }
 
   _options.allowdyn = 1;
@@ -728,7 +892,8 @@ int main(int argc, char **argv) {
       _options.dynip = STRDUP(args_info.dynip_arg);
       if (option_aton(&addr, &mask, _options.dynip, 0)) {
 	log_err(0, "Failed to parse dynamic IP address pool!");
-	goto end_processing;
+	if (!args_info.forgiving_flag)
+	  goto end_processing;
       }
     }
   }
@@ -770,10 +935,11 @@ int main(int argc, char **argv) {
     if (!inet_aton(args_info.dns1_arg, &_options.dns1)) {
       log_err(0,"Invalid primary DNS address: %s!", 
 	      args_info.dns1_arg);
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
   }
-  else if (_res.nscount >= 1) {
+  else if (!args_info.nosystemdns_flag &&_res.nscount >= 1) {
     _options.dns1 = _res.nsaddr_list[0].sin_addr;
   }
   else {
@@ -784,10 +950,11 @@ int main(int argc, char **argv) {
     if (!inet_aton(args_info.dns2_arg, &_options.dns2)) {
       log_err(0,"Invalid secondary DNS address: %s!", 
 	      args_info.dns1_arg);
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
   }
-  else if (_res.nscount >= 2) {
+  else if (!args_info.nosystemdns_flag && _res.nscount >= 2) {
     _options.dns2 = _res.nsaddr_list[1].sin_addr;
   }
   else {
@@ -801,7 +968,8 @@ int main(int argc, char **argv) {
     if (!(host = gethostbyname(args_info.radiuslisten_arg))) {
       log_err(0, "Invalid listening address: %s! [%s]", 
 	      args_info.radiuslisten_arg, strerror(errno));
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
     else {
       memcpy(&_options.radiuslisten.s_addr, host->h_addr, host->h_length);
@@ -853,13 +1021,12 @@ int main(int argc, char **argv) {
     }
   }
 
-  /* If no option is specified terminate                          */
-  /* Do hostname lookup to translate hostname to IP address       */
   if (args_info.radiusserver1_arg) {
     if (!(host = gethostbyname(args_info.radiusserver1_arg))) {
       log_err(0, "Invalid radiusserver1 address: %s! [%s]", 
 	      args_info.radiusserver1_arg, strerror(errno));
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
     else {
       memcpy(&_options.radiusserver1.s_addr, host->h_addr, host->h_length);
@@ -867,17 +1034,16 @@ int main(int argc, char **argv) {
   }
   else {
     log_err(0,"No radiusserver1 address given!");
-    goto end_processing;
+    if (!args_info.forgiving_flag)
+      goto end_processing;
   }
 
-  /* radiusserver2 */
-  /* If no option is specified terminate                          */
-  /* Do hostname lookup to translate hostname to IP address       */
   if (args_info.radiusserver2_arg) {
     if (!(host = gethostbyname(args_info.radiusserver2_arg))) {
       log_err(0, "Invalid radiusserver2 address: %s! [%s]", 
 	      args_info.radiusserver2_arg, strerror(errno));
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
     else {
       memcpy(&_options.radiusserver2.s_addr, host->h_addr, host->h_length);
@@ -894,7 +1060,8 @@ int main(int argc, char **argv) {
     if (!(host = gethostbyname(args_info.proxylisten_arg))) {
       log_err(0, "Invalid listening address: %s! [%s]", 
 	      args_info.proxylisten_arg, strerror(errno));
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
     else {
       memcpy(&_options.proxylisten.s_addr, host->h_addr, host->h_length);
@@ -913,7 +1080,8 @@ int main(int argc, char **argv) {
     if(option_aton(&_options.proxyaddr, &_options.proxymask, 
 		   args_info.proxyclient_arg, 0)) {
       log_err(0,"Invalid proxy client address: %s!", args_info.proxyclient_arg);
-      goto end_processing;
+      if (!args_info.forgiving_flag)
+	goto end_processing;
     }
   }
   else {
@@ -995,6 +1163,10 @@ int main(int argc, char **argv) {
   _options.unixipc = STRDUP(args_info.unixipc_arg);
 #endif
 
+#ifdef ENABLE_WPAD
+  _options.wpadpacfile = STRDUP(args_info.wpadpacfile_arg);
+#endif
+
 #ifdef HAVE_NETFILTER_COOVA
   _options.kname = STRDUP(args_info.kname_arg);
 #endif
@@ -1027,12 +1199,66 @@ int main(int argc, char **argv) {
     log_err(0, "option moddir given when no support built-in");
 #endif
   
+#ifdef ENABLE_RADPROXY
   if (!args_info.proxysecret_arg) {
     _options.proxysecret = STRDUP(args_info.radiussecret_arg);
   }
   else {
     _options.proxysecret = STRDUP(args_info.proxysecret_arg);
   }
+#endif
+
+#ifdef ENABLE_REDIRINJECT
+  _options.inject = STRDUP(args_info.inject_arg);
+  _options.inject_ext = STRDUP(args_info.injectext_arg);
+  _options.inject_wispr = args_info.injectwispr_flag;
+#endif
+
+#ifdef ENABLE_EXTADMVSA
+  if (args_info.extadmvsa_given) {
+    for (numargs = 0; numargs < args_info.extadmvsa_given 
+	   && numargs < EXTADMVSA_ATTR_CNT; ++numargs)  {
+      int len = strlen(args_info.extadmvsa_arg[numargs]);
+      if (len > 0 && len < 256) {
+	unsigned int i[2];
+	char s[256];
+	
+	if (sscanf(args_info.extadmvsa_arg[numargs], 
+		   "%u,%u:%s", &i[0], &i[1], s) == 3) {
+	  char *idx = strchr(s, ':');
+	  _options.extadmvsa[numargs].attr_vsa = i[0];
+	  _options.extadmvsa[numargs].attr = i[1];
+	  if (idx) *idx = 0;
+	  safe_strncpy(_options.extadmvsa[numargs].script, 
+		       s, sizeof(_options.extadmvsa[numargs].script)-1);
+	  if (idx) {
+	    safe_strncpy(_options.extadmvsa[numargs].data, 
+			 idx + 1, sizeof(_options.extadmvsa[numargs].data)-1);
+	  }
+	} else if (sscanf(args_info.extadmvsa_arg[numargs], 
+			  "%u:%s", &i[0], s) == 2) {
+	  char *idx = strchr(s, ':');
+	  _options.extadmvsa[numargs].attr = i[0];
+	  if (idx) *idx = 0;
+	  safe_strncpy(_options.extadmvsa[numargs].script, 
+		       s, sizeof(_options.extadmvsa[numargs].script)-1);
+	  if (idx) {
+	    safe_strncpy(_options.extadmvsa[numargs].data, 
+			 idx + 1, sizeof(_options.extadmvsa[numargs].data)-1);
+	  }
+	} else {
+	  log_err(0, "invalid input %s", args_info.extadmvsa_arg[numargs]);
+	}
+      }
+
+      log_dbg("Extended admin-user attr (%d/%d) data=%s script=%s", 
+	      (int)_options.extadmvsa[numargs].attr_vsa, 
+	      (int)_options.extadmvsa[numargs].attr,
+	      _options.extadmvsa[numargs].data,
+	      _options.extadmvsa[numargs].script);
+    }
+  }
+#endif
 
   _options.peerkey = STRDUP(args_info.peerkey_arg);
   _options.routeif = STRDUP(args_info.routeif_arg);
@@ -1061,7 +1287,12 @@ int main(int argc, char **argv) {
   _options.radiuslocationname = STRDUP(args_info.radiuslocationname_arg);
   _options.locationname = STRDUP(args_info.locationname_arg);
   _options.radiussecret = STRDUP(args_info.radiussecret_arg);
+#ifdef ENABLE_LARGELIMITS
+  /*_options.radiusacctsecret = STRDUP(args_info.radiusacctsecret_arg);
+    _options.radiusadmsecret = STRDUP(args_info.radiusadmsecret_arg);*/
+#endif
   _options.cmdsocket = STRDUP(args_info.cmdsocket_arg);
+  _options.cmdsocketport = args_info.cmdsocketport_arg;
   _options.domain = STRDUP(args_info.domain_arg);
   _options.ipup = STRDUP(args_info.ipup_arg);
   _options.ipdown = STRDUP(args_info.ipdown_arg);
@@ -1076,8 +1307,16 @@ int main(int argc, char **argv) {
   _options.uamhostname = STRDUP(args_info.uamhostname_arg);
   _options.binconfig = STRDUP(args_info.bin_arg);
   _options.ethers = STRDUP(args_info.ethers_arg);
-#ifdef ENABLE_REDIRINJECT
-  _options.inject = STRDUP(args_info.inject_arg);
+#ifdef ENABLE_IEEE8021Q
+  _options.ieee8021q = args_info.ieee8021q_flag;
+  _options.ieee8021q_only = args_info.only8021q_flag;
+  _options.vlanupdate = STRDUP(args_info.vlanupdate_arg);
+#endif
+#ifdef ENABLE_PROXYVSA
+  _options.locationupdate = STRDUP(args_info.locationupdate_arg);
+#endif
+#ifdef EX_OPT_MAIN
+#include EX_OPT_MAIN
 #endif
 
   ret = 0;
