@@ -1,9 +1,10 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
- * Copyright (C) 2009-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2009-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -24,7 +25,6 @@
 
 #ifdef USING_CURL
 #include <curl/curl.h>
-#include <curl/types.h>
 #include <curl/easy.h>
 #endif
 
@@ -47,6 +47,7 @@ typedef struct _proxy_request {
 
   char reserved:4;
   char authorized:1;
+  char challenge:1;
   char inuse:1;
   char past_headers:1;
   char nline:1;
@@ -55,6 +56,7 @@ typedef struct _proxy_request {
   bstring data;
   bstring post;
   bstring wbuf;
+  bstring dbuf;
 
   struct radius_packet_t radius_req;
   struct radius_packet_t radius_res;
@@ -84,6 +86,8 @@ static CURLM * curl_multi;
 static int still_running = 0;
 #endif
 
+static char nas_hwaddr[PKT_ETH_ALEN];
+
 static void print_requests() {
   proxy_request * req = 0;
   int i;
@@ -162,6 +166,26 @@ static int radius_reply(struct radius_t *this,
   return 0;
 }
 
+static void bhex(bstring src, bstring dst) {
+  int i;
+  char b[4];
+  for (i=0; i < src->slen; i++) {
+    snprintf(b, sizeof(b), "%.2X", src->data[i]);
+    bcatcstr(dst, b);
+  }
+}
+
+static void bunhex(bstring src, bstring dst) {
+  int i;
+  unsigned int ir;
+  uint8_t r;
+  for (i=0; i < src->slen; i+=2) {
+    sscanf((char *) &src->data[i], "%2x", &ir);
+    r = (uint8_t) ir;
+    bcatblk(dst, &r, 1);
+  }
+}
+
 static void close_request(proxy_request *req) {
 
   log_dbg("%s", __FUNCTION__);
@@ -170,13 +194,16 @@ static void close_request(proxy_request *req) {
   if (req->data) bdestroy(req->data);
   if (req->post) bdestroy(req->post);
   if (req->wbuf) bdestroy(req->wbuf);
+  if (req->dbuf) bdestroy(req->dbuf);
 
   req->url =
     req->data = 
     req->post = 
+    req->dbuf =
     req->wbuf = 0;
   
   req->authorized = 0;
+  req->challenge = 0;
 
   req->inuse = 0;
   if (requests_free) {
@@ -216,6 +243,7 @@ static int http_aaa_finish(proxy_request *req) {
     log_dbg("Received: %s\n",req->data->data);
 #endif
     req->authorized = !memcmp(req->data->data, "Auth: 1", 7);
+    req->challenge = !memcmp(req->data->data, "Auth: 2", 7);
   }
 
   /* initialize response packet */
@@ -229,10 +257,12 @@ static int http_aaa_finish(proxy_request *req) {
     
   case RADIUS_CODE_ACCESS_REQUEST:
 #if(_debug_)
-    log_dbg("Access-%s", req->authorized ? "Accept" : "Reject");
+    log_dbg("Access-%s", req->authorized ? "Accept" : 
+	    req->challenge ? "Challenge" : "Reject");
 #endif
     radius_default_pack(radius, &req->radius_res, 
 			req->authorized ? RADIUS_CODE_ACCESS_ACCEPT :
+			req->challenge ? RADIUS_CODE_ACCESS_CHALLENGE :
 			RADIUS_CODE_ACCESS_REJECT);
     break;
 
@@ -262,6 +292,7 @@ static int http_aaa_finish(proxy_request *req) {
 	  { "Reply-Message:", RADIUS_ATTR_REPLY_MESSAGE, 0, 0, 1 },
 	  { "Session-Timeout:", RADIUS_ATTR_SESSION_TIMEOUT, 0, 0, 0 },
 	  { "Acct-Interim-Interval:", RADIUS_ATTR_ACCT_INTERIM_INTERVAL, 0, 0, 0 },
+	  { "EAP-Message:", RADIUS_ATTR_EAP_MESSAGE, 0, 0, 2 },
 	  { "ChilliSpot-Version:", 
 	    RADIUS_ATTR_VENDOR_SPECIFIC, RADIUS_VENDOR_CHILLISPOT, 
 	    RADIUS_ATTR_CHILLISPOT_VERSION, 1 },
@@ -323,6 +354,44 @@ static int http_aaa_finish(proxy_request *req) {
 	      {
 		radius_addattr(radius, &req->radius_res, attrs[i].a, attrs[i].v, attrs[i].va, 0, 
 			       (uint8_t *)ptr+strlen(attrs[i].n), strlen(ptr)-strlen(attrs[i].n));
+#if(_debug_)
+		log_dbg("Setting %s = %s", attrs[i].n, ptr+strlen(attrs[i].n));
+#endif
+	      }
+	      break;
+	    case 2: /*binary*/
+	      {
+		bstring tmp = bfromcstr("");
+		bstring tmp2 = bfromcstr("");
+
+		uint8_t *resp;
+		size_t offset = 0;
+		size_t resplen;
+		size_t eaplen;
+
+		bassignblk(tmp,
+			   (uint8_t *)ptr+strlen(attrs[i].n), 
+			   strlen(ptr)-strlen(attrs[i].n));
+
+		bunhex(tmp, tmp2);
+
+		resp = tmp2->data;
+		resplen = tmp2->slen;
+
+		while (offset < resplen) {
+		  
+		  if ((resplen - offset) > RADIUS_ATTR_VLEN)
+		    eaplen = RADIUS_ATTR_VLEN;
+		  else
+		    eaplen = resplen - offset;
+		  
+		  radius_addattr(radius, &req->radius_res, attrs[i].a, 
+				 attrs[i].v, attrs[i].va, 0, 
+				 resp + offset,
+				 eaplen);
+		  offset += eaplen;
+		} 
+		
 #if(_debug_)
 		log_dbg("Setting %s = %s", attrs[i].n, ptr+strlen(attrs[i].n));
 #endif
@@ -530,6 +599,9 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
   
   if (!req->wbuf) 
     req->wbuf = bfromcstr("");
+
+  if (!req->dbuf) 
+    req->dbuf = bfromcstr("");
   
   bassignformat(req->wbuf, 
 		"GET %s HTTP/1.1\r\n"
@@ -539,7 +611,7 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
 		"\r\n",
 		req->url->data + uripos, hostname, port);
   
-  if (conn_setup(&req->conn, hostname, port, req->wbuf))
+  if (conn_setup(&req->conn, hostname, port, req->wbuf, req->dbuf))
     return -1;
   
   conn_set_readhandler(&req->conn, http_conn_read, req);
@@ -601,23 +673,11 @@ static void http_aaa_register(int argc, char **argv, int i) {
   if (_options.nasmac) {
     bcatcstr(req.url, _options.nasmac);
   } else {
-    char nas_hwaddr[PKT_ETH_ALEN];
-    struct ifreq ifr;
     char mac[32];
-
-    int fd = socket(AF_INET, SOCK_DGRAM, 0);
-
-    safe_strncpy(ifr.ifr_name, _options.dhcpif, sizeof(ifr.ifr_name));
-
-    if (ioctl(fd, SIOCGIFHWADDR, (caddr_t)&ifr) == 0) {
-      memcpy(nas_hwaddr, ifr.ifr_hwaddr.sa_data, PKT_ETH_ALEN);
-      safe_snprintf(mac, sizeof(mac), "%.2X-%.2X-%.2X-%.2X-%.2X-%.2X", 
-		    nas_hwaddr[0],nas_hwaddr[1],nas_hwaddr[2],
-		    nas_hwaddr[3],nas_hwaddr[4],nas_hwaddr[5]);
-      bcatcstr(req.url, mac);
-    }
-      
-    close(fd);
+    safe_snprintf(mac, sizeof(mac), "%.2X-%.2X-%.2X-%.2X-%.2X-%.2X", 
+		  nas_hwaddr[0],nas_hwaddr[1],nas_hwaddr[2],
+		  nas_hwaddr[3],nas_hwaddr[4],nas_hwaddr[5]);
+    bcatcstr(req.url, mac);
   }
 
   bcatcstr(req.url, "&nasid=");
@@ -681,6 +741,8 @@ static void http_aaa_register(int argc, char **argv, int i) {
 #else
   if (req.wbuf)
     bdestroy(req.wbuf);
+  if (req.dbuf)
+    bdestroy(req.dbuf);
 #endif
   exit(0);
 }
@@ -864,8 +926,27 @@ static void process_radius(struct radius_t *radius, struct radius_packet_t *pack
       bconcat(req->url, tmp);
     }
 
+    if (!radius_getattr(pack, &attr, RADIUS_ATTR_EAP_MESSAGE, 0,0,0)) {
+      int instance = 1;
+      bcatcstr(req->url, "&eap=");
+      bassignblk(tmp, attr->v.t, (size_t)attr->l-2);
+      bassigncstr(tmp2, "");
+
+      do {
+	attr=NULL;
+	if (!radius_getattr(pack, &attr, RADIUS_ATTR_EAP_MESSAGE, 0, 0, 
+			    instance++) && attr) {
+	  bcatblk(tmp, attr->v.t, (size_t)attr->l-2);
+	}
+      } while (attr);
+
+      bhex(tmp, tmp2);
+      bconcat(req->url, tmp2);
+    }
+    
     if (!radius_getattr(pack, &attr, RADIUS_ATTR_VENDOR_SPECIFIC,
-		   RADIUS_VENDOR_CHILLISPOT, RADIUS_ATTR_CHILLISPOT_VLAN_ID, 0)) {
+			RADIUS_VENDOR_CHILLISPOT, 
+			RADIUS_ATTR_CHILLISPOT_VLAN_ID, 0)) {
       uint32_t val = ntohl(attr->v.i);
       bassignformat(tmp, "&vlan=%ld", (long) val);
       bconcat(req->url, tmp);
@@ -1025,6 +1106,24 @@ int main(int argc, char **argv) {
 
   process_options(argc, argv, 1);
 
+#if defined (__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
+  net_getmac(_options.dhcpif, nas_hwaddr);
+#else
+  {
+    struct ifreq ifr;
+    
+    int fd = socket(AF_INET, SOCK_DGRAM, 0);
+    
+    safe_strncpy(ifr.ifr_name, _options.dhcpif, sizeof(ifr.ifr_name));
+    
+    if (ioctl(fd, SIOCGIFHWADDR, (caddr_t)&ifr) == 0) {
+      memcpy(nas_hwaddr, ifr.ifr_hwaddr.sa_data, PKT_ETH_ALEN);
+    }
+    
+    close(fd);
+  }
+#endif
+
 #ifdef USING_CURL
   curl_global_init(CURL_GLOBAL_ALL);
   curl_multi = curl_multi_init();
@@ -1033,22 +1132,26 @@ int main(int argc, char **argv) {
   radiuslisten.s_addr = htonl(INADDR_ANY);
 
   if (radius_new(&radius_auth, &radiuslisten, 
-		 _options.radiusauthport ? _options.radiusauthport : RADIUS_AUTHPORT, 
-		 0, 0)) {
+		 _options.radiusauthport ? 
+		 _options.radiusauthport : RADIUS_AUTHPORT, 
+		 0, 0) || 
+      radius_init_q(radius_auth, 0)) {
     log_err(0, "Failed to create radius");
     return -1;
   }
 
   if (radius_new(&radius_acct, &radiuslisten, 
-		 _options.radiusacctport ? _options.radiusacctport : RADIUS_ACCTPORT, 
-		 0, 0)) {
+		 _options.radiusacctport ? 
+		 _options.radiusacctport : RADIUS_ACCTPORT, 
+		 0, 0) || 
+      radius_init_q(radius_acct, 0)) {
     log_err(0, "Failed to create radius");
     return -1;
   }
-
+  
   radius_set(radius_auth, 0, 0);
   radius_set(radius_acct, 0, 0);
-
+  
   if (_options.gid && setgid(_options.gid)) {
     log_err(errno, "setgid(%d) failed while running with gid = %d\n", 
 	    _options.gid, getgid());
