@@ -1,9 +1,10 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -39,8 +40,6 @@ static cmd_info commands[] = {
   { CMDSOCK_LIST_IPPOOL,   "listippool",    NULL },
   { CMDSOCK_LIST_RADQUEUE, "listradqueue",  NULL },
   { CMDSOCK_LIST_GARDEN,   "listgarden",    NULL },
-  { CMDSOCK_ENTRY_FOR_IP,  "listip",        NULL },
-  { CMDSOCK_ENTRY_FOR_MAC, "listmac",       NULL },
   { CMDSOCK_RELOAD,        "reload",        NULL },
   { CMDSOCK_DHCP_LIST,     "dhcp-list",     NULL },
   { CMDSOCK_DHCP_RELEASE,  "dhcp-release",  NULL },
@@ -67,6 +66,10 @@ static cmd_info commands[] = {
   { CMDSOCK_REM_GARDEN,  "remgarden", NULL },
 #ifdef ENABLE_INSPECT
   { CMDSOCK_INSPECT,  "inspect", NULL },
+#endif
+#if defined(ENABLE_LOCATION) && defined(HAVE_AVL)
+  { CMDSOCK_LISTLOC,       "listloc",       NULL },
+  { CMDSOCK_LISTLOCSUM,    "listlocsum",    NULL },
 #endif
   { 0, NULL, NULL }
 };
@@ -183,6 +186,13 @@ static struct cmd_arguments args[] = {
     sizeof(request.d.sess.params.routeidx),
     &request.d.sess.params.routeidx,
     "Route interface index",  0, 0 },
+#endif
+#ifdef ENABLE_LOCATION
+  { "location",
+    CMDSOCK_FIELD_STRING, 
+    sizeof(request.d.sess.location),
+    request.d.sess.location,
+    "Location of session to perform action on", 0, 0 },
 #endif
   { "noacct",
     CMDSOCK_FIELD_NONE, 0, 0,
@@ -233,8 +243,9 @@ static int parse_mac(uint8_t *mac, char *string) {
 static int usage(char *program) {
   int i;
 
-  fprintf(stderr, "Usage: %s [ -s <socket> ] <command> [<arguments>]\n", program);
+  fprintf(stderr, "Usage: %s [ -s <socket> ] [ -P <port> ] <command> [<arguments>]\n", program);
   fprintf(stderr, "  socket = full path to UNIX domain socket (e.g. /var/run/chilli.sock)\n");
+  fprintf(stderr, "  port = TCP socket port to connect to. Default is 42424\n");
 
   fprintf(stderr, "  Available Commands:\n    ");
   for (i=0; commands[i].command; i++) {
@@ -359,6 +370,32 @@ static int process_args(int argc, char *argv[], int argidx) {
   return argidx;
 }
 
+static int chilli_communicate(int s,
+			       struct cmdsock_request *buffer,
+			       size_t blen) {
+  char line[1024];
+  int len;
+
+  if (safe_write(s, buffer, blen) != blen) {
+    perror("write");
+    return -1;
+  }
+  
+  while((len = safe_read(s, line, sizeof(line)-1)) > 0) {
+    if (write(1, line, len) != len) {
+      perror("write");
+      return -1;
+    }
+  }
+  
+  if (len < 0) {
+    perror("read");
+    return -1;
+  }
+
+  return 0;
+}
+
 int main(int argc, char **argv) {
   /*
    *   chilli_query [ -s <unix-socket> ] <command> [<argument>]
@@ -372,10 +409,12 @@ int main(int argc, char **argv) {
 #else
   char *cmdsock = DEFSTATEDIR"/chilli.sock";
 #endif
-  int s, len;
+  int s;
+  int cmdsockport = 0;
   struct sockaddr_un remote;
-  char line[1024], *cmd;
+  char *cmd;
   int argidx = 1;
+  int len;
 
   struct itimerval itval;
 
@@ -425,6 +464,10 @@ int main(int argc, char **argv) {
     } else if (!strcmp(argv[argidx], "-json")) {
       request.options |= CMDSOCK_OPT_JSON;
       argidx++;
+    } else if (!strcmp(argv[argidx], "-P")) {
+      argidx++;
+      if (argidx >= argc) return usage(argv[0]);
+      cmdsockport = atoi(argv[argidx++]);
     }
   }
   
@@ -440,6 +483,11 @@ int main(int argc, char **argv) {
 #ifdef ENABLE_INSPECT
       case CMDSOCK_INSPECT:
 #endif
+#if defined(ENABLE_LOCATION) && defined(HAVE_AVL)
+      case CMDSOCK_LISTLOC:
+      case CMDSOCK_LISTLOCSUM:
+#endif
+      case CMDSOCK_LIST:
       case CMDSOCK_LOGIN:
       case CMDSOCK_LOGOUT:
       case CMDSOCK_UPDATE:
@@ -452,7 +500,6 @@ int main(int argc, char **argv) {
 	/* else, drop through */
       case CMDSOCK_DHCP_DROP:
       case CMDSOCK_DHCP_RELEASE:
-      case CMDSOCK_ENTRY_FOR_MAC:
 	{
 	  if (argc < argidx+1) {
 	    fprintf(stderr, "%s requires a MAC address argument\n", cmd);
@@ -465,24 +512,6 @@ int main(int argc, char **argv) {
 	  /* do another switch to pick up param configs for authorize */
 	}
 	break;
-      case CMDSOCK_ENTRY_FOR_IP:
-	{
-	  struct in_addr ip;
-
-	  /* Test for a valid ip argument. */
-  	  if (argc < argidx+1) {
-  	    fprintf(stderr, "%s requires an IP address argument\n", cmd);
-  	    return usage(argv[0]);
-  	  }
-	  
-	  if (!inet_aton(argv[argidx], &ip)) {
-	    fprintf(stderr, "Invalid IP Address: %s\n", argv[argidx]);
-	    return usage(argv[0]);
-	  }
-	 
-	  request.ip.s_addr = ip.s_addr;
-	}
-	break;
 #ifdef ENABLE_MULTIROUTE
       case CMDSOCK_ROUTE:
       case CMDSOCK_ROUTE_GW:
@@ -563,8 +592,40 @@ int main(int argc, char **argv) {
 		       (struct sockaddr *)&s, 
 		       sizeof(struct sockaddr_in));
 
-  } else {
+    return 0;
+  }
 #endif
+
+  if (cmdsockport) {
+    struct sockaddr_in remote_port;
+    struct in_addr addr;
+
+    if ((s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
+      perror("socket");
+      exit(1);
+    }
+
+    inet_aton("127.0.0.1", &addr);
+    memset(&remote_port, 0, sizeof(struct sockaddr_in));
+    remote_port.sin_family = AF_INET;
+    remote_port.sin_port = htons(cmdsockport);
+    remote_port.sin_addr.s_addr = addr.s_addr;
+
+    if (connect(s, (struct sockaddr *)&remote_port, sizeof(remote_port)) == -1) {
+      perror("connect");
+      exit(1);
+    }
+
+    if (chilli_communicate(s, &request, sizeof(request)) != 0) {
+      perror("write");
+      exit(1);
+    }
+
+    shutdown(s, 2);
+    close(s);
+
+    return 0;
+  }
   
 #ifdef HAVE_GLOB
   globbuf.gl_offs = 0;
@@ -607,24 +668,12 @@ int main(int argc, char **argv) {
     perror("connect");
     exit(1);
   }
-  
-  if (safe_write(s, &request, sizeof(request)) != sizeof(request)) {
+
+  if (chilli_communicate(s, &request, sizeof(request)) != 0) {
     perror("write");
     exit(1);
   }
 
-  while((len = safe_read(s, line, sizeof(line)-1)) > 0) {
-    if (write(1, line, len) != len) {
-      perror("write");
-      exit(1);
-    }
-  }
-
-  if (len < 0) {
-    perror("read");
-    exit(1);
-  }
-
   shutdown(s, 2);
   close(s);
 
@@ -632,10 +681,6 @@ int main(int argc, char **argv) {
   }
   globfree(&globbuf);
 #endif
-
-#ifdef ENABLE_CLUSTER
-  }
-#endif
   
   return 0;
 }
