@@ -1,9 +1,10 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
- * Copyright (C) 2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -15,7 +16,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  */
-
 #define MAIN_FILE
 
 #include "chilli.h"
@@ -34,11 +34,33 @@ static redir_request * requests = 0;
 static redir_request * requests_free = 0;
 
 #ifdef ENABLE_REDIRINJECT
-static char *inject_script = "<script src='%s'></script>\r\n";
-static char * inject_fmt(char *b, size_t blen, char *url) {
+static bstring inject_fmt(redir_request *req, struct redir_conn_t *conn) {
+  char *url = req->inject_url;
+  if (req->ibuf->slen) return req->ibuf;
   if (!url || !*url) url = _options.inject;
-  safe_snprintf(b, blen, inject_script, url);
-  return b;
+  if (!url || !*url) return 0;
+
+  bassigncstr(req->ibuf, "");
+  if (conn) {
+    char hexchal[1+(2*REDIR_MD5LEN)];
+    extern void redir_wispr2_reply 
+      (struct redir_t *redir, struct redir_conn_t *conn, 
+       int res, long int timeleft, char* hexchal, 
+       char* reply, char* redirurl, bstring b);
+    redir_chartohex(conn->s_state.redir.uamchal, hexchal, REDIR_MD5LEN);
+    redir_wispr2_reply(req->parent, conn, 
+		       REDIR_NOTYET, 0, hexchal, 0, 0, req->ibuf);
+  }
+  
+  if (_options.inject_ext) {
+    bcatcstr(req->ibuf, "<script type=\"text/javascript\">");
+    bcatcstr(req->ibuf, _options.inject_ext);
+    bcatcstr(req->ibuf, "</script>");
+  }
+  bcatcstr(req->ibuf, "<script src='");
+  bcatcstr(req->ibuf, url);
+  bcatcstr(req->ibuf, "'></script>");
+  return req->ibuf;
 }
 #endif
 
@@ -90,19 +112,26 @@ static redir_request * get_request() {
     return 0;
   }
 
-  req->url  = string_init_reset(req->url);
-  req->data = string_init_reset(req->data);
-  req->post = string_init_reset(req->post);
+  /*req->url  = string_init_reset(req->url);*/
+  /*req->data = string_init_reset(req->data);*/
+  /*req->post = string_init_reset(req->post);*/
+  req->dbuf = string_init_reset(req->dbuf);
   req->wbuf = string_init_reset(req->wbuf);
   req->hbuf = string_init_reset(req->hbuf);
+  req->ibuf = string_init_reset(req->ibuf);
 
   /*
     log_dbg("url->len  %d",req->url->slen);
     log_dbg("data->len %d",req->data->slen);
     log_dbg("post->len %d",req->post->slen);
-    log_dbg("wbuf->len %d",req->wbuf->slen);
   */
+  log_dbg("dbuf->len %d",req->dbuf->slen);
+  log_dbg("wbuf->len %d",req->wbuf->slen);
+  log_dbg("hbuf->len %d",req->hbuf->slen);
+  log_dbg("ibuf->len %d",req->ibuf->slen);
   
+  req->read_closed = 0;
+  req->write_closed = 0;
   req->state = 0;
   req->next = req->prev = 0;
   req->html = req->proxy = req->headers = 0;
@@ -146,7 +175,7 @@ sock_redir_getstate(struct redir_t *redir,
   memcpy(&msg.mdata.baddress, baddress, sizeof(msg.mdata.baddress));
 
   if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
-    perror("socket");
+    log_err(errno, "socket()");
     return -1;
   }
 
@@ -197,160 +226,279 @@ static int redir_conn_finish(struct conn_t *conn, void *ctx) {
   if (req->socket_fd) {
 
 #ifdef ENABLE_REDIRINJECT
-    if (*req->inject_url && req->html && !req->chunked) {
+    /**
+    if (0 && *req->inject_url && req->html && !req->chunked) {
       char b[256];
-      char *inject = inject_fmt(b, sizeof(b), req->inject_url);
+      char *inject = inject_fmt(req);
       int w = net_write(req->socket_fd, inject, strlen(inject));
       log_dbg("injected %d bytes", w);
     }
+    */
 #endif
     
     if (req->state & REDIR_SOCKET_FD) {
       net_select_rmfd(&sctx, req->socket_fd);
     }
     close(req->socket_fd);
+    req->socket_fd=0;
   }
 
   close_request(req);
   return 0;
 }
 
+#define conn_write_remaining(conn) \
+  ((conn)->read_buf &&		   \
+   (conn)->read_buf->slen &&	   \
+   (conn)->read_pos <		   \
+   (conn)->read_buf->slen)
+
+/*
+ * Return values:
+ *    0 == nothing to write
+ *   -1 == error that closed socket
+ *    1 == normal write
+ */
+static int redir_cli_rewrite(redir_request *req, struct conn_t *conn) {
+  if (conn_write_remaining(conn)) {
+    int w = net_write(req->socket_fd,
+		      conn->read_buf->data +
+		      conn->read_pos,
+		      conn->read_buf->slen -
+		      conn->read_pos);
+    if (w < 0 && errno != EWOULDBLOCK && errno != EAGAIN) {
+      log_err(errno, "net_write(%d)",
+	      conn->read_buf->slen -
+	      conn->read_pos);
+      redir_conn_finish(conn, req);
+      return -1;
+    } else if (w > 0) {
+#if(_debug_)      
+      log_dbg("client (re)write: %d", w);
+#endif
+      conn->read_pos += w;
+      if (conn->read_pos == conn->read_buf->slen) {
+	log_dbg("emptying write queue");
+	conn->read_pos = 0;
+	bassigncstr(conn->read_buf, "");
+      }
+    }
+    return 1;
+  }
+  return 0;
+}
+
+static int redir_cli_write(redir_request *req, uint8_t *d, int l) {
+  int w = 0;
+
+  if (redir_cli_rewrite(req, &req->conn) == -1)
+    return -1;
+
+  if (!conn_write_remaining(&req->conn)) {
+    
+    w = net_write(req->socket_fd, d, l);
+
+#if(_debug_ )      
+    log_dbg("client write: %d", w);
+    /*log_dbg("write: [%s]", b);*/
+#endif
+  }
+  
+  if (w >= 0) {
+    if (w < l) {
+      bcatblk(req->dbuf, d + w, l - w);
+      log_warn(errno, "buffering %d - %d = %d (%d queued)", 
+	       l, w, l-w, req->dbuf->slen);
+    }
+  }
+  
+  return w;
+}
+
 static int redir_conn_read(struct conn_t *conn, void *ctx) {
   redir_request *req = (redir_request *)ctx;
-  char bb[PKT_MAX_LEN];
-  char *b = bb;
+  uint8_t bb[PKT_MAX_LEN];
+  int s, r;
 
-  int r = safe_read(conn->sock, b, sizeof(b)-1);
-
-#if(_debug_ > 1)
-  log_dbg("conn_read: %d", r);
+  if ((s=redir_cli_rewrite(req, conn)) != 0) 
+    return 0;
+  
+  r = safe_recv(conn->sock, bb, sizeof(bb)-1, 0);
+  
+#if(_debug_)
+  log_dbg("conn_read: %d clen=%d", r, req->clen);
 #endif
+  
+  if (r == 0) {
 
-  if (r <= 0) {
+    if (req->read_closed && redir_cli_rewrite(req, conn) == 0) {
+      log_dbg("done reading and writing");
+      redir_conn_finish(conn, req);
+      return -1;
+    }
 
+    req->read_closed = 1;
+    
+  } else if (r < 0 && 
+	     errno != EWOULDBLOCK && errno != EAGAIN) {
+    
+    log_dbg("ERRNO %d", errno);
     redir_conn_finish(conn, ctx);
-
+    
   } else if (r > 0) {
-
-    b[r]=0;
-    req->last_active = mainclock_tick();
-
 #ifdef ENABLE_REDIRINJECT
+    bstring inject = inject_fmt(req, 0);
+#endif
+    
+    bb[r]=0;
+    req->last_active = mainclock_tick();
+    
+#ifdef ENABLE_REDIRINJECT
+
     /**
      *
      */
-    if (*req->inject_url && !req->headers) {
+    if (inject && !req->headers) {
       char *newline = "\r\n\r\n";
       char *eoh;
-
-      bcatblk(req->hbuf, b, r);
-
+      
+      bcatblk(req->hbuf, bb, r);
+      
       if ((eoh = strstr((char *)req->hbuf->data, newline))) {
+	
+	int header_len = eoh - (char *)req->hbuf->data;
 	bstring newhdr = bfromcstr("");
-	char *hdr, *p;
 
-	hdr = (char *)req->hbuf->data;
-	
-	while (hdr && *hdr) {
-	  int l;
-	  int skip = 0;
-
-	  p = strstr(hdr, "\r\n");
-
-	  if (p == hdr) {
-	    break;
-	  } else if (p) {
-	    l = (p - hdr);
-	  } else {
-	    l = (eoh - hdr);
-	  }
-
-	  if (!strncasecmp(hdr, "content-length:", 15)) {
-	    char tmp[128];
-	    char c = hdr[l];
-	    int clen;
-
-	    char b[256];
-	    char *inject = inject_fmt(b, sizeof(b), req->inject_url);
-
-	    hdr[l] = 0;
-	    clen = req->clen = atoi(hdr+15);
-	    log_dbg("Detected Content Length %d", req->clen);
-	    clen += strlen(inject);
-	    safe_snprintf(tmp, sizeof(tmp), "Content-Length: %d\r\n", clen);
-	    bcatcstr(newhdr, tmp);
-	    hdr[l] = c;
-	    skip = 1;
-	  } else if (!strncasecmp(hdr, "accept-ranges:", 14)) {
-	    skip = 1;
-	  } else if (!strncasecmp(hdr, "content-type:", 13)) {
-	    if (strstr(hdr, "text/html")) {
-	      req->html = 1;
-	    }
-	  } else if (strcasestr(hdr, "content-encoding: gzip")) {
-	    req->gzip = 1;
-	  } else if (strcasestr(hdr, "transfer-encoding: chunked")) {
-	    req->chunked = 1;
-	  }
-
-	  log_dbg("Header [%d] %.*s%s", l, l, hdr, skip ? " [Skipped]" : "");
-
-	  if (!skip) {
-	    bcatblk(newhdr, hdr, l + 2);
-	  }
-
-	  hdr += l + 2;
-	  if (!p) break;
-	}
-
-	/* process headers */
-	/* Is HTML */
-	/* Check content-encoding chunked */
-	/* Adjust content-length */
-
-	net_write(req->socket_fd, newhdr->data, newhdr->slen);
-	net_write(req->socket_fd, newline, 2);
-	
-	if (req->html && req->chunked) {
-	  char tmp[56]; int w;
-
-	  char b[256];
-	  char *inject = inject_fmt(b, sizeof(b), req->inject_url);
+	if (strncmp((char *)req->hbuf->data, "HTTP/1.", 7) ||
+	    strncmp((char *)req->hbuf->data+8, " 2", 2)) {
 	  
-	  safe_snprintf(tmp, sizeof(tmp), "%x\r\n", strlen(inject));
-	  net_write(req->socket_fd, tmp, strlen(tmp));
-	  w = net_write(req->socket_fd, inject, strlen(inject));
-#if(_debug_ > 1)
-	  log_dbg("--->>> chunked write %d", w);
-#endif
-	  net_write(req->socket_fd, "\r\n", 2);
+	  log_dbg("Not HTTP/1.X 2XX reply");
+	  bcatblk(newhdr, req->hbuf->data, header_len + 4);
+	  
+	} else {
+	  
+	  char *hdr, *p;
+	  int clen = 0;
+	  
+	  hdr = (char *)req->hbuf->data;
+	  
+	  while (hdr && *hdr) {
+	    int l;
+	    
+	    p = strstr(hdr, "\r\n");
+	    
+	    if (p == hdr) {
+	      break;
+	    } else if (p) {
+	      l = (p - hdr);
+	    } else {
+	      l = (eoh - hdr);
+	    }
+	    
+	    if (!strncasecmp(hdr, "content-length:", 15)) {
+	      char c = hdr[l];
+	      hdr[l] = 0;
+	      clen = req->clen = atoi(hdr+15);
+	      log_dbg("Detected Content Length %d", req->clen);
+	      hdr[l] = c;
+	    } else if (!strncasecmp(hdr, "content-type:", 13)) {
+	      if (strstr(hdr, "text/html")) {
+		req->html = 1;
+	      }
+	    } else if (strcasestr(hdr, "content-encoding: gzip")) {
+	      req->gzip = 1;
+	    } else if (strcasestr(hdr, "transfer-encoding:") && 
+		       strstr(hdr,"chunked")) {
+	      req->chunked = 1;
+	    }
+	    
+	    hdr += l + 2;
+	    if (!p) break;
+	  }
+
+	  hdr = (char *)req->hbuf->data;
+	  
+	  while (hdr && *hdr) {
+	    int l;
+	    int skip = 0;
+	    
+	    p = strstr(hdr, "\r\n");
+	    
+	    if (p == hdr) {
+	      break;
+	    } else if (p) {
+	      l = (p - hdr);
+	    } else {
+	      l = (eoh - hdr);
+	    }
+	    
+	    if (req->html) {
+	      if (clen && !strncasecmp(hdr, "content-length:", 15)) {
+		char tmp[128];
+		if (inject) clen += inject->slen;
+		safe_snprintf(tmp, sizeof(tmp), "Content-Length: %d\r\n", clen);
+		bcatcstr(newhdr, tmp);
+		skip = 1;
+	      } else if (!strncasecmp(hdr, "connection:", 11)) {
+		skip = 1;
+	      } else if (!strncasecmp(hdr, "accept-ranges:", 14)) {
+		skip = 1;
+	      }
+	    }
+	    
+	    log_dbg("Resp Header [%d] %.*s%s", 
+		    l, l, hdr, skip ? " [Skipped]" : "");
+	    
+	    if (!skip) {
+	      bcatblk(newhdr, hdr, l + 2);
+	    }
+	    
+	    hdr += l + 2;
+	    if (!p) break;
+	  }
+
+	  bcatcstr(newhdr, "Connection: close\r\n");
+	
+	  /* process headers */
+	  /* Is HTML */
+	  /* Check content-encoding chunked */
+	  /* Adjust content-length */
+	  
+	  bcatblk(newhdr, newline, 2);
+	  
+	  if (req->html) {
+	    if (req->chunked) {
+	      char tmp[56];
+	      safe_snprintf(tmp, sizeof(tmp), "%x\r\n", 
+			    inject->slen);
+	      bcatcstr(newhdr, tmp);
+	    }
+	    bconcat(newhdr, inject);
+	    if (req->chunked) {
+	      bcatblk(newhdr, newline, 2);
+	    }
+	  }
 	}
-	
-	net_write(req->socket_fd, eoh + 4, req->hbuf->slen - 
-		  (hdr - (char *)req->hbuf->data) - 2);
-	
+
+	bcatblk(newhdr, eoh + 4, req->hbuf->slen - header_len - 4);
+	if (req->clen > 0) /* adjust clen */
+	  req->clen -= (req->hbuf->slen - header_len - 4);
+	redir_cli_write(req, newhdr->data, newhdr->slen);
 	req->headers = 1;
 	bdestroy(newhdr);
       }
-    }
-    else {
+    } else {
 #endif
 
-      int w = net_write(req->socket_fd, b, r);
+      redir_cli_write(req, bb, r);
+      if (req->clen > 0)
+	req->clen -= r;
 
-#if(_debug_ > 1)      
-      log_dbg("write: %d", w);
-      /*log_dbg("write: [%s]", b);*/
-#endif
-      
-      if (r != w) {
-	log_err(errno, "problem writing what we read");
-	redir_conn_finish(conn, ctx);
-      }
 #ifdef ENABLE_REDIRINJECT
     }
 #endif
   }
+  /*log_dbg("leaving redir_conn_read()");*/
   return 0;
 }
 
@@ -395,8 +543,9 @@ redir_handle_url(struct redir_t *redir,
 		 struct sockaddr_in *peer, 
 		 redir_request *req) {
   int port = 80;
-  int i;
-  int matches = 1;
+  int matches = 0;
+  int i = -1;
+  char *p = 0;
 
 #ifdef ENABLE_REDIRINJECT
   char hasInject = 0;
@@ -404,68 +553,70 @@ redir_handle_url(struct redir_t *redir,
     safe_strncpy((char *) req->inject_url,
 		 (char *) conn->s_params.url,
 		 REDIRINJECT_MAX);
-    hasInject = 1;
-  } else if  (_options.inject) { 
+    matches = hasInject = 1;
+  } else if (_options.inject && *_options.inject) { 
     safe_strncpy((char *) req->inject_url,
 		 (char *) _options.inject,
 		 REDIRINJECT_MAX);
-    hasInject = 1;
-  } else
+    matches = hasInject = 1;
+  } else {
 #endif
-    
-  for (i=0; i < MAX_REGEX_PASS_THROUGHS; i++) {
-    
-    if ( ! _options.regex_pass_throughs[i].inuse )
-      break;
-
-    /*
-    if ( ! _options.regex_pass_throughs[i].regex_host[0] &&
-	 ! _options.regex_pass_throughs[i].regex_path[0] &&
-	 ! _options.regex_pass_throughs[i].regex_qs[0] )
-      break;
-    */
-
+    for (i=0; i < MAX_REGEX_PASS_THROUGHS; i++) {
+      
+      if ( ! _options.regex_pass_throughs[i].inuse )
+	break;
+      
+      /*
+	if ( ! _options.regex_pass_throughs[i].regex_host[0] &&
+	! _options.regex_pass_throughs[i].regex_path[0] &&
+	! _options.regex_pass_throughs[i].regex_qs[0] )
+	break;
+      */
+      
 #if(_debug_)
-    log_dbg("REGEX host=[%s] path=[%s] qs=[%s]",
-	    _options.regex_pass_throughs[i].regex_host,
-	    _options.regex_pass_throughs[i].regex_path,
-	    _options.regex_pass_throughs[i].regex_qs);
-
-    log_dbg("Host %s", httpreq->host);
+      log_dbg("REGEX host=[%s] path=[%s] qs=[%s]",
+	      _options.regex_pass_throughs[i].regex_host,
+	      _options.regex_pass_throughs[i].regex_path,
+	      _options.regex_pass_throughs[i].regex_qs);
+      
+      log_dbg("Host %s", httpreq->host);
 #endif
-
-    if (_options.regex_pass_throughs[i].regex_host[0]) {
-      switch(check_regex(&_options.regex_pass_throughs[i].re_host, 
-			 _options.regex_pass_throughs[i].regex_host, 
-			 httpreq->host)) {
-      case -1: return -1;  
-      case 1: matches = _options.regex_pass_throughs[i].neg_host; break;
-      case 0: matches = !_options.regex_pass_throughs[i].neg_host; break;
+      
+      if (_options.regex_pass_throughs[i].regex_host[0]) {
+	switch(check_regex(&_options.regex_pass_throughs[i].re_host, 
+			   _options.regex_pass_throughs[i].regex_host, 
+			   httpreq->host)) {
+	case -1: return -1;  
+	case 1: matches = _options.regex_pass_throughs[i].neg_host; break;
+	case 0: matches = !_options.regex_pass_throughs[i].neg_host; break;
+	}
       }
-    }
-
-    if (matches && _options.regex_pass_throughs[i].regex_path[0]) {
-      switch(check_regex(&_options.regex_pass_throughs[i].re_path, 
-			 _options.regex_pass_throughs[i].regex_path, 
-			 httpreq->path)) {
-      case -1: return -1;  
-      case 1: matches = _options.regex_pass_throughs[i].neg_path; break;
-      case 0: matches = !_options.regex_pass_throughs[i].neg_path; break;
+      
+      if (matches && _options.regex_pass_throughs[i].regex_path[0]) {
+	switch(check_regex(&_options.regex_pass_throughs[i].re_path, 
+			   _options.regex_pass_throughs[i].regex_path, 
+			   httpreq->path)) {
+	case -1: return -1;  
+	case 1: matches = _options.regex_pass_throughs[i].neg_path; break;
+	case 0: matches = !_options.regex_pass_throughs[i].neg_path; break;
+	}
       }
-    }
-
-    if (matches && _options.regex_pass_throughs[i].regex_qs[0]) {
-      switch(check_regex(&_options.regex_pass_throughs[i].re_qs, 
-			 _options.regex_pass_throughs[i].regex_qs, 
-			 httpreq->qs)) {
-      case -1: return -1;  
-      case 1: matches = _options.regex_pass_throughs[i].neg_qs; break;
-      case 0: matches = !_options.regex_pass_throughs[i].neg_qs; break;
+      
+      if (matches && _options.regex_pass_throughs[i].regex_qs[0]) {
+	switch(check_regex(&_options.regex_pass_throughs[i].re_qs, 
+			   _options.regex_pass_throughs[i].regex_qs, 
+			   httpreq->qs)) {
+	case -1: return -1;  
+	case 1: matches = _options.regex_pass_throughs[i].neg_qs; break;
+	case 0: matches = !_options.regex_pass_throughs[i].neg_qs; break;
+	}
       }
+      
+      if (matches) break;
     }
-
-    if (matches) break;
+#ifdef ENABLE_REDIRINJECT
   }
+#endif
 
   if (matches) {
     log_dbg("Matched for Host %s", httpreq->host);
@@ -478,6 +629,9 @@ redir_handle_url(struct redir_t *redir,
     if (hasInject) {
       bstring newhdr = bfromcstr("");
       char *hdr = (char *)req->wbuf->data;
+
+      if (_options.inject_wispr)
+	(void) inject_fmt(req, conn);
       
       while (hdr && *hdr) {
 	char *p = strstr(hdr, "\r\n");
@@ -521,15 +675,21 @@ redir_handle_url(struct redir_t *redir,
     }
     /* XXX */
 #endif
-    
-    if (conn_setup(&req->conn, httpreq->host, port, req->wbuf)) {
+
+    if ((p = strchr(httpreq->host, ':'))) {
+      *p++ = 0;
+      port = atoi(p);
+    }
+
+    if (conn_setup(&req->conn, httpreq->host, port, 
+		   req->wbuf, req->dbuf)) {
       log_err(errno, "conn_setup()");
       return -1;
     }
     
     req->state |= REDIR_CONN_FD;
     net_select_addfd(&sctx, req->conn.sock, SELECT_READ);
-    
+
     return 0;
   }
   
@@ -584,7 +744,8 @@ int redir_accept2(struct redir_t *redir, int idx) {
       return 0; 
     }
     
-    safe_snprintf(buffer,sizeof(buffer),"%s",inet_ntoa(address.sin_addr));
+    safe_snprintf(buffer,sizeof(buffer),"%s",
+		  inet_ntoa(address.sin_addr));
     setenv("TCPREMOTEIP",buffer,1);
     setenv("REMOTE_ADDR",buffer,1);
     safe_snprintf(buffer,sizeof(buffer),"%d",ntohs(address.sin_port));
@@ -594,10 +755,14 @@ int redir_accept2(struct redir_t *redir, int idx) {
     char *binqqargs[2] = { _options.uamui, 0 } ;
     
     execv(*binqqargs, binqqargs);
+
+    exit(-1);
     
   } else {
 
     redir_request *req = get_request();
+    
+    req->parent = redir;
 
     log_dbg("redir_main() for %s", inet_ntoa(address.sin_addr));
 
@@ -614,7 +779,9 @@ int redir_accept2(struct redir_t *redir, int idx) {
     switch (redir_main(redir, new_socket, new_socket,
 		       &address, &baddress, idx, req)) {
     case 1:
-      log_dbg("redir queued %s", inet_ntoa(address.sin_addr));
+      log_dbg("redir queued %s socket_fd=%d conn.fd=%d", 
+	      inet_ntoa(address.sin_addr),
+	      req->socket_fd, req->conn.sock);
       req->state |= REDIR_SOCKET_FD;
       net_select_addfd(&sctx, req->socket_fd, SELECT_READ);
       return 1;
@@ -623,6 +790,7 @@ int redir_accept2(struct redir_t *redir, int idx) {
       redir_conn_finish(&req->conn, req);
       return 0;
     default:
+      log_dbg("redir completed %s", inet_ntoa(address.sin_addr));
       redir_conn_finish(&req->conn, req);
       return -1;
     }
@@ -740,8 +908,11 @@ int main(int argc, char **argv) {
 	  log_dbg("timeout connection %d", idx);
 	  redir_conn_finish(&requests[idx].conn, &requests[idx]);
 	} else {
+	  int evt = SELECT_READ;
 	  timeout = 0;
-	  net_select_fd(&sctx, fd, SELECT_READ);
+	  if (conn_write_remaining(&requests[idx].conn))
+	    evt |= SELECT_WRITE;
+	  net_select_fd(&sctx, fd, evt);
 	  active++;
 	}
 	
@@ -750,21 +921,24 @@ int main(int argc, char **argv) {
 	  struct sockaddr_in address;
 	  socklen_t addrlen = sizeof(address);
 	  
-	  if (getpeername(requests[idx].socket_fd, (struct sockaddr *)&address, 
+	  if (getpeername(fd, (struct sockaddr *)&address, 
 			  &addrlen) >= 0) {
 	    char line[512];
 	    
 	    safe_snprintf(line, sizeof(line),
 			  "#%d (%d) %d connection from %s %d",
-			  timeout ? -1 : active, fd, (int) requests[idx].last_active,
+			  timeout ? -1 : active, fd, 
+			  (int) requests[idx].last_active,
 			  inet_ntoa(address.sin_addr),
 			  ntohs(address.sin_port));
 	    
 	    if (requests[idx].conn.sock) {
 	      addrlen = sizeof(address);
-	      if (getpeername(requests[idx].conn.sock, (struct sockaddr *)&address,
+	      if (getpeername(requests[idx].conn.sock,
+			      (struct sockaddr *)&address,
 			      &addrlen) >= 0) {
-		safe_snprintf(line+strlen(line), sizeof(line)-strlen(line),
+		safe_snprintf(line+strlen(line),
+			      sizeof(line)-strlen(line),
 			      " to %s %d",
 			      inet_ntoa(address.sin_addr),
 			      ntohs(address.sin_port));
@@ -772,7 +946,8 @@ int main(int argc, char **argv) {
 	    }
 	    
 	    if (timeout) {
-	      safe_snprintf(line+strlen(line), sizeof(line)-strlen(line),
+	      safe_snprintf(line+strlen(line), 
+			    sizeof(line)-strlen(line),
 			    " (timeout)");
 	    }
 
@@ -810,46 +985,61 @@ int main(int argc, char **argv) {
     default:
       if (status > 0) {
 
-	if (net_select_read_fd(&sctx, selfpipe))
+	if (net_select_read_fd(&sctx, selfpipe)==1) {
 	  chilli_handle_signal(0, 0);
-
+	}
+	
 	if (redir->fd[0])
-	  if (net_select_read_fd(&sctx, redir->fd[0]) && 
+	  if (net_select_read_fd(&sctx, redir->fd[0])==1 && 
 	      redir_accept2(redir, 0) < 0)
 	    log_err(0, "redir_accept() failed!");
 	
 	if (redir->fd[1])
-	  if (net_select_read_fd(&sctx, redir->fd[1]) && 
+	  if (net_select_read_fd(&sctx, redir->fd[1])==1 && 
 	      redir_accept2(redir, 1) < 0)
 	    log_err(0, "redir_accept() failed!");
       
 	for (idx=0; idx < max_requests; idx++) {
-	  
+
+	  /*
+	   *  Update remote connections with activity
+	   */
 	  conn_select_update(&requests[idx].conn, &sctx);
-	
+
+	  /*
+	   *  Check client connections with activity
+	   */
 	  if (requests[idx].inuse && requests[idx].socket_fd) {
 	    int fd = requests[idx].socket_fd;
 	    
 #ifdef HAVE_SSL
 	    if (requests[idx].sslcon) {
 	      if (openssl_check_accept(requests[idx].sslcon, 0) < 0) {
+		log_dbg("ssl error %d", errno);
 		redir_conn_finish(&requests[idx].conn, &requests[idx]);
 		continue;
 	      }
 	    }
 #endif
+
+	    switch (net_select_write_fd(&sctx, fd)) {
+	    case 1:
+	      log_dbg("client writeable");
+	      redir_cli_rewrite(&requests[idx], &requests[idx].conn);
+	      break;
+	    }
 	    
 	    switch (net_select_read_fd(&sctx, fd)) {
 	    case -1:
-	      {
-		log_dbg("EXCEPTION");
-	      }
+	      log_dbg("EXCEPTION");
+	      redir_conn_finish(&requests[idx].conn, 
+				&requests[idx]);
 	      break;
 
 	    case 1:
 	      {
 		if (requests[idx].proxy) {
-		  char b[1500];
+		  char b[PKT_MAX_LEN];
 		  int r;
 		  
 #ifdef HAVE_SSL
@@ -861,7 +1051,7 @@ int main(int argc, char **argv) {
 				     b, sizeof(b)-1, 0);
 		  } else
 #endif
-		    r = recv(fd, b, sizeof(b)-1, 0);
+		    r = safe_read(fd, b, sizeof(b)-1);
 		  
 		  /*
 		    log_dbg("proxy_read: %d %d", fd, r);
@@ -869,20 +1059,34 @@ int main(int argc, char **argv) {
 		  
 		  if (r <= 0) {
 
-		    redir_conn_finish(&requests[idx].conn, &requests[idx]);
+		    log_dbg("recv %d %d %d", r, 
+			    requests[idx].conn.read_buf->slen -
+			    requests[idx].conn.read_pos,
+			    errno);
+
+		    if (!(r == -1 && 
+			  (errno == EWOULDBLOCK || errno == EAGAIN))) {
+		      if (redir_cli_rewrite(&requests[idx], 
+					    &requests[idx].conn) == 0) {
+			log_dbg("done reading and writing");
+			redir_conn_finish(&requests[idx].conn, 
+					  &requests[idx]);
+		      }
+		    }
 		    
 		  } else if (r > 0) {
 
 		    int w;
 		    requests[idx].last_active = mainclock_tick();
-		    w = safe_write(requests[idx].conn.sock, b, r);
+		    w = net_write(requests[idx].conn.sock, b, r);
 		    
 		    /*
 		      log_dbg("proxy_write: %d", w);
 		    */
 		    if (r != w) {
-		      log_err(errno, "problem writing what we read");
-		      redir_conn_finish(&requests[idx].conn, &requests[idx]);
+		      log_err(errno, "problem writing what we read from client");
+		      redir_conn_finish(&requests[idx].conn, 
+					&requests[idx]);
 		    }
 		  }
 		  
@@ -909,13 +1113,14 @@ int main(int argc, char **argv) {
 		    log_dbg("redir error");
 		  default:
 		    log_dbg("redir completed");
-		    redir_conn_finish(&requests[idx].conn, &requests[idx]);
+		    redir_conn_finish(&requests[idx].conn, 
+				      &requests[idx]);
 		    break;
 		  }
 		}
 	      }
 	      break;
-	    } /* switch(net_select_read_fd()) */
+	    }
 	  }
 	}
       }
@@ -926,6 +1131,8 @@ int main(int argc, char **argv) {
 
   redir_free(redir);
 
+  child_killall(SIGKILL);
+
   selfpipe_finish();
   
   return 0;
