@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2013 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -33,7 +34,7 @@ static void set_buffer(net_interface *iface, int what, int size);
 static void setup_rings(net_interface *iface, unsigned size, int mtu);
 static void setup_rings2(net_interface *iface);
 static void destroy_one_ring(net_interface *iface, int what);
-static void setup_filter(net_interface *iface);
+//static void setup_filter(net_interface *iface);
 #endif
 
 static int default_sndbuf = 0;
@@ -90,29 +91,28 @@ int dev_get_flags(char const *dev, int *flags) {
 
 int dev_set_address(char const *devname, struct in_addr *address, 
 		    struct in_addr *dstaddr, struct in_addr *netmask) {
+#if defined(__linux__)
   struct ifreq ifr;
+#elif defined(__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
+  struct ifaliasreq ifr;
+#endif
   int fd;
 
   memset(&ifr, 0, sizeof (ifr));
-  ifr.ifr_addr.sa_family = AF_INET;
-  ifr.ifr_dstaddr.sa_family = AF_INET;
-  
-#if defined(__linux__)
-  ifr.ifr_netmask.sa_family = AF_INET;
-  
-#elif defined(__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
-  ((struct sockaddr_in *) &ifr.ifr_addr)->sin_len = sizeof (struct sockaddr_in);
-  ((struct sockaddr_in *) &ifr.ifr_dstaddr)->sin_len = sizeof (struct sockaddr_in);
-#endif
-  
-  safe_strncpy(ifr.ifr_name, devname, IFNAMSIZ);
-  
+
   /* Create a channel to the NET kernel. */
   if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
     log_err(errno, "socket() failed");
     return -1;
   }
-  
+
+#if defined(__linux__)
+  safe_strncpy(ifr.ifr_name, devname, IFNAMSIZ);
+  ifr.ifr_addr.sa_family = AF_INET;
+  ifr.ifr_dstaddr.sa_family = AF_INET;
+  ifr.ifr_netmask.sa_family = AF_INET;
+  ifr.ifr_name[IFNAMSIZ-1] = 0; /* Make sure to terminate */
+
   if (address) { /* Set the interface address */
     ((struct sockaddr_in *) &ifr.ifr_addr)->sin_addr.s_addr = address->s_addr;
     if (ioctl(fd, SIOCSIFADDR, (void *) &ifr) < 0) {
@@ -137,25 +137,46 @@ int dev_set_address(char const *devname, struct in_addr *address,
   }
 
   if (netmask) { /* Set the netmask */
-#if defined(__linux__)
     ((struct sockaddr_in *) &ifr.ifr_netmask)->sin_addr.s_addr =  netmask->s_addr;
-
-#elif defined(__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
-    ((struct sockaddr_in *) &ifr.ifr_addr)->sin_addr.s_addr =  netmask->s_addr;
-
-#elif defined(__sun__)
-    ((struct sockaddr_in *) &ifr.ifr_addr)->sin_addr.s_addr =  netmask->s_addr;
-#else
-#error  "Unknown platform!" 
-#endif
-
     if (ioctl(fd, SIOCSIFNETMASK, (void *) &ifr) < 0) {
       log_err(errno, "ioctl(SIOCSIFNETMASK) failed");
       close(fd);
       return -1;
     }
   }
+
   
+#elif defined(__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
+  strncpy(ifr.ifra_name, devname, IFNAMSIZ);
+  ifr.ifra_name[IFNAMSIZ-1] = 0; /* Make sure to terminate */
+
+  ((struct sockaddr_in*) &ifr.ifra_addr)->sin_family = AF_INET;
+  ((struct sockaddr_in*) &ifr.ifra_addr)->sin_len = sizeof(ifr.ifra_addr);
+  ((struct sockaddr_in*) &ifr.ifra_addr)->sin_addr.s_addr = address->s_addr;
+
+  ((struct sockaddr_in*) &ifr.ifra_mask)->sin_family = AF_INET;
+  ((struct sockaddr_in*) &ifr.ifra_mask)->sin_len    = sizeof(ifr.ifra_mask);
+  ((struct sockaddr_in*) &ifr.ifra_mask)->sin_addr.s_addr = netmask->s_addr;
+
+  /* For some reason FreeBSD uses ifra_broadcast for specifying dstaddr */
+  ((struct sockaddr_in*) &ifr.ifra_broadaddr)->sin_family = AF_INET;
+  ((struct sockaddr_in*) &ifr.ifra_broadaddr)->sin_len = sizeof(ifr.ifra_broadaddr);
+  ((struct sockaddr_in*) &ifr.ifra_broadaddr)->sin_addr.s_addr = dstaddr->s_addr;
+
+    if (ioctl(fd, SIOCAIFADDR, (void *) &ifr) < 0) {
+      if (errno != EEXIST) {
+	  log_err(errno, "ioctl(SIOCAIFADDR) failed");
+      }
+      else {
+	  log_warn(errno, "ioctl(SIOCAIFADDR): Address already exists");
+      }
+      close(fd);
+      return -1;
+    }
+#else
+#error  "Unknown platform!"
+#endif
+
   close(fd);
   
   return dev_set_flags(devname, IFF_UP | IFF_RUNNING); 
@@ -264,48 +285,56 @@ int net_select_init(select_ctx *sctx) {
 }
 
 int net_select_prepare(select_ctx *sctx) {
-  int i;
 
 #ifdef ENABLE_MODULES
-  for (i=0; i < MAX_MODULES; i++) {
-    if (!_options.modules[i].name[0]) break;
-    if (_options.modules[i].ctx) {
-      struct chilli_module *m = 
+  {
+    int i;
+    for (i=0; i < MAX_MODULES; i++) {
+      if (!_options.modules[i].name[0]) break;
+      if (_options.modules[i].ctx) {
+	struct chilli_module *m = 
 	(struct chilli_module *)_options.modules[i].ctx;
-      if (m->net_select)
-	m->net_select(sctx); 
+	if (m->net_select)
+	  m->net_select(sctx); 
+      }
     }
   }
 #endif
 
 #if defined(USING_POLL) && defined(HAVE_SYS_EPOLL_H)
-  i = 0; /* for compiler */
+  /* nothing */
 #else
 #ifdef USING_POLL
-  for (i=0; i < sctx->count; i++) {
-    if (sctx->desc[i].fd) {
-      sctx->pfds[i].fd = sctx->desc[i].fd;
-      sctx->pfds[i].events = 0;
-      if (sctx->desc[i].evts & SELECT_READ)
-	sctx->pfds[i].events |= POLLIN;
-      if (sctx->desc[i].evts & SELECT_WRITE)
-	sctx->pfds[i].events |= POLLOUT;
+  {
+    int i;
+    for (i=0; i < sctx->count; i++) {
+      if (sctx->desc[i].fd) {
+	sctx->pfds[i].fd = sctx->desc[i].fd;
+	sctx->pfds[i].events = 0;
+	if (sctx->desc[i].evts & SELECT_READ)
+	  sctx->pfds[i].events |= POLLIN;
+	if (sctx->desc[i].evts & SELECT_WRITE)
+	  sctx->pfds[i].events |= POLLOUT;
+      }
     }
   }
 #else
   fd_zero(&sctx->rfds);
   fd_zero(&sctx->wfds);
   fd_zero(&sctx->efds);
-  for (i=0; i < sctx->count; i++) {
-    if (sctx->desc[i].fd) {
-      if (sctx->desc[i].evts & SELECT_READ) {
-	fd_set(sctx->desc[i].fd, &sctx->rfds);
-	fd_set(sctx->desc[i].fd, &sctx->efds);
+  {
+    int i;
+    for (i=0; i < sctx->count; i++) {
+      if (sctx->desc[i].fd) {
+	if (sctx->desc[i].evts & SELECT_READ) {
+	  fd_set(sctx->desc[i].fd, &sctx->rfds);
+	  fd_set(sctx->desc[i].fd, &sctx->efds);
+	}
+	if (sctx->desc[i].evts & SELECT_WRITE)
+	  fd_set(sctx->desc[i].fd, &sctx->wfds);
+      } else if (sctx->desc[i].evts & SELECT_RESET) {
+	sctx->desc[i].cb(&sctx->desc[i], -1);
       }
-      if (sctx->desc[i].evts & SELECT_WRITE)
-	fd_set(sctx->desc[i].fd, &sctx->wfds);
-    } else if (sctx->desc[i].evts & SELECT_RESET) {
-      sctx->desc[i].cb(&sctx->desc[i], -1);
     }
   }
 #endif
@@ -645,13 +674,16 @@ net_pcap_handler(u_char *user, const struct pcap_pkthdr *hdr,
   if (!bytes || hdr->caplen < sizeof(struct pkt_ethhdr_t))
     np->read = -1;
   else {
-    np->read = np->func(np->d, (void *)bytes, hdr->caplen);
+    struct pkt_buffer pb;
+    pkt_buffer_init(&pb, (uint8_t *)bytes, hdr->caplen, 0);
+    np->read = np->func(np->d, &pb);
   }
 }
 #endif
 
 ssize_t 
 net_read_dispatch_eth(net_interface *netif, net_handler func, void *ctx) {
+
 #if defined(USING_PCAP)
   if (netif->pd) {
     struct netpcap np; 
@@ -671,9 +703,8 @@ net_read_dispatch_eth(net_interface *netif, net_handler func, void *ctx) {
 #ifdef USING_MMAP
   if (netif->rx_ring.frames) {
     return rx_ring(netif, func, ctx);
-  }
+  } else
 #endif
-
   {
     struct pkt_buffer pb;
     uint8_t packet[PKT_MAX_LEN];
@@ -715,7 +746,7 @@ net_read_eth(net_interface *netif, void *d, size_t dlen) {
     np.dlen = dlen;
     np.read = 0;
 
-    cnt = pcap_dispatch(netif->pd, 1, net_pcap_read, &np);
+    cnt = pcap_dispatch(netif->pd, 1, net_pcap_read, (u_char *)&np);
     
     return cnt ? np.read : -1;
   }
@@ -725,7 +756,7 @@ net_read_eth(net_interface *netif, void *d, size_t dlen) {
   if (netif->rx_ring.frames) {
     log_err(0, "shouldn't be reading a mmap'ed interface this way, use dispatch");
     return -1;
-  }
+  } else 
 #endif
 
   if (netif->fd) {
@@ -764,13 +795,18 @@ net_read_eth(net_interface *netif, void *d, size_t dlen) {
     memset (&s_addr, 0, sizeof (struct sockaddr_ll));
 
 #if defined(HAVE_LINUX_TPACKET_AUXDATA_TP_VLAN_TCI) 
+
     len = safe_recvmsg(netif->fd, &msg, MSG_TRUNC);
+
 #else
+
     addr_len = sizeof (s_addr);
+
     len = safe_recvfrom(netif->fd, d, dlen,
 			MSG_DONTWAIT | MSG_TRUNC, 
 			(struct sockaddr *) &s_addr, 
 			(socklen_t *) &addr_len);
+
 #endif
     if (len < 0) {
 
@@ -778,13 +814,14 @@ net_read_eth(net_interface *netif, void *d, size_t dlen) {
 
     } else {
 
-      if (len == 0)
+      if (len == 0) {
 	log_dbg("read zero, enable ieee8021q?");
+      }
       
       if (len > dlen) {
-	log_warn(0, "data truncated, sending ICMP error %d/%d", 
+	log_warn(0, "data truncated %d/%d, sending ICMP error", 
 		 len, dlen);
-	len = dlen;
+	return -1;
       }
     }
 
@@ -821,9 +858,9 @@ net_read_eth(net_interface *netif, void *d, size_t dlen) {
 	  continue;
 	
 	ulen = len > iov.iov_len ? iov.iov_len : len;
-	
+
 	if (ulen < 2 * PKT_ETH_ALEN ||
-	    len >= PKT_MAX_LEN - 4) {
+	    len >= (dlen - 4)) {
 	  log_err(0, "bad pkt length to add 802.1q header %d/%d",
 		  ulen, len);
 	  break;
@@ -923,14 +960,18 @@ ssize_t net_write_eth(net_interface *netif, void *d, size_t dlen, struct sockadd
 int net_set_mtu(net_interface *netif, size_t mtu) {
 #if !defined(USING_PCAP)
   struct ifreq ifr;
+  int fd = socket(AF_INET, SOCK_DGRAM, 0);
+  if (fd < 0) return -1;
   memset(&ifr, 0, sizeof(ifr));
   safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
   ifr.ifr_mtu = mtu;
-  if (ioctl(netif->fd, SIOCSIFMTU, &ifr) < 0) {
-    log_err(errno, "could not set MTU on fd %d",
-	    netif->fd);
+  if (ioctl(fd, SIOCSIFMTU, &ifr) < 0) {
+    log_err(errno, "could not set MTU of %d on dev=%s",
+	    mtu, netif->devname);
+    close(fd);
     return -1;
   }
+  close(fd);
 #endif
   return 0;
 }
@@ -1183,7 +1224,6 @@ int net_open_eth(net_interface *netif) {
  **/
 int net_open_eth(net_interface *netif) {
   struct ifreq ifr;
-  struct packet_mreq mr;
   struct sockaddr_ll sa;
   int option;
 
@@ -1191,7 +1231,7 @@ int net_open_eth(net_interface *netif) {
 
   /* Create socket */
   if ((netif->fd = socket(PF_PACKET, 
-			  netif->idx ? SOCK_DGRAM : SOCK_RAW, 
+			  /*XXX netif->idx ? SOCK_DGRAM : */SOCK_RAW, 
 			  htons(netif->protocol))) < 0) {
     if (errno == EPERM) {
       log_err(errno, "Cannot create raw socket. Must be root.");
@@ -1203,40 +1243,45 @@ int net_open_eth(net_interface *netif) {
     return -1;
   }
 
-#ifndef USING_MMAP
-  /* Let's make this non-blocking */
-  ndelay_on(netif->fd);
-  coe(netif->fd);
+#ifdef USING_MMAP
+  if (!_options.mmapring) {
+#endif
 
-  option = 1;
-  if (net_setsockopt(netif->fd, SOL_SOCKET, TCP_NODELAY, 
-		     &option, sizeof(option)) < 0)
-    return -1;
-
-  /* Enable reception and transmission of broadcast frames */
-  option = 1;
-  if (net_setsockopt(netif->fd, SOL_SOCKET, SO_BROADCAST, 
-		     &option, sizeof(option)) < 0)
-    return -1;
-
-  if (_options.sndbuf > 0) {
-    option = _options.sndbuf;
-    net_setsockopt(netif->fd, SOL_SOCKET, SO_SNDBUF, &option, sizeof(option));
-  }
-  
-  if (_options.rcvbuf > 0) {
-    option = _options.rcvbuf;
-    net_setsockopt(netif->fd, SOL_SOCKET, SO_RCVBUF, &option, sizeof(option));
-  }
-
-  {
-    socklen_t len;
-    len = sizeof(default_sndbuf);
-    getsockopt(netif->fd, SOL_SOCKET, SO_SNDBUF, &default_sndbuf, &len);
-    log_dbg("Net SNDBUF %d", default_sndbuf);
-    len = sizeof(default_sndbuf);
-    getsockopt(netif->fd, SOL_SOCKET, SO_RCVBUF, &default_rcvbuf, &len);
-    log_dbg("Net RCVBUF %d", default_rcvbuf);
+    /* Let's make this non-blocking */
+    ndelay_on(netif->fd);
+    coe(netif->fd);
+    
+    option = 1;
+    if (net_setsockopt(netif->fd, SOL_SOCKET, TCP_NODELAY, 
+		       &option, sizeof(option)) < 0)
+      return -1;
+    
+    /* Enable reception and transmission of broadcast frames */
+    option = 1;
+    if (net_setsockopt(netif->fd, SOL_SOCKET, SO_BROADCAST, 
+		       &option, sizeof(option)) < 0)
+      return -1;
+    
+    if (_options.sndbuf > 0) {
+      option = _options.sndbuf;
+      net_setsockopt(netif->fd, SOL_SOCKET, SO_SNDBUF, &option, sizeof(option));
+    }
+    
+    if (_options.rcvbuf > 0) {
+      option = _options.rcvbuf;
+      net_setsockopt(netif->fd, SOL_SOCKET, SO_RCVBUF, &option, sizeof(option));
+    }
+    
+    {
+      socklen_t len;
+      len = sizeof(default_sndbuf);
+      getsockopt(netif->fd, SOL_SOCKET, SO_SNDBUF, &default_sndbuf, &len);
+      log_dbg("Net SNDBUF %d", default_sndbuf);
+      len = sizeof(default_sndbuf);
+      getsockopt(netif->fd, SOL_SOCKET, SO_RCVBUF, &default_rcvbuf, &len);
+      log_dbg("Net RCVBUF %d", default_rcvbuf);
+    }
+#ifdef USING_MMAP
   }
 #endif
 
@@ -1329,10 +1374,18 @@ int net_open_eth(net_interface *netif) {
   }
 #endif
 
-#ifndef USING_MMAP
+#ifdef USING_MMAP
+  if (_options.mmapring) {
+    setup_rings(netif,
+		_options.ringsize ? _options.ringsize : DEF_RING_SIZE, 
+		netif->mtu + 20);
+  } 
+#endif
+    
   /* Set interface in promisc mode */
   if (netif->flags & NET_PROMISC) {
-
+    struct packet_mreq mr;
+    
     memset(&ifr, 0, sizeof(ifr));
     safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
     if (ioctl(netif->fd, SIOCGIFFLAGS, (caddr_t)&ifr) == -1) {
@@ -1344,22 +1397,15 @@ int net_open_eth(net_interface *netif) {
 	log_err(errno, "Could not set flag IFF_PROMISC");
       }
     }
-
+    
     memset(&mr,0,sizeof(mr));
     mr.mr_ifindex = netif->ifindex;
     mr.mr_type = PACKET_MR_PROMISC;
-
+    
     if (net_setsockopt(netif->fd, SOL_PACKET, PACKET_ADD_MEMBERSHIP, 
 		       (char *)&mr, sizeof(mr)) < 0)
       return -1;
   }
-#endif
-
-#ifdef USING_MMAP
-  setup_rings(netif,
-	      _options.ringsize ? _options.ringsize : DEF_RING_SIZE, 
-	      netif->mtu + 20);
-#endif
 
   /* Bind to particular interface */
   memset(&sa, 0, sizeof(sa));
@@ -1388,16 +1434,20 @@ int net_open_eth(net_interface *netif) {
 #endif
 
 #ifdef USING_MMAP
-  setup_rings2(netif);
-  /*setup_filter(netif);*/
-
-  if (_options.sndbuf > 0)
-    set_buffer(netif, SO_SNDBUF, _options.sndbuf * 1024);
-
-  if (_options.rcvbuf > 0)
-    set_buffer(netif, SO_RCVBUF, _options.rcvbuf * 1024);
+  if (_options.mmapring) {
+    setup_rings2(netif);
+    /*setup_filter(netif);*/
+    
+    if (_options.sndbuf > 0)
+      set_buffer(netif, SO_SNDBUF, _options.sndbuf * 1024);
+    
+    if (_options.rcvbuf > 0)
+      set_buffer(netif, SO_RCVBUF, _options.rcvbuf * 1024);
+  }
 #endif
 
+  net_set_mtu(netif, _options.mtu);
+
   return 0;
 }
 
@@ -1570,14 +1620,11 @@ void net_run(net_interface *iface) {
 
 #ifdef USING_MMAP
 
-/*
- *  Credits: Based on that of http://code.google.com/p/ggaoed/
- */
-
 static int rx_ring(net_interface *iface, net_handler func, void *ctx) {
   unsigned cnt, was_drop;
   struct tpacket2_hdr *h;
   /*  struct timespec tv;*/
+  struct pkt_buffer pb;
   void *data;
 
   was_drop = 0;
@@ -1601,20 +1648,18 @@ static int rx_ring(net_interface *iface, net_handler func, void *ctx) {
     tv.tv_sec = h->tp_sec;
     tv.tv_nsec = h->tp_nsec;*/
     
-    /* The AoE header also contains the ethernet header, so we have
-     * start from h->tp_mac instead of h->tp_net */
-
-    if (_options.logfacility > 100)
+    if (_options.debug > 100)
       log_dbg("RX len=%d spanlen=%d (idx %d)", h->tp_len, h->tp_snaplen, iface->ifindex);
 
-    func(ctx, data + h->tp_mac, h->tp_snaplen);
+    pkt_buffer_init(&pb, (uint8_t *)data, h->tp_snaplen, h->tp_mac);
+    pb.length = h->tp_len;
+
+    func(ctx, &pb);
 
     was_drop |= h->tp_status & TP_STATUS_LOSING;
     
   next:
     h->tp_status = TP_STATUS_KERNEL;
-    /* Make sure other CPUs know about the status change */
-    /*AO_nop_full();*/
   }
 
   if (cnt >= iface->rx_ring.cnt)
@@ -1687,16 +1732,9 @@ static int tx_ring(net_interface *iface, void *packet, size_t length) {
   iface->stats.tx_bytes += h->tp_len;
   ++iface->stats.tx_cnt;
   
-  /*drop_request(q);*/
-  
-  /* Make sure buffer writes are stable before we update the status */
-  /*AO_nop_write();*/
   h->tp_status = TP_STATUS_SEND_REQUEST;
-  /* Make sure other CPUs know about the status change */
-  /*AO_nop_full();*/
 
-
-  if (_options.logfacility > 100)
+  if (_options.debug > 100)
     log_dbg("TX sent=%d (idx %d)", length, iface->ifindex);
   
   if (!iface->is_active) {
@@ -1716,7 +1754,10 @@ static void setup_one_ring(net_interface *iface, unsigned ring_size, int mtu, in
   name = what == PACKET_RX_RING ? "RX" : "TX";
   ring = what == PACKET_RX_RING ? &iface->rx_ring : &iface->tx_ring;
 
-  log_dbg("Creating %s ring: ring_size=%d; mtu=%d", name, ring_size, mtu);
+  page_size = sysconf(_SC_PAGESIZE);
+
+  log_dbg("Creating %s ring: ring_size=%d; page_size=%d; mtu=%d", 
+	  name, ring_size, page_size, mtu);
   
   /* For RX, the frame looks like:
    * - struct tpacket2_hdr
@@ -1753,7 +1794,6 @@ static void setup_one_ring(net_interface *iface, unsigned ring_size, int mtu, in
   req.tp_frame_size = ring->frame_size;
   
   /* The number of blocks is limited by the kernel implementation */
-  page_size = sysconf(_SC_PAGESIZE);
   max_blocks = page_size / sizeof(void *);
   
   /* Start with a large block size and if that fails try to lower it */
@@ -1823,7 +1863,6 @@ static void setup_frames(struct ring *ring, void *data) {
 
 /* Allocate and map the shared ring buffer */
 static void setup_rings(net_interface *iface, unsigned size, int mtu) {
-  const char *unit;
   socklen_t len;
   int ret, val;
   
@@ -1912,7 +1951,6 @@ static void setup_rings2(net_interface *iface) {
 /* Setting SO_SNDBUF/SO_RCVBUF is just advisory, so report the real value being
  * used */
 static void set_buffer(net_interface *iface, int what, int size) {
-  const char *unit;
   socklen_t len;
   int ret, val;
   
@@ -1932,6 +1970,7 @@ static void set_buffer(net_interface *iface, int what, int size) {
   what == SO_SNDBUF ? "send" : "receive", ret, unit);*/
 }
 
+#if(0)
 static void setup_filter(net_interface *iface) {
 
   if (iface->idx > 0) {
@@ -1981,6 +2020,6 @@ static void setup_filter(net_interface *iface) {
       log_err(errno, "Failed to set up the socket filter");
   }
 }
-
+#endif
 
 #endif
