@@ -1,9 +1,10 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -155,7 +156,7 @@ struct queue_item
 #define SELECT_WRITE 2
 #define SELECT_RESET 4
 
-typedef void (*select_callback)(void *data, int idx);
+typedef int (*select_callback) (void *data, int idx);
 
 typedef struct {
   int fd;
@@ -260,17 +261,11 @@ typedef struct _net_interface {
 
 } net_interface;
 
-typedef int (*net_handler)(void *ctx, void *data, size_t len);
+typedef int (*net_handler)(void *ctx, struct pkt_buffer *pb);
 
 #define net_sflags(n,f) dev_set_flags((n)->devname, (f))
 #define net_gflags(n) dev_get_flags((n)->devname, &(n)->devflags)
 
-int safe_accept(int fd, struct sockaddr *sa, socklen_t *lenptr);
-int safe_connect(int s, struct sockaddr *sock, size_t len);
-int safe_write(int s, void *b, size_t blen);
-int safe_read(int s, void *b, size_t blen);
-int safe_close (int fd);
-
 int net_open(net_interface *netif);
 int net_open_eth(net_interface *netif);
 int net_reopen(net_interface *netif);
@@ -298,7 +293,8 @@ int net_select_write_fd(select_ctx *sctx, int fd);
 ssize_t net_read_eth(net_interface *netif, void *d, size_t slen);
 
 #if defined(__linux__)
-ssize_t net_write_eth(net_interface *netif, void *d, size_t dlen, struct sockaddr_ll *dest);
+ssize_t net_write_eth(net_interface *netif, void *d, size_t dlen, 
+		      struct sockaddr_ll *dest);
 #endif
 
 ssize_t net_read_dispatch(net_interface *netif, net_handler func, void *ctx);
@@ -306,7 +302,8 @@ ssize_t net_read_dispatch_eth(net_interface *netif, net_handler func, void *ctx)
 
 int net_open_nfqueue(net_interface *netif, uint16_t q, int (*cb)());
 
-int net_select_reg(select_ctx *sctx, int fd, char evts, select_callback cb, void *ctx, int idx);
+int net_select_reg(select_ctx *sctx, int fd, char evts, 
+		   select_callback cb, void *ctx, int idx);
 int net_select_rereg(select_ctx *sctx, int oldfd, int newfd);
 int net_select_dereg(select_ctx *sctx, int oldfd);
 
@@ -315,6 +312,8 @@ int net_getip(char *dev, struct in_addr *addr);
 int net_getip6(char *dev, struct in6_addr *addr);
 #endif
 
+int net_getmac(const char *ifname, char *macaddr);
+
 int net_close(net_interface *netif);
 
 /*
