@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -155,13 +156,13 @@ int options_load(int argc, char **argv, bstring bt) {
 
   fd = open(file, O_RDONLY);
 
-  while (fd <= 0) {
+  while (fd < 0) {
     int status = 0;
     int pid = opt_run(argc, argv, 0);
     waitpid(pid, &status, 0);
     if (WIFEXITED(status) && WEXITSTATUS(status) == 2) exit(0);
     fd = open(file, O_RDONLY);
-    if (fd <= 0) {
+    if (fd < 0) {
       if (done_before) break;
       else {
 	char *offline = getenv("CHILLI_OFFLINE");
@@ -183,7 +184,7 @@ int options_load(int argc, char **argv, bstring bt) {
     }
   }
 
-  if (fd <= 0) return 0;
+  if (fd < 0) return 0;
   done_before = 1;
 
   log_dbg("PID %d rereading binary file %s", getpid(), file);
@@ -323,6 +324,9 @@ int options_fromfd(int fd, bstring bt) {
 #ifdef USING_IPC_UNIX
   if (!option_s_l(bt, &o.unixipc)) return 0;
 #endif
+#ifdef ENABLE_WPAD
+  if (!option_s_l(bt, &o.wpadpacfile)) return 0;
+#endif
 #ifdef HAVE_NETFILTER_COOVA
   if (!option_s_l(bt, &o.kname)) return 0;
 #endif
@@ -518,6 +522,9 @@ int options_save(char *file, bstring bt) {
 #ifdef USING_IPC_UNIX
   if (!option_s_s(bt, &o.unixipc)) return 0;
 #endif
+#ifdef ENABLE_WPAD
+  if (!option_s_s(bt, &o.wpadpacfile)) return 0;
+#endif
 #ifdef HAVE_NETFILTER_COOVA
   if (!option_s_s(bt, &o.kname)) return 0;
 #endif
@@ -568,7 +575,7 @@ int options_save(char *file, bstring bt) {
 
   umask(oldmask);
 
-  if (fd <= 0) {
+  if (fd < 0) {
 
     log_err(errno, "could not save to %s", file);
 
@@ -607,7 +614,7 @@ int options_save(char *file, bstring bt) {
 int options_binload(char *file) {
   int fd = open(file, O_RDONLY);
   int ok = 0;
-  if (fd > 0) {
+  if (fd >= 0) {
     bstring bt = bfromcstr("");
     log_dbg("PID %d loading binary options file %s", getpid(), file);
     ok = options_fromfd(fd, bt);
