@@ -1,11 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
+ * Copyright (C) 2007-2013 David Bird (Coova Technologies) <support@coova.com>
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2006 PicoPoint B.V.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -65,17 +65,38 @@ struct options_t {
   char * macdown;
   int txqlen;
 
+#ifdef USING_MMAP
   int ringsize;
+#endif
+
+#ifdef ENABLE_IPV6
+  struct in6_addr dns1_v6;
+  struct in6_addr dns2_v6;
+  struct in6_addr v6prefix;
+#endif
+
   int sndbuf;
   int rcvbuf;
 
   /* Radius parameters */
   struct in_addr radiuslisten;   /* IP address to listen to */
+
   struct in_addr radiusserver1;  /* IP address of radius server 1 */
   struct in_addr radiusserver2;  /* IP address of radius server 2 */
+
+  char* radiussecret;            /* Radius shared secret */
+
+#ifdef ENABLE_LARGELIMITS
+  struct in_addr radiusacctserver1; 
+  struct in_addr radiusacctserver2; 
+  struct in_addr radiusadmserver1;  
+  struct in_addr radiusadmserver2;  
+  char* radiusacctsecret;
+  char* radiusadmsecret;
+#endif
+
   uint16_t radiusauthport;       /* Authentication UDP port */
   uint16_t radiusacctport;       /* Accounting UDP port */
-  char* radiussecret;            /* Radius shared secret */
   char* radiusnasid;             /* Radius NAS-Identifier */
   char* radiuslocationid;        /* WISPr location ID */
   char* radiuslocationname;      /* WISPr location name */
@@ -88,12 +109,14 @@ struct options_t {
   int radiusretry;               /* Total amount of retries */
   int radiusretrysec;            /* Amount of retries after we switch to secondary */
 
+#ifdef ENABLE_RADPROXY
   /* Radius proxy parameters */
   int proxyport;                 /* UDP port to listen to */
   struct in_addr proxylisten;    /* IP address to listen to */
   struct in_addr proxyaddr;      /* IP address of proxy client(s) */
   struct in_addr proxymask;      /* IP mask of proxy client(s) */
   char* proxysecret;             /* Proxy shared secret */
+#endif
 
   struct in_addr postauth_proxyip;  /* IP address to proxy http to */
   int postauth_proxyport;           /* TCP port to proxy to */
@@ -108,6 +131,16 @@ struct options_t {
   int dhcpstart;
   int dhcpend;
 
+#ifdef ENABLE_MULTILAN
+#define MAX_MOREIF (MAX_RAWIF-1)
+  struct {
+    char *dhcpif;
+    char *vlan;
+  } moreif[MAX_MOREIF];
+#endif
+
+  int childmax;
+
   uint8_t peerid;
   char *peerkey;
 
@@ -126,16 +159,12 @@ struct options_t {
   uint16_t tcpmss;               /* TCP Maximum Segment Size (zero to leave unchanged) */
 
   /* UAM parameters */
-  struct in_addr uamserver[UAMSERVER_MAX]; /* IP address of UAM server */
-  int uamserverlen;              /* Number of UAM servers */
-  int uamserverport;             /* Port of UAM server */
   char* uamsecret;               /* Shared secret */
   char* uamurl;                  /* URL of authentication server */
   char* uamaaaurl;               /* URL to use for HTTP based AAA */
   char* uamhomepage;             /* URL of redirection homepage */
   char* wisprlogin;              /* Specific WISPr login url */
   char* usestatusfile;           /* Specific status file to use */
-  int uamhomepageport;		 /* Port of redirection homepage */
 
   struct in_addr uamlisten;      /* IP address of local authentication */
   int uamport;                   /* TCP port to listen to */
@@ -143,12 +172,21 @@ struct options_t {
   int uamuiport;                 /* TCP port to listen to */
 #endif
   int max_clients;               /* Max subscriber/clients */
+  int dhcphashsize;              /* DHCP MAC Hash table size */
+  int radiusqsize;               /* Size of RADIUS queue, 0 for default */
 
   struct in_addr uamlogout;      /* IP address of HTTP auto-logout */
   struct in_addr uamalias;       /* IP address of UAM Alias */
   char *uamaliasname;            /* Simple hostname (no dots) DNS name for uamalias */
   char *uamhostname;             /* Simple hostname (no dots) DNS name for uamlisten */
 
+#ifdef ENABLE_FORCEDNS
+  struct in_addr forcedns1_addr;  /* IP address to force DNS to */
+  struct in_addr forcedns2_addr;  /* IP address to force DNS to */
+  uint16_t forcedns1_port;        /* Port to force DNS to */
+  uint16_t forcedns2_port;        /* Port to force DNS to */
+#endif
+
   /* booleans */
   uint8_t layer3;                   /* Layer3 only support */
   uint8_t allowdyn:1;               /* Allow dynamic address allocation */
@@ -157,6 +195,7 @@ struct options_t {
   uint8_t noc2c:1;
   uint8_t framedservice:1;
   uint8_t usetap:1;
+  uint8_t noarpentries:1;
   uint8_t eapolenable:1;            /* Use eapol */
   uint8_t swapoctets:1;
   uint8_t chillixml:1;
@@ -180,7 +219,6 @@ struct options_t {
   uint8_t radiusoriginalurl:1;      /* Send ChilliSpot-OriginalURL in AccessRequest */
   uint8_t dhcpradius:1;             /* Send certain DHCP options in RADIUS attributes */
   uint8_t has_nexthop:1;            /* Has a nexthop entry */
-  uint8_t ieee8021q:1;              /* check for VLAN tags */
   uint8_t dhcp_broadcast:1;         /* always broadcast DHCP (when not relaying) */
   uint8_t seskeepalive:1;           /* Keep sessions alive during shutdown */
   uint8_t strictmacauth:1;          /* Be strict about DHCP macauth (don't reply DHCP until we get RADIUS) */
@@ -188,24 +226,100 @@ struct options_t {
   uint8_t dhcpmacset:1;             /* Set the dhcpif interface with the dhcpmac */
   uint8_t uamallowpost:1;           /* Set to true if the UAMPORT is allowed to access a POST */
   uint8_t redir:1;                  /* Launch redir sub-process instead of forking */
+  uint8_t redirurl:1;               /* Send redirection URL in UAM query string instead of HTTP redirect */
   uint8_t redirssl:1;               /* Enable redirection of SSL/HTTPS port (requires SSL support) */
   uint8_t uamuissl:1;               /* Enable SSL/HTTPS on uamuiport (requires SSL support) */
   uint8_t domaindnslocal:1;         /* Wildcard option to consider all hostnames *.domain local */
   uint8_t radsec:1;                 /* Use RadSec tunneling */
-  uint8_t proxymacaccept:1;         /* Auto-accept non-EAP requests on proxy port */
   uint8_t noradallow:1;             /* Authorize all sessions when RADIUS is not available */
   uint8_t redirdnsreq:1;
   uint8_t routeonetone:1;
+  uint8_t statusfilesave:1;
+  uint8_t dhcpnotidle:1;
+  uint8_t uamauthedallowed:1;
+  uint8_t postauth_proxyssl:1;
+
+#ifdef USING_MMAP
+  uint8_t mmapring:1;
+#endif
+
+#ifdef ENABLE_IPV6
+  uint8_t ipv6:1;
+  uint8_t ipv6only:1;
+  uint8_t ipv6to4:1;
+  uint8_t ipv4to6:1;
+#endif
+
+#ifdef ENABLE_LEAKYBUCKET
+  uint8_t scalewin:1;
+#endif
+
+#ifdef HAVE_PATRICIA
+  uint8_t patricia:1;
+#endif
+
+#ifdef ENABLE_GARDENACCOUNTING
+  uint8_t nousergardendata:1;
+  uint8_t uamgardendata:1;
+  uint8_t uamotherdata:1;
+#endif
+
+#ifdef ENABLE_IEEE8021Q
+  uint8_t ieee8021q:1;              /* check for VLAN tags */
+  uint8_t ieee8021q_only:1;              /* check for VLAN tags */
+#endif
+#ifdef ENABLE_RADPROXY
+  uint8_t proxymacaccept:1;         /* Auto-accept non-EAP requests on proxy port */
+  uint8_t proxyonacct:1;
+#endif
+#ifdef ENABLE_PROXYVSA
+  uint8_t vlanlocation:1;
+  uint8_t location_stop_start:1;
+  uint8_t location_copy_called:1;
+  uint8_t location_immediate_update:1;
+  uint8_t location_option_82:1;
+#endif
+#ifdef ENABLE_REDIRINJECT
+  uint8_t inject_wispr:1;
+#endif
   /* */
+#ifdef EX_OPTIONS
+#include EX_OPTIONS
+#endif
+
+#ifdef ENABLE_REDIRINJECT
+  char *inject;
+  char *inject_ext;
+#endif
+
+#ifdef ENABLE_EXTADMVSA
+#define EXTADMVSA_ATTR_CNT 4
+  struct {
+    uint32_t attr_vsa;
+    uint8_t attr;
+    char data[128];
+    char script[128];
+  } extadmvsa[EXTADMVSA_ATTR_CNT];
+#endif
 
   pass_through pass_throughs[MAX_PASS_THROUGHS];
   uint32_t num_pass_throughs;
 
+#ifdef ENABLE_AUTHEDALLOWED
+  pass_through authed_pass_throughs[MAX_PASS_THROUGHS];
+  uint32_t num_authed_pass_throughs;
+#endif
+
 #ifdef ENABLE_CHILLIREDIR
   regex_pass_through regex_pass_throughs[MAX_REGEX_PASS_THROUGHS];
   uint32_t regex_num_pass_throughs;
 #endif
 
+#ifdef ENABLE_LAYER3
+  pass_through ipsrc_pass_throughs[MAX_IPSRC_PASS_THROUGHS];
+  uint32_t ipsrc_num_pass_throughs;
+#endif
+
   char* uamdomains[MAX_UAM_DOMAINS];
   int uamdomain_ttl;
 
@@ -230,6 +344,7 @@ struct options_t {
     uint32_t attr_vsa;
     uint8_t attr;
   } proxy_loc[PROXYVSA_ATTR_CNT];
+  char * locationupdate;
 #endif
 
 #ifdef HAVE_SSL
@@ -276,6 +391,7 @@ struct options_t {
 
   /* Command-Socket */
   char *cmdsocket;
+  uint16_t cmdsocketport;
 
 #ifdef ENABLE_IEEE8021Q
   char * vlanupdate;
@@ -291,12 +407,12 @@ struct options_t {
   char *unixipc;
 #endif
 
-#ifdef HAVE_NETFILTER_COOVA
-  char *kname;
+#ifdef ENABLE_WPAD
+  char *wpadpacfile;
 #endif
 
-#ifdef ENABLE_REDIRINJECT
-  char *inject;
+#ifdef HAVE_NETFILTER_COOVA
+  char *kname;
 #endif
 
 #ifdef ENABLE_MODULES
