@@ -1,9 +1,10 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -36,10 +37,11 @@ int pkt_shape_tcpwin(struct pkt_iphdr_t *iph, uint16_t win) {
 
 int pkt_shape_tcpmss(uint8_t *packet, size_t *length) {
   int optval = _options.tcpmss;
-  struct pkt_iphdr_t *iph = iphdr(packet);
+  struct pkt_iphdr_t *iph = pkt_iphdr(packet);
+
   if (iph->protocol == PKT_IP_PROTO_TCP) {
     
-    struct pkt_tcphdr_t *tcph = tcphdr(packet);
+    struct pkt_tcphdr_t *tcph = pkt_tcphdr(packet);
     int off = tcph->offres >> 4;
     int hasmss = 0;
     
@@ -47,7 +49,8 @@ int pkt_shape_tcpmss(uint8_t *packet, size_t *length) {
     log_dbg("-->> offset: %d", off);
 #endif
     
-    if (off > 15 || off < 0) return -1;
+    if (off > 15 || off < 0) 
+      return -1;
 
     if (off > 5) {
       uint8_t *opts = tcph->options;
@@ -76,14 +79,27 @@ int pkt_shape_tcpmss(uint8_t *packet, size_t *length) {
 	    return -1;
 	  }
 	  if (type == 2 && len == 4) {
-#if(0)
-	    log_dbg("TCP OPTIONS: MSS");
+#if(1)
+	    log_dbg("TCP OPTIONS: MSS %d",
+		    ntohs(*((uint16_t *)&opts[i])));
 #endif
 	    if (ntohs(*((uint16_t *)&opts[i])) > optval) {
+
+	      log_dbg("Rewriting TCP MSS to %d", optval);
+
 	      *((uint16_t *)&opts[i]) = htons(optval);
 	      chksum(iph);
 	    }
 	    hasmss = 1;
+#ifdef ENABLE_LEAKYBUCKET
+	  } else if (_options.scalewin && type == 3 && len == 3) {
+	    log_dbg("TCP OPTIONS: window scale was %d",
+		    (int) opts[i]);
+	    if (opts[i] > 0) {
+	      opts[i]=0;
+	      chksum(iph);
+	    }
+#endif
 	  } else {
 #if(0)
 	    log_dbg("TCP OPTIONS: type %d len %d", type, len); 
@@ -98,9 +114,12 @@ int pkt_shape_tcpmss(uint8_t *packet, size_t *length) {
     if (!hasmss && *length < 1400 && tcphdr_syn(tcph)) {
       uint8_t p[PKT_BUFFER];
       memcpy(p, packet, *length);
+
+      log_dbg("Adding TCP MSS to %d", optval);
+
       {
-	struct pkt_iphdr_t *p_iph = iphdr(p);
-	struct pkt_tcphdr_t *p_tcph = tcphdr(p);
+	struct pkt_iphdr_t *p_iph = pkt_iphdr(p);
+	struct pkt_tcphdr_t *p_tcph = pkt_tcphdr(p);
 	
 	uint8_t *dst_opt = p_tcph->options + ((off - 5) * 4);
 	uint8_t *src_opt = tcph->options + ((off - 5) * 4);
