@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -44,6 +45,7 @@
 #define PKT_IP_VER_HLEN        0x45 
 #define PKT_IP_ALEN               4
 #define PKT_IP_HLEN              20
+#define PKT_IPv6_HLEN            40
 
 #define PKT_IP_PROTO_ICMP         1 /* ICMP Protocol number */
 #define PKT_IP_PROTO_IGMP         2 /* IGMP Protocol number */
@@ -61,7 +63,7 @@
 #define PKT_TCP_HLEN             20
 #define PKT_DOT1X_HLEN            4
 
-#define PKT_EAP_PLEN           1500 /* Dot1x Payload length */
+#define PKT_EAP_PLEN          10240 /* Dot1x Payload length */
 
 #define DHCP_TAG_VLEN           255 /* Tag value always shorter than this */
 #define EAPOL_TAG_VLEN          255 /* Tag value always shorter than this */
@@ -74,7 +76,6 @@
 #define DHCP_MIN_LEN   28+16+64+128 /* Length of packet excluding options */
 #define DHCP_LEN  (DHCP_MIN_LEN+DHCP_OPTIONS_LEN)
 
-/*#define PKT_BUFFER (PKT_IP_PLEN + PKT_ETH_HLEN + 4)*/
 #define PKT_BUFFER PKT_MAX_LEN
 
 struct pkt_ethhdr_t {
@@ -87,6 +88,9 @@ struct pkt_ethhdr8021q_t {
   uint8_t  dst[PKT_ETH_ALEN];
   uint8_t  src[PKT_ETH_ALEN];
   uint16_t tpid;
+#define PKT_8021Q_MASK_VID htons(0x0FFF)
+#define PKT_8021Q_MASK_PCP htons(0xE000)
+#define PKT_8021Q_MASK_CFI htons(0x1000)
   uint16_t pcp_cfi_vid;
   uint16_t prot;
 } __attribute__((packed));
@@ -167,6 +171,10 @@ struct pkt_iphdr_t {
   uint32_t daddr;
 } __attribute__((packed));
 
+#define iphdr_dont_frag(p) ((p)->opt_off_high & 0x40)
+#define iphdr_more_frag(p) ((p)->opt_off_high & 0x20)
+#define iphdr_offset(p) ntohs((((p)->opt_off_high & 0x13) << 8)|(p)->off_low)
+
 #ifdef ENABLE_IPV6
 #define PKT_IPv6_ALEN 16
 struct pkt_ip6hdr_t {
@@ -178,6 +186,21 @@ struct pkt_ip6hdr_t {
   uint8_t dst_addr[PKT_IPv6_ALEN];
 } __attribute__((packed));
 
+struct pkt_ip6pseudo_t {
+  uint8_t src_addr[PKT_IPv6_ALEN];
+  uint8_t dst_addr[PKT_IPv6_ALEN];
+  uint32_t packet_len;
+  uint8_t zero[3];
+  uint8_t next_header;
+} __attribute__((packed));
+
+int chksum6(struct pkt_ip6hdr_t *iph);
+
+struct pkt_dhcp6hdr_t {
+  uint8_t type;
+  uint8_t id[3];
+} __attribute__((packed));
+
 #define ICMPv6_NEXT_HEADER 58
 #define ipv6_version(x)  /*lazy!*/ \
 ((((1<<31)|(1<<30)|(1<<29)|(1<<28))&(ntohl((x)->ver_class_label)))>>28)
@@ -190,6 +213,30 @@ struct pkt_ip6hdr_t {
   (int) x[6], (int) x[7], (int) x[8], (int) x[9], (int) x[10], (int) x[11],\
   (int) x[12], (int) x[13], (int) x[14], (int) x[15]
 
+#define ipv6_nat64_pack(d, p) \
+  *(d)++ = 0x11; *(d)++ = 0x12; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = (p)[0]; *(d)++ = (p)[1]; *(d)++ = (p)[2] ;*(d)++ = (p)[3]
+
+#define ipv6_nat64_prefix(d) \
+  *(d)++ = 0x11; *(d)++ = 0x12; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; *(d)++ = 0
+
+#define ipv6_eui64_pack(d, p) \
+  *(d)++ = 0x11; *(d)++ = 0x11; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = (p)[0]; *(d)++ = (p)[1]; *(d)++ = (p)[2] ;*(d)++ = 0xff; \
+  *(d)++ = 0xfe; *(d)++ = (p)[3]; *(d)++ = (p)[4] ;*(d)++ = (p)[5]
+
+#define ipv6_eui64_prefix(d) \
+  *(d)++ = 0x11; *(d)++ = 0x11; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; \
+  *(d)++ = 0; *(d)++ = 0; *(d)++ = 0; *(d)++ = 0
+
 #endif
 
 struct pkt_ipphdr_t {
@@ -288,6 +335,8 @@ struct pkt_tcphdr_t {
 #define tcphdr_fin(hdr) (((hdr)->flags & TCPHDR_FLAG_FIN)==TCPHDR_FLAG_FIN)
 #define tcphdr_syn(hdr) (((hdr)->flags & TCPHDR_FLAG_SYN)==TCPHDR_FLAG_SYN)
 #define tcphdr_rst(hdr) (((hdr)->flags & TCPHDR_FLAG_RST)==TCPHDR_FLAG_RST)
+#define tcphdr_ack(hdr) (((hdr)->flags & TCPHDR_FLAG_ACK)==TCPHDR_FLAG_ACK)
+#define tcphdr_psh(hdr) (((hdr)->flags & TCPHDR_FLAG_PSH)==TCPHDR_FLAG_PSH)
 
 /*
   0                   1                   2                   3
@@ -395,7 +444,6 @@ struct eap_packet_t {
   uint8_t  payload[PKT_EAP_PLEN];
 } __attribute__((packed));
 
-
 #ifdef ENABLE_CLUSTER
 struct pkt_chillihdr_t {
   uint8_t from;
@@ -421,6 +469,11 @@ struct pkt_chillihdr_t {
 #define sizeofdot1x2(is8021q) (sizeofeth2(is8021q)+PKT_DOT1X_HLEN)
 #define sizeofudp2(is8021q)   (sizeofip2(is8021q)+PKT_UDP_HLEN)
 #define sizeoftcp2(is8021q)   (sizeofip2(is8021q)+PKT_TCP_HLEN)
+#ifdef ENABLE_IPV6
+#define sizeofip62(is8021q)   (sizeofeth2(is8021q)+PKT_IPv6_HLEN)
+#define sizeofudp62(is8021q)  (sizeofip62(is8021q)+PKT_UDP_HLEN)
+#define sizeoftcp62(is8021q)  (sizeofip62(is8021q)+PKT_TCP_HLEN)
+#endif
 
 #define sizeofeth(pkt)   sizeofeth2(is_8021q(pkt))
 #define sizeofip(pkt)    sizeofip2(is_8021q(pkt))
@@ -429,6 +482,11 @@ struct pkt_chillihdr_t {
 #define sizeoftcp(pkt)   sizeoftcp2(is_8021q(pkt))
 #define sizeofarp(pkt)   (sizeofeth(pkt)+sizeof(struct arp_packet_t))
 #define ethhdr8021q(pkt) ((struct pkt_ethhdr8021q_t *)pkt)
+#ifdef ENABLE_IPV6
+#define sizeofip6(pkt)   sizeofip62(is_8021q(pkt))
+#define sizeofudp6(pkt)  sizeofudp62(is_8021q(pkt))
+#define sizeoftcp6(pkt)  sizeoftcp62(is_8021q(pkt))
+#endif
 
 #define copy_ethproto(o,n)  \
   if (is_8021q(o)) { \
@@ -442,7 +500,7 @@ struct pkt_chillihdr_t {
 #else
 
 #define sizeofeth2(x)   (PKT_ETH_HLEN)
-#define sizeofip2(x)    (sizeofeth2(x)+PKT_IP_HLEN)
+#define sizeofip2(x)    (sizeofeth2(x)+((pkt_iphdr(x)->version_ihl&0x0f)*4))   // PKT_IP_HLEN)
 #define sizeofdot1x2(x) (sizeofeth2(x)+PKT_DOT1X_HLEN)
 #define sizeofudp2(x)   (sizeofip2(x)+PKT_UDP_HLEN)
 #define sizeoftcp2(x)   (sizeofip2(x)+PKT_TCP_HLEN)
@@ -458,20 +516,22 @@ struct pkt_chillihdr_t {
   }
 #endif
 
-#define ethhdr(pkt)   ((struct pkt_ethhdr_t *)pkt)
-#define ipphdr(pkt)   ((struct pkt_ipphdr_t *)  (((uint8_t*)(pkt)) + sizeofeth(pkt)))
-#define iphdr(pkt)    ((struct pkt_iphdr_t *)   (((uint8_t*)(pkt)) + sizeofeth(pkt)))
+#define pkt_ethhdr(pkt)   ((struct pkt_ethhdr_t *)pkt)
+#define pkt_ipphdr(pkt)   ((struct pkt_ipphdr_t *)  (((uint8_t*)(pkt)) + sizeofeth(pkt)))
+#define pkt_iphdr(pkt)    ((struct pkt_iphdr_t *)   (((uint8_t*)(pkt)) + sizeofeth(pkt)))
+#define pkt_icmphdr(pkt)  ((struct pkt_icmphdr_t *) (((uint8_t*)(pkt)) + sizeofip(pkt)))
+#define pkt_udphdr(pkt)   ((struct pkt_udphdr_t *)  (((uint8_t*)(pkt)) + sizeofip(pkt)))
+#define pkt_tcphdr(pkt)   ((struct pkt_tcphdr_t *)  (((uint8_t*)(pkt)) + sizeofip(pkt)))
+#define pkt_dot1xhdr(pkt) ((struct pkt_dot1xhdr_t *)(((uint8_t*)(pkt)) + sizeofeth(pkt)))
+#define pkt_dhcppkt(pkt)  ((struct dhcp_packet_t *) (((uint8_t*)(pkt)) + sizeofudp(pkt)))
+#define pkt_arppkt(pkt)   ((struct arp_packet_t *)  (((uint8_t*)(pkt)) + sizeofeth(pkt)))
+#define pkt_dnspkt(pkt)   ((struct dns_packet_t *)  (((uint8_t*)(pkt)) + sizeofudp(pkt)))
+#define pkt_eappkt(pkt)   ((struct eap_packet_t *)  (((uint8_t*)(pkt)) + sizeofdot1x(pkt)))
 #ifdef ENABLE_IPV6
-#define ip6hdr(pkt)   ((struct pkt_ip6hdr_t *)  (((uint8_t*)(pkt)) + sizeofeth(pkt)))
+#define pkt_ip6hdr(pkt)   ((struct pkt_ip6hdr_t *)  (((uint8_t*)(pkt)) + sizeofeth(pkt)))
+#define pkt_udp6hdr(pkt)  ((struct pkt_udphdr_t *)  (((uint8_t*)(pkt)) + sizeofip6(pkt)))
+#define pkt_tcp6hdr(pkt)  ((struct pkt_tcphdr_t *)  (((uint8_t*)(pkt)) + sizeofip6(pk)))
 #endif
-#define icmphdr(pkt)  ((struct pkt_icmphdr_t *) (((uint8_t*)(pkt)) + sizeofip(pkt)))
-#define udphdr(pkt)   ((struct pkt_udphdr_t *)  (((uint8_t*)(pkt)) + sizeofip(pkt)))
-#define tcphdr(pkt)   ((struct pkt_tcphdr_t *)  (((uint8_t*)(pkt)) + sizeofip(pkt)))
-#define dot1xhdr(pkt) ((struct pkt_dot1xhdr_t *)(((uint8_t*)(pkt)) + sizeofeth(pkt)))
-#define dhcppkt(pkt)  ((struct dhcp_packet_t *) (((uint8_t*)(pkt)) + sizeofudp(pkt)))
-#define arppkt(pkt)   ((struct arp_packet_t *)  (((uint8_t*)(pkt)) + sizeofeth(pkt)))
-#define dnspkt(pkt)   ((struct dns_packet_t *)  (((uint8_t*)(pkt)) + sizeofudp(pkt)))
-#define eappkt(pkt)   ((struct eap_packet_t *)  (((uint8_t*)(pkt)) + sizeofdot1x(pkt)))
 
 #define chilli_ethhdr(pkt)((struct pkt_chillihdr_t *)(((uint8_t*)(pkt)) + sizeofeth(pkt)))
 
@@ -485,4 +545,40 @@ int chksum(struct pkt_iphdr_t *iph);
 int pkt_shape_tcpwin(struct pkt_iphdr_t *iph, uint16_t win);
 int pkt_shape_tcpmss(uint8_t *packet, size_t *length);
 
+#if defined(ENABLE_IPV6)
+#define PKT_BUFFER_IPOFF  (sizeof(struct pkt_ethhdr8021q_t)+20)
+#else
+#define PKT_BUFFER_IPOFF  (sizeof(struct pkt_ethhdr8021q_t))
+#endif
+
+struct pkt_buffer {
+  uint8_t *   buf;
+  size_t      buflen;
+  size_t      offset;
+  size_t      length;
+};
+
+#define pkt_buffer_init(pb, b, blen, off)	\
+      (pb)->buf = (b);				\
+      (pb)->buflen = (blen);			\
+      (pb)->offset = (off);			\
+      (pb)->length = 0
+
+#define pkt_buffer_init2(pb, b, blen, off, len)	\
+      (pb)->buf = (b);				\
+      (pb)->buflen = (blen);			\
+      (pb)->offset = (off);			\
+      (pb)->length = (len)
+
+#define pkt_buffer_head(pb)    ((pb)->buf + (pb)->offset)
+#define pkt_buffer_length(pb)  ((pb)->length)
+#define pkt_buffer_size(pb)    ((pb)->buflen - (pb)->offset)
+#define pkt_buffer_grow(pb,l)  (pb)->offset -= (l); (pb)->length += (l)
+#define pkt_buffer_is_ip(pb)   ((pb)->offset == PKT_BUFFER_IPOFF)
+#define pkt_buffer_is_eth(pb)  ((pb)->offset == PKT_BUFFER_ETHOFF)
+#define pkt_buffer_is_vlan(pb) ((pb)->offset == PKT_BUFFER_VLANOFF)
+
+#define MAC_FMT "%.2X-%.2X-%.2X-%.2X-%.2X-%.2X"
+#define MAC_ARG(x) (x)[0],(x)[1],(x)[2],(x)[3],(x)[4],(x)[5]
+
 #endif
