@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -18,6 +19,9 @@
  */
 
 #include "chilli.h"
+#ifdef ENABLE_MODULES
+#include "chilli_module.h"
+#endif
 
 #define deeplog 0
 
@@ -42,12 +46,23 @@ void radius_addnasip(struct radius_t *radius, struct radius_packet_t *pack)  {
   if (!paddr)
     paddr = &_options.uamlisten;
     
-  radius_addattr(radius, pack, RADIUS_ATTR_NAS_IP_ADDRESS, 0, 0, ntohl(paddr->s_addr), NULL, 0); 
+  radius_addattr(radius, pack, RADIUS_ATTR_NAS_IP_ADDRESS, 
+		 0, 0, ntohl(paddr->s_addr), NULL, 0); 
 }
 
-void radius_addcalledstation(struct radius_t *radius, struct radius_packet_t *pack)  {
+void radius_addcalledstation(struct radius_t *radius, 
+			     struct radius_packet_t *pack,
+			     struct session_state *state)  {
   uint8_t b[32];
-  uint8_t *mac= 0;
+  uint8_t *mac = 0;
+
+#ifdef ENABLE_PROXYVSA
+  if (state->redir.calledlen) {
+    radius_addattr(radius, pack, RADIUS_ATTR_CALLED_STATION_ID, 0, 0, 0, 
+		   state->redir.called, state->redir.calledlen); 
+    return;
+  }
+#endif
 
   if (_options.nasmac)
     mac = (uint8_t *) _options.nasmac;
@@ -77,7 +92,7 @@ int radius_printqueue(int fd, struct radius_t *this) {
       safe_snprintf(line, sizeof(line), 
 		    "n=%3d id=%3d state=%3d next=%3d prev=%3d %8d %8d %d\n",
 		    n, 
-		    this->queue[n].p.id,
+		    RADIUS_QUEUE_PKT(this->queue[n].p,id),
 		    this->queue[n].state,
 		    this->queue[n].next,
 		    this->queue[n].prev,
@@ -199,6 +214,70 @@ int radius_authresp_authenticator(struct radius_t *this,
   return 0;
 }
 
+static int radius_queue_next(struct radius_t *this) {
+  int attempt = 0;
+  int qnext;
+  
+ try_again:
+  
+  qnext = this->qnext;
+
+#if(_debug_ > 1)
+  log_dbg("qnext=%d",qnext);
+#endif
+
+  if (this->queue[qnext].state == 1) {
+    
+    log_dbg("skipping over active idx %d radius-id=%d", 
+	    qnext, RADIUS_QUEUE_PKT(this->queue[qnext].p,id));
+    
+    if (attempt++ < (this->qsize ? this->qsize : 256)) {
+      this->qnext++;
+      
+      if (this->qsize) 
+	this->qnext %= this->qsize;
+      
+      goto try_again;
+    }
+    
+    log_err(0, "radius queue is full! qnext=%d qsize=%d",
+	    qnext, this->qsize);
+    
+    return -1;
+  }
+  
+  return qnext;
+}
+
+static int radius_queue_idx(struct radius_t *this, int id) {
+  int idx = id;
+  int cnt, sz;
+
+  if (id < 0 || id >= RADIUS_QUEUESIZE) {
+    return -1;
+  }
+  
+  if (this->qsize) {
+    sz = cnt = this->qsize;
+  } else {
+    sz = cnt = RADIUS_QUEUESIZE;
+  }
+
+  while (cnt-- > 0) {
+    idx %= sz;
+    if (RADIUS_QUEUE_HASPKT(this->queue[idx].p)) {
+#if(_debug_ > 1)
+      log_dbg("idx %d pid %d id %d", idx, 
+	      RADIUS_QUEUE_PKT(this->queue[idx].p,id), id);
+#endif
+      if (RADIUS_QUEUE_PKT(this->queue[idx].p,id) == id)
+	return idx;
+    }
+    idx++;
+  }
+  
+  return -1;
+}
 
 /* 
  * radius_queue_in()
@@ -210,8 +289,7 @@ int radius_queue_in(struct radius_t *this,
   struct radius_attr_t *ma = NULL; /* Message authenticator */
   struct timeval *tv;
 
-  int qnext = this->qnext;
-  int attempt = 0;
+  int qnext = radius_queue_next(this);
 
 #if(deeplog)
   if (_options.debug) {
@@ -219,30 +297,20 @@ int radius_queue_in(struct radius_t *this,
   }
 #endif
 
- try_again:
-
-  if (this->queue[qnext].state == 1) {
-    
-    log_err(0, "radius queue is full!");
-    
-    if (attempt++ < (this->qsize ? this->qsize : 255)) {
-      
-      this->qnext++;
-      
-      if (this->qsize) 
-	this->qnext %= this->qsize;
-      
-      goto try_again;
-    }
-    
+  if (qnext == -1)
     return -1;
-  }
-
+  
+#if(_debug_ > 1)
   log_dbg("RADIUS queue-in id=%d idx=%d", pack->id, qnext);
+#endif
   
   /* If packet contains message authenticator: Calculate it! */
-  if (!radius_getattr(pack, &ma, RADIUS_ATTR_MESSAGE_AUTHENTICATOR, 0,0,0)) {
-    radius_hmac_md5(this, pack, this->secret, this->secretlen, ma->v.t);
+  if (!radius_getattr(pack, &ma, 
+		      RADIUS_ATTR_MESSAGE_AUTHENTICATOR, 
+		      0,0,0)) {
+    radius_hmac_md5(this, pack, 
+		    this->secret, this->secretlen, 
+		    ma->v.t);
   }
   
   /* If accounting request: Calculate authenticator */
@@ -250,7 +318,9 @@ int radius_queue_in(struct radius_t *this,
     radius_acctreq_authenticator(this, pack);
   }
 
-  memcpy(&this->queue[qnext].p, pack, RADIUS_PACKSIZE);
+  RADIUS_QUEUE_PKTALLOC(this->queue[qnext].p);
+  if (!RADIUS_QUEUE_HASPKT(this->queue[qnext].p)) return -1;
+  memcpy(RADIUS_QUEUE_PKTPTR(this->queue[qnext].p), pack, RADIUS_PACKSIZE);
 
   this->queue[qnext].state = 1;
   this->queue[qnext].cbp = cbp;
@@ -290,7 +360,7 @@ int radius_queue_in(struct radius_t *this,
     this->qnext %= this->qsize;
   }
   
-#if(deeplog)
+#if(_debug_ > 1)
   if (_options.debug) {
     log_dbg("sending radius packet (code=%d, id=%d, len=%d)\n",
 	    pack->code, pack->id, ntohs(pack->length));
@@ -302,30 +372,6 @@ int radius_queue_in(struct radius_t *this,
   return 0;
 }
 
-static int radius_queue_idx(struct radius_t *this, int id) {
-  int idx = id;
-  int cnt, sz;
-
-  if (id < 0 || id >= RADIUS_QUEUESIZE) {
-    return -1;
-  }
-  
-  if (this->qsize) {
-    sz = cnt = this->qsize;
-  } else {
-    sz = cnt = RADIUS_QUEUESIZE;
-  }
-  
-  while (cnt-- > 0) {
-    idx %= sz;
-    if (this->queue[idx].p.id == id)
-      return idx;
-    idx++;
-  }
-  
-  return -1;
-}
-
 /* 
  * radius_queue_out()
  * Remove data from queue.
@@ -341,39 +387,47 @@ radius_queue_out(struct radius_t *this, int idx,
     id = pack_in->id;
     idx = radius_queue_idx(this, id);
   }
-
+  
   if (idx < 0) {
     log_err(0, "bad idx (%d)", idx);
     return -1;
   }
-
+  
   if (this->queue[idx].state != 1) {
     log_err(0, "RADIUS id=%d idx=%d with state != 1", 
 	    id, idx);
     return -1;
   }
   
-#if(deeplog)
+#if(_debug_ > 1)
   if (_options.debug) {
     log_dbg("radius_queue_out");
     radius_printqueue(0, this);
   }
 #endif
 
-  if (pack_in &&
-      radius_authcheck(this, pack_in, &this->queue[idx].p)) {
-    log_warn(0, "Authenticator does not match! req-id=%d res-id=%d", 
-	     this->queue[idx].p.id, pack_in->id);
-    return -1;
+  if (RADIUS_QUEUE_HASPKT(this->queue[idx].p)) {
+    if (pack_in &&
+	radius_authcheck(this, pack_in, 
+			 RADIUS_QUEUE_PKTPTR(this->queue[idx].p))) {
+      log_warn(0, "Authenticator does not match! req-id=%d res-id=%d", 
+	       RADIUS_QUEUE_PKT(this->queue[idx].p,id), 
+	       pack_in->id);
+      return -1;
+    }
+    
+    memcpy(pack_out, 
+	   RADIUS_QUEUE_PKTPTR(this->queue[idx].p), RADIUS_PACKSIZE);
+
+    RADIUS_QUEUE_PKTFREE(this->queue[idx].p);
   }
-  
-  memcpy(pack_out, &this->queue[idx].p, RADIUS_PACKSIZE);
+
   *cbp = this->queue[idx].cbp;
   
   log_dbg("RADIUS queue-out id=%d idx=%d", pack_out->id, idx);
-  
+
   this->queue[idx].state = 0;
-  
+
   /* Remove from linked list */
   if (this->queue[idx].next == -1) /* Are we the last in queue? */
     this->last = this->queue[idx].prev;
@@ -385,7 +439,7 @@ radius_queue_out(struct radius_t *this, int idx,
   else
     this->queue[this->queue[idx].prev].next = this->queue[idx].next;
   
-#if(deeplog)
+#if(_debug_ > 1)
   if (_options.debug) {
     log_dbg("radius_queue_out end");
     radius_printqueue(0, this);
@@ -402,15 +456,15 @@ radius_queue_out(struct radius_t *this, int idx,
 static int radius_queue_reschedule(struct radius_t *this, int idx) {
   struct timeval *tv;
 
-  log_dbg("Rescheduling RADIUS request id=%d idx=%d",
-	  this->queue[idx].p.id, idx);
-
   if (this->queue[idx].state != 1) {
     log_err(0, "No such id in radius queue: id=%d!", idx);
     return -1;
   }
-  
-#if(deeplog)
+
+  log_dbg("Rescheduling RADIUS request id=%d idx=%d",
+	  RADIUS_QUEUE_PKT(this->queue[idx].p,id), idx);
+
+#if(_debug_ > 1)
   if (_options.debug) {
     log_dbg("radius_reschedule");
     radius_printqueue(0, this);
@@ -597,7 +651,7 @@ int radius_timeout(struct radius_t *this) {
 
   gettimeofday(&now, NULL);
 
-#if(deeplog)
+#if(_debug_ > 1)
   if (_options.debug) {
     log_dbg("radius_timeout(%d) %8d %8d", this->first, 
 	    (int)now.tv_sec, (int)now.tv_usec);
@@ -609,6 +663,7 @@ int radius_timeout(struct radius_t *this) {
       radius_cmptv(&now, &this->queue[this->first].timeout) >= 0) {
     
     if (this->queue[this->first].retrans < _options.radiusretry) {
+
       memset(&addr, 0, sizeof(addr));
       addr.sin_family = AF_INET;
       
@@ -633,21 +688,26 @@ int radius_timeout(struct radius_t *this) {
 	}
       }
       
-      /* Use the correct port for accounting and authentication */
-      if (this->queue[this->first].p.code == RADIUS_CODE_ACCOUNTING_REQUEST)
-	addr.sin_port = htons(this->acctport);
-      else
-	addr.sin_port = htons(this->authport);
-      
-      if (sendto(this->fd, &this->queue[this->first].p,
-		 ntohs(this->queue[this->first].p.length), 0,
-		 (struct sockaddr *) &addr, sizeof(addr)) < 0) {
+      if (RADIUS_QUEUE_HASPKT(this->queue[this->first].p)) {
+
+	/* Use the correct port for accounting and authentication */
+	if (RADIUS_QUEUE_PKT(this->queue[this->first].p,code)
+	    == RADIUS_CODE_ACCOUNTING_REQUEST)
+	  addr.sin_port = htons(this->acctport);
+	else
+	  addr.sin_port = htons(this->authport);
 	
-	log_err(errno, "sendto() failed!");
-	radius_queue_reschedule(this, this->first);
-	return -1;
+	if (sendto(this->fd, 
+		   RADIUS_QUEUE_PKTPTR(this->queue[this->first].p),
+		   ntohs(RADIUS_QUEUE_PKT(this->queue[this->first].p, length)),
+		   0, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
+	  
+	  log_err(errno, "sendto() failed!");
+	  radius_queue_reschedule(this, this->first);
+	  return -1;
+	}
       }
-      
+
       if (radius_queue_reschedule(this, this->first)) {
 	log_warn(0, "Matching request was not found in queue: %d!", this->first);
 	return -1;
@@ -656,8 +716,8 @@ int radius_timeout(struct radius_t *this) {
     else { /* Finished retrans */
       if (radius_queue_out(this, this->first,
 			   0, &pack_req, &cbp)) {
-	log_warn(0, "RADIUS idx=%d id=%d was not found in queue!", 
-		 this->first, (int) this->queue[this->first].p.id);
+	log_warn(0, "RADIUS idx=%d was not found in queue!", 
+		 this->first);
 	return -1;
       }
       
@@ -671,7 +731,7 @@ int radius_timeout(struct radius_t *this) {
     }    
   }
   
-#if(deeplog)
+#if(_debug_ > 1)
   if (_options.debug) {
     log_dbg("radius_timeout");
     if (this->first > 0) {
@@ -800,7 +860,7 @@ radius_addvsa(struct radius_packet_t *pack, struct redir_state *state) {
     memcpy(m, state->vsa, state->vsalen);
     length += state->vsalen;
     pack->length = htons(length);
-    log_dbg("Recalled VSA");
+    log_dbg("Recalled VSA with length %d",length);
   }
   return 0;
 }
@@ -1044,7 +1104,9 @@ int radius_pwdecode(struct radius_t *this,
   MD5_CTX context;
   unsigned char output[RADIUS_MD5LEN];
 
+#if(_debug_ > 1)
   log_dbg("pw decode secret=%s", secret);
+#endif
 
   if (srclen > dstsize) {
     log_err(0, "radius_pwdecode srclen larger than dstsize");
@@ -1129,7 +1191,9 @@ int radius_pwencode(struct radius_t *this,
   MD5_CTX context;
   size_t i, n;
 
+#if(_debug_ > 1)
   log_dbg("pw encode secret=%s", secret);
+#endif
 
   memset(dst, 0, dstsize);
 
@@ -1234,11 +1298,6 @@ int radius_new(struct radius_t **this,
   new_radius->first = -1;
   new_radius->last = -1;
   
-  if ((new_radius->urandom_fp = fopen("/dev/urandom", "r")) == 0) {
-    log_err(errno, "fopen(/dev/urandom, r) failed");
-    return -1;
-  }
-  
   /* Initialise radius socket */
   if ((new_radius->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0 ) {
     log_err(errno, "socket() failed!");
@@ -1251,6 +1310,10 @@ int radius_new(struct radius_t **this,
   addr.sin_family = AF_INET;
   addr.sin_addr = new_radius->ouraddr;
   addr.sin_port = htons(new_radius->ourport);
+
+  log_dbg("RADIUS client %s:%d",
+	  inet_ntoa(new_radius->ouraddr),
+	  new_radius->ourport);
   
   if (bind(new_radius->fd, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
     log_err(errno, "bind() failed!");
@@ -1260,6 +1323,11 @@ int radius_new(struct radius_t **this,
     return -1;
   }
 
+  if ((new_radius->urandom_fp = fopen("/dev/urandom", "r")) == 0) {
+    log_err(errno, "fopen(/dev/urandom, r) failed");
+    return -1;
+  }
+  
 #ifdef ENABLE_RADPROXY
   if (proxy) {     /* Initialise proxy socket */
 
@@ -1313,8 +1381,7 @@ int radius_init_q(struct radius_t *this, int size) {
  * Free a radius instance. (Undo radius_new() 
  */
 int 
-radius_free(struct radius_t *this)
-{
+radius_free(struct radius_t *this) {
   if (this->queue) {
     free(this->queue);
   }
@@ -1461,11 +1528,12 @@ int radius_req(struct radius_t *this,
   else
     addr.sin_port = htons(this->authport);
   
-  if (_options.debug)
+#if(_debug_ > 1)
     log_dbg("RADIUS id=%d sent to %s:%d", 
 	    pack->id,
 	    inet_ntoa(addr.sin_addr), 
 	    ntohs(addr.sin_port));
+#endif
   
   if (sendto(this->fd, pack, len, 0,
 	     (struct sockaddr *) &addr, sizeof(addr)) < 0) {
@@ -1561,15 +1629,22 @@ radius_default_pack(struct radius_t *this,
   pack->length = htons(RADIUS_HDRSIZE);
 
   if (this->qsize == 0) {
-    /* using the full-queue-size */
-    pack->id = this->qnext;
+
+    int qnext = radius_queue_next(this);
+
+    if (qnext == -1)
+      return -1;
+
+    pack->id = qnext;
+
   } else {
     pack->id = this->nextid++;
     if (pack->id == 0) /* bump based zero */
       pack->id = this->nextid++;
   }
 
-  if (fread(pack->authenticator, 1, RADIUS_AUTHLEN, this->urandom_fp) != RADIUS_AUTHLEN) {
+  if (fread(pack->authenticator, 1, RADIUS_AUTHLEN, 
+	    this->urandom_fp) != RADIUS_AUTHLEN) {
     log_err(errno, "fread() failed");
     return -1;
   }
@@ -1583,7 +1658,8 @@ radius_default_pack(struct radius_t *this,
   case RADIUS_CODE_ACCOUNTING_REQUEST:
 
     radius_addattr(this, pack, RADIUS_ATTR_VENDOR_SPECIFIC,
-		   RADIUS_VENDOR_CHILLISPOT, RADIUS_ATTR_CHILLISPOT_VERSION, 
+		   RADIUS_VENDOR_CHILLISPOT, 
+		   RADIUS_ATTR_CHILLISPOT_VERSION, 
 		   0, (uint8_t*)VERSION, strlen(VERSION));
 
     if (code == RADIUS_CODE_ACCOUNTING_REQUEST) {
@@ -1627,6 +1703,7 @@ radius_authcheck(struct radius_t *this, struct radius_packet_t *pack,
 {
   uint8_t auth[RADIUS_AUTHLEN];
   MD5_CTX context;
+  int res;
 
   MD5Init(&context);
   MD5Update(&context, (uint8_t *) pack, RADIUS_HDRSIZE-RADIUS_AUTHLEN);
@@ -1635,8 +1712,15 @@ radius_authcheck(struct radius_t *this, struct radius_packet_t *pack,
 	    ntohs(pack->length) - RADIUS_HDRSIZE);
   MD5Update(&context, (uint8_t *)this->secret, this->secretlen);
   MD5Final(auth, &context);
+
+  res = memcmp(pack->authenticator, auth, RADIUS_AUTHLEN);
+  if (res) 
+    log_warn(0, "Authenticator "
+	     AUTH_FMT"(pkt) != "AUTH_FMT"(calc)",
+	     AUTH_ARG(pack->authenticator),
+	     AUTH_ARG(auth));
   
-  return memcmp(pack->authenticator, auth, RADIUS_AUTHLEN);
+  return res;
 }
 
 /* 
@@ -1661,6 +1745,134 @@ radius_acctcheck(struct radius_t *this, struct radius_packet_t *pack)
   return memcmp(pack->authenticator, auth, RADIUS_AUTHLEN);
 }
 
+#ifdef ENABLE_EXTADMVSA
+static void chilli_extadmvsa(struct radius_t *radius,
+			     struct app_conn_t *appconn, 
+			     struct radius_packet_t *req,
+			     struct radius_packet_t *resp) {
+  uint32_t service_type = 0;
+  struct radius_attr_t *attr = 0;
+  
+  if (!radius_getattr(req, &attr, RADIUS_ATTR_SERVICE_TYPE, 
+		      0, 0, 0)) {
+    service_type = ntohl(attr->v.i);
+  }
+  
+  if (_options.extadmvsa[0].attr && 
+      service_type == RADIUS_SERVICE_TYPE_ADMIN_USER) {
+    
+    if (req && !resp) {
+      struct stat statbuf;
+      uint8_t b[256];
+      int i, fd, len;
+      
+      memset(&statbuf, 0, sizeof(statbuf));
+      
+      for (i=0; i < EXTADMVSA_ATTR_CNT; i++) {
+	if (!_options.extadmvsa[i].attr_vsa && 
+	    !_options.extadmvsa[i].attr)
+	  break;
+	
+	if (!_options.extadmvsa[i].data[0])
+	  continue;
+	
+	if (stat(_options.extadmvsa[i].data, &statbuf)) {
+	  log_dbg("Skipping %s, does not exist",
+		  _options.extadmvsa[i].data);
+	  continue;
+	}
+	
+	if (statbuf.st_size > 127) {
+	  log_err(errno, "File %s too big", 
+		  _options.extadmvsa[i].data);
+	  continue;
+	}
+	
+	if ((fd = open(_options.extadmvsa[i].data, O_RDONLY)) < 0) {
+	  log_err(errno, "Failed to open %s", _options.extadmvsa[i].data);
+	  continue;
+	}
+	
+	log_dbg("Reading %s", _options.extadmvsa[i].data);
+	
+	len = read(fd, b, sizeof(b));
+	close(fd);
+	
+	if (len > 0 && len < sizeof(b)-1) {
+	  while (len > 1 && isspace(b[len-1])) len--;
+	  if (!_options.extadmvsa[i].attr_vsa) {
+	    radius_addattr(radius, req, _options.extadmvsa[i].attr, 
+			   0, 0, 0, b, len);
+	  } else {
+	    radius_addattr(radius, req,
+			   RADIUS_ATTR_VENDOR_SPECIFIC, 
+			   _options.extadmvsa[i].attr_vsa, 
+			   _options.extadmvsa[i].attr, 0, b, len);
+	  }
+	}
+      }
+    } else if (req && resp) {
+      int i;
+      for (i=0; i < EXTADMVSA_ATTR_CNT; i++) {
+	
+	if (!_options.extadmvsa[i].attr_vsa && 
+	    !_options.extadmvsa[i].attr)
+	  break;
+	
+	if (!_options.extadmvsa[i].attr_vsa) {
+#if(_debug_)
+	  log_dbg("looking for attr %d", _options.extadmvsa[i].attr);
+#endif
+	  if (radius_getattr(resp, &attr, _options.extadmvsa[i].attr, 
+			     0, 0, 0)) {
+	    log_dbg("didn't find attr %d", _options.extadmvsa[i].attr);
+	    attr = 0;
+	  }
+	} else {
+#if(_debug_)
+	  log_dbg("looking for attr %d/%d", _options.extadmvsa[i].attr_vsa, 
+		  _options.extadmvsa[i].attr);
+#endif
+	  if (radius_getattr(resp, &attr, 
+			     RADIUS_ATTR_VENDOR_SPECIFIC, 
+			     _options.extadmvsa[i].attr_vsa, 
+			     _options.extadmvsa[i].attr, 0)) {
+	    log_dbg("didn't find attr %d/%d", _options.extadmvsa[i].attr_vsa, 
+		    _options.extadmvsa[i].attr);
+	    attr = 0;
+	  }
+	}
+	if (attr && attr->l - 2 < 255) {
+	  char v[256];
+	  memset(v, 0, sizeof(v));
+	  memcpy(v, attr->v.t, attr->l - 2);
+	  log_dbg("Running script %s %d",
+		  _options.extadmvsa[i].script, 
+		  attr->l - 2);
+	  if (chilli_fork(CHILLI_PROC_SCRIPT, 	 
+			  _options.extadmvsa[i].script) == 0) {
+	    if (execl(
+#ifdef ENABLE_CHILLISCRIPT
+		      SBINDIR "/chilli_script", SBINDIR "/chilli_script", 
+		      _options.binconfig, 
+#else
+		      _options.extadmvsa[i].script,
+#endif
+		      _options.extadmvsa[i].script, 
+		      v, (char *) 0) != 0) {
+	      log_err(errno, "exec %s failed",
+		      _options.extadmvsa[i].script);
+	    }
+	    exit(0);
+	  } else {
+	    log_err(errno, "forking %s", _options.extadmvsa[i].script);
+	  }
+	}
+      }
+    }
+  }
+}
+#endif
 
 /* 
  * radius_decaps()
@@ -1746,7 +1958,34 @@ int radius_decaps(struct radius_t *this, int idx) {
   }
   
   /* TODO: Check consistency of attributes vs packet length */
-  
+
+#ifdef ENABLE_EXTADMVSA
+  chilli_extadmvsa(this, (struct app_conn_t *)cbp,
+		   &pack, &pack_req);
+#endif
+
+#ifdef ENABLE_MODULES
+  { int i;
+    for (i=0; i < MAX_MODULES; i++) {
+      if (!_options.modules[i].name[0]) break;
+      if (_options.modules[i].ctx) {
+	struct chilli_module *m = 
+	  (struct chilli_module *)_options.modules[i].ctx;
+	if (m->radius_handler) {
+	  int res = m->radius_handler(this, (struct app_conn_t *)cbp,
+				      &pack, &pack_req);
+	  switch (res) {
+	  case CHILLI_RADIUS_OK:
+	    break;
+	  default:
+	    return 0;
+	  }
+	}
+      }
+    }
+  }
+#endif
+
   switch (pack.code) {
   case RADIUS_CODE_ACCESS_ACCEPT:
   case RADIUS_CODE_ACCESS_REJECT:
