@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -185,26 +186,55 @@ struct radius_queue_t {      /* Holder for queued packets */
   int retrans;               /* How many times did we retransmit this? */
   int lastsent;              /* 0 or 1 indicates last server used */
   struct sockaddr_in peer;   /* Address packet was sent to / received from */
-  struct radius_packet_t p;  /* The packet stored */
+  struct radius_packet_t 
+#ifdef RADIUS_QUEUE_PACKET_PTR
+  *
+#endif
+                         p;  /* The packet stored */
   int next;                  /* Pointer to the next in queue. -1: Last */
   int prev;                  /* Pointer to the previous in queue. -1: First */
 };
 
 typedef struct radius_queue_t * radius_queue;
+struct session_state;
+
+struct radius_server_t {
+  struct in_addr addr0;
+  struct in_addr addr1;
+  char secret[RADIUS_SECRETSIZE];
+  size_t secretlen;
+  uint16_t authport;
+  uint16_t acctport;
+};
 
 struct radius_t {
   int fd;                        /* Socket file descriptor */
+
   FILE *urandom_fp;              /* /dev/urandom FILE pointer */
+
   struct in_addr ouraddr;        /* Address to listen to */
   uint16_t ourport;              /* Port to listen to */
   int coanocheck;                /* Accept coa from all IP addresses */
+
+
   int lastreply;                 /* 0 or 1 indicates last server reply */
+
   uint16_t authport;             /* His port for authentication */
   uint16_t acctport;             /* His port for accounting */
+
   struct in_addr hisaddr0;       /* Server address */
   struct in_addr hisaddr1;       /* Server address */
   char secret[RADIUS_SECRETSIZE];/* Shared secret */
-  size_t secretlen;                 /* Length of sharet secret */
+  size_t secretlen;              /* Length of sharet secret */
+
+  uint8_t nextid;                /* Next RADIUS id */
+  radius_queue queue;            /* Outstanding replies */
+  uint8_t qsize;                 /* Queue length */
+  uint8_t qnext;                 /* Next location in queue to use */
+  int first;                     /* First packet in queue (oldest timeout) */
+  int last;                      /* Last packet in queue (youngest timeout) */
+
+
 #ifdef ENABLE_RADPROXY
   int proxyfd;                   /* Proxy socket file descriptor */
   struct in_addr proxylisten;    /* Proxy address to listen to */
@@ -214,17 +244,10 @@ struct radius_t {
   char proxysecret[RADIUS_SECRETSIZE]; /* Proxy secret */
   size_t proxysecretlen;            /* Length of sharet secret */
 #endif
+
   unsigned char nas_hwaddr[6];   /* Hardware address of NAS */
-
   int debug;                     /* Print debug messages */
 
-  uint8_t nextid;                /* Next RADIUS id */
-  radius_queue queue;            /* Outstanding replies */
-  uint8_t qsize;                 /* Queue length */
-  uint8_t qnext;                 /* Next location in queue to use */
-  int first;                     /* First packet in queue (oldest timeout) */
-  int last;                      /* Last packet in queue (youngest timeout) */
-
   int (*cb_ind)       (struct radius_t *radius, struct radius_packet_t *pack,
 		       struct sockaddr_in *peer);
   int (*cb_auth_conf) (struct radius_t *radius, struct radius_packet_t *pack,
@@ -255,21 +278,25 @@ void radius_set(struct radius_t *this,
 
 /* Callback function for received request */
 int radius_set_cb_ind(struct radius_t *this,
-		      int (*cb_ind) (struct radius_t *radius, struct radius_packet_t *pack,
+		      int (*cb_ind) (struct radius_t *radius,
+				     struct radius_packet_t *pack,
 				     struct sockaddr_in *peer));
 
 /* Callback function for response to access request */
 int radius_set_cb_auth_conf(struct radius_t *this,
-			    int (*cb_auth_conf) (struct radius_t *radius, struct radius_packet_t *pack,
+			    int (*cb_auth_conf) (struct radius_t *radius, 
+						 struct radius_packet_t *pack,
 						 struct radius_packet_t *pack_req, void *cbp));
 
 /* Callback function for response to accounting request */
 int radius_set_cb_acct_conf(struct radius_t *this,
-			    int (*cb_acct_conf) (struct radius_t *radius, struct radius_packet_t *pack,
+			    int (*cb_acct_conf) (struct radius_t *radius, 
+						 struct radius_packet_t *pack,
 						 struct radius_packet_t *pack_req, void *cbp));
 
 int radius_set_cb_coa_ind(struct radius_t *this,
-			  int (*cb_coa_ind) (struct radius_t *radius, struct radius_packet_t *pack,
+			  int (*cb_coa_ind) (struct radius_t *radius, 
+					     struct radius_packet_t *pack,
 					     struct sockaddr_in *peer));
 
 /* Send of a request */
@@ -304,7 +331,8 @@ int radius_default_pack(struct radius_t *this,
 			int code);
 
 /* Extract an attribute from a packet */
-int radius_getnextattr(struct radius_packet_t *pack, struct radius_attr_t **attr,
+int radius_getnextattr(struct radius_packet_t *pack, 
+		       struct radius_attr_t **attr,
 		       uint8_t type, uint32_t vendor_id, uint8_t vendor_type,
 		       int instance, size_t *roffset);
 
@@ -341,7 +369,10 @@ int radius_timeout(struct radius_t *this);
 int radius_timeleft(struct radius_t *this, struct timeval *timeout);
 
 void radius_addnasip(struct radius_t *radius, struct radius_packet_t *pack);
-void radius_addcalledstation(struct radius_t *radius, struct radius_packet_t *pack);
+
+void radius_addcalledstation(struct radius_t *radius, 
+			     struct radius_packet_t *pack,
+			     struct session_state *state);
 
 int radius_authresp_authenticator(struct radius_t *this,
 				  struct radius_packet_t *pack,
@@ -352,4 +383,18 @@ int radius_authresp_authenticator(struct radius_t *this,
 int radius_hmac_md5(struct radius_t *this, struct radius_packet_t *pack, 
 		    char *secret, int secretlen, uint8_t *dst);
 
+#ifdef RADIUS_QUEUE_PACKET_PTR
+#define RADIUS_QUEUE_PKT(p, field) (p)->field
+#define RADIUS_QUEUE_PKTPTR(p) (p)
+#define RADIUS_QUEUE_PKTFREE(p) if((p))free((p));(p)=NULL;log_dbg("Freeing RADIUS packet")
+#define RADIUS_QUEUE_PKTALLOC(p) if((p))free((p));(p)=calloc(sizeof(struct radius_packet_t),1);log_dbg("Allocating RADIUS packet")
+#define RADIUS_QUEUE_HASPKT(p) ((p) != NULL)
+#else
+#define RADIUS_QUEUE_PKT(p, field) (p).field
+#define RADIUS_QUEUE_PKTPTR(p) &(p)
+#define RADIUS_QUEUE_PKTFREE(p) 
+#define RADIUS_QUEUE_PKTALLOC(p) 
+#define RADIUS_QUEUE_HASPKT(p) (1)
+#endif
+
 #endif	/* !_RADIUS_H */
