@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -36,11 +37,18 @@
 #define RADIUS_ATTR_CHILLISPOT_ROUTE_TO_INTERFACE        13 /* integer */
 #define RADIUS_ATTR_CHILLISPOT_CONFIG_FILE               14 /* string */
 #define RADIUS_ATTR_CHILLISPOT_SESSION_STATE             15 /* integer */
+#define RADIUS_ATTR_CHILLISPOT_SESSION_ID                16 /* string */
+#define RADIUS_ATTR_CHILLISPOT_AP_SESSION_ID             17 /* string */
+#define RADIUS_ATTR_CHILLISPOT_USER_AGENT                18 /* string */
+#define RADIUS_ATTR_CHILLISPOT_ACCEPT_LANGUAGE           19 /* string */
+
 #define	RADIUS_ATTR_CHILLISPOT_MAX_INPUT_GIGAWORDS       21 /* integer */
 #define	RADIUS_ATTR_CHILLISPOT_MAX_OUTPUT_GIGAWORDS      22 /* integer */
 #define	RADIUS_ATTR_CHILLISPOT_MAX_TOTAL_GIGAWORDS       23 /* integer */
 #define	RADIUS_ATTR_CHILLISPOT_VLAN_ID                   24 /* integer */
 #define	RADIUS_ATTR_CHILLISPOT_LOCATION                  25 /* string */
+#define	RADIUS_ATTR_CHILLISPOT_OLD_LOCATION              26 /* string */
+#define	RADIUS_ATTR_CHILLISPOT_LOCATION_CHANGE_COUNT     27 /* integer */
 
 #define RADIUS_ATTR_CHILLISPOT_SYS_UPTIME                40 /* integer */
 #define RADIUS_ATTR_CHILLISPOT_SYS_LOADAVG               41 /* string */
@@ -61,13 +69,36 @@
 #define RADIUS_ATTR_CHILLISPOT_DHCP_DNS2                 63 /* ipaddr */
 #define RADIUS_ATTR_CHILLISPOT_DHCP_GATEWAY              64 /* ipaddr */
 #define RADIUS_ATTR_CHILLISPOT_DHCP_DOMAIN               65 /* string */
+#define RADIUS_ATTR_CHILLISPOT_DHCP_RELAY                66 /* ipaddr */
 
 #define RADIUS_ATTR_CHILLISPOT_INJECT_URL                70 /* string */
 
+#define RADIUS_ATTR_CHILLISPOT_POSTAUTHPROXY_ADDRESS     75 /* ipaddr */
+#define RADIUS_ATTR_CHILLISPOT_POSTAUTHPROXY_PORT        76 /* integer */
+
+#define RADIUS_ATTR_CHILLISPOT_GARDEN_INPUT_OCTETS       80 /* integer */
+#define RADIUS_ATTR_CHILLISPOT_GARDEN_OUTPUT_OCTETS      81 /* integer */
+#define RADIUS_ATTR_CHILLISPOT_GARDEN_INPUT_GIGAWORDS    82 /* integer */
+#define RADIUS_ATTR_CHILLISPOT_GARDEN_OUTPUT_GIGAWORDS   83 /* integer */
+#define RADIUS_ATTR_CHILLISPOT_OTHER_INPUT_OCTETS        84 /* integer */
+#define RADIUS_ATTR_CHILLISPOT_OTHER_OUTPUT_OCTETS       85 /* integer */
+#define RADIUS_ATTR_CHILLISPOT_OTHER_INPUT_GIGAWORDS     86 /* integer */
+#define RADIUS_ATTR_CHILLISPOT_OTHER_OUTPUT_GIGAWORDS    87 /* integer */
+
 #define RADIUS_VALUE_CHILLISPOT_NAS_VIEWPOINT             1
 #define RADIUS_VALUE_CHILLISPOT_CLIENT_VIEWPOINT          2
 
 #define RADIUS_VALUE_CHILLISPOT_SESSION_AUTH              1
 #define RADIUS_VALUE_CHILLISPOT_SESSION_NOAUTH            2
+#define RADIUS_VALUE_CHILLISPOT_SESSION_STARTED           3
+#define RADIUS_VALUE_CHILLISPOT_SESSION_STOPPED           4
+#define RADIUS_VALUE_CHILLISPOT_SESSION_USER_LOGOUT_URL          10
+#define RADIUS_VALUE_CHILLISPOT_SESSION_IDLE_TIMEOUT_REACHED     11
+#define RADIUS_VALUE_CHILLISPOT_SESSION_TIMEOUT_REACHED          12
+#define RADIUS_VALUE_CHILLISPOT_SESSION_LOGOUT_TIME_REACHED      13
+#define RADIUS_VALUE_CHILLISPOT_SESSION_IN_DATALIMIT_REACHED     14
+#define RADIUS_VALUE_CHILLISPOT_SESSION_OUT_DATALIMIT_REACHED    15
+#define RADIUS_VALUE_CHILLISPOT_SESSION_TOTAL_DATALIMIT_REACHED  16
+#define RADIUS_VALUE_CHILLISPOT_SESSION_LOCATION_CHANGE          17
 
 #endif	/* !_RADIUS_CHILLISPOT_H */
