@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +18,6 @@
  * 
  */
 
-
 #ifndef _RADIUS_PKT_H
 #define _RADIUS_PKT_H
 
@@ -51,4 +51,7 @@ struct radius_attr_t {
   } v;        /* value */
 } __attribute__((packed));
 
+#define AUTH_FMT "%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X"
+#define AUTH_ARG(x) (x)[0],(x)[1],(x)[2],(x)[3],(x)[4],(x)[5],(x)[6],(x)[7],(x)[8],(x)[9],(x)[10],(x)[11],(x)[12],(x)[13],(x)[14],(x)[15]
+
 #endif
