@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -46,8 +47,22 @@ struct session_params {
 #define NO_SCRIPT          (1<<5)
 #define ADMIN_LOGIN        (1<<6)
 #define UAM_INJECT_URL     (1<<7)
+#define UAM_CLEAR_URL      (1<<8)
   uint16_t flags;
 
+#ifdef ENABLE_SESSPROXY
+  struct in_addr postauth_proxy;
+  uint16_t postauth_proxyport;
+#endif
+
+#ifdef ENABLE_SESSDHCP
+  struct in_addr dhcp_relay;
+#endif
+
+#ifdef ENABLE_SESSDNS
+  struct in_addr dns1;
+#endif
+
 #ifdef ENABLE_SESSGARDEN
   pass_through pass_throughs[SESSION_PASS_THROUGH_MAX];
   uint32_t pass_through_count;
@@ -80,8 +95,18 @@ struct redir_state {
   /* UAM protocol used */
   uint8_t uamprotocol;
 
+#ifdef ENABLE_USERAGENT
+  char useragent[REDIR_USERAGENTSIZE]; 
+#endif
+
+#ifdef ENABLE_ACCEPTLANGUAGE
+  char acceptlanguage[128];
+#endif
+
 #ifdef ENABLE_PROXYVSA
-#define RADIUS_PROXYVSA 128
+#define RADIUS_PROXYVSA 256
+  uint8_t called[RADIUS_ATTR_VLEN];
+  uint8_t calledlen;
   uint8_t vsa[RADIUS_PROXYVSA];
   size_t vsalen;
 #endif
@@ -91,14 +116,22 @@ struct redir_state {
 struct session_state {
   struct redir_state redir;
 
-  int authenticated;           /* 1 if user was authenticated */  
+  int authenticated; /* 1 if user was authenticated */  
 
-  char sessionid[REDIR_SESSIONID_LEN]; /* Accounting session ID */
+  char sessionid[REDIR_SESSIONID_LEN];
+#ifdef ENABLE_SESSIONID
+  char chilli_sessionid[56]; 
+#endif
+#ifdef ENABLE_APSESSIONID
+  char ap_sessionid[128]; 
+#endif
 
   time_t start_time;
   time_t interim_time;
 
-  time_t last_sent_time; /* Last time a packet was sent. Used for idle timeout calculations */
+  struct timespec last_bw_time; 
+
+  time_t last_up_time; 
   time_t last_time; /* Last time a packet was received or sent */
   time_t uamtime;
 
@@ -109,10 +142,51 @@ struct session_state {
   uint32_t terminate_cause;
   uint32_t session_id;
 
+#ifdef ENABLE_GARDENACCOUNTING
+  char garden_sessionid[REDIR_SESSIONID_LEN];
+  time_t garden_start_time;
+  time_t garden_interim_time;
+  uint64_t garden_input_octets;
+  uint64_t garden_output_octets;
+  uint64_t other_input_octets;
+  uint64_t other_output_octets;
+#endif
+
+#if defined(ENABLE_LOCATION) && defined(HAVE_AVL)
+  uint64_t input_octets_old;
+  uint64_t output_octets_old;
+#ifdef ENABLE_GARDENACCOUNTING
+  uint64_t garden_input_octets_old;
+  uint64_t garden_output_octets_old;
+  uint64_t other_input_octets_old;
+  uint64_t other_output_octets_old;
+#endif
+#endif
+
+#ifdef ENABLE_SESSIONSTATE
+  uint32_t session_state;
+#endif
+
 #ifdef ENABLE_IEEE8021Q
   uint16_t tag8021q;
 #endif
 
+#ifdef ENABLE_LOCATION
+#define MAX_LOCATION_LENGTH 56
+  char location[MAX_LOCATION_LENGTH];
+  char pending_location[MAX_LOCATION_LENGTH];
+  uint16_t location_changes;
+#endif
+
+#ifdef ENABLE_MULTILAN
+#define app_conn_idx(x)       ((x)->s_state.lanidx)
+#define app_conn_set_idx(x,c) ((x)->s_state.lanidx = (c)->lanidx)
+  int lanidx;
+#else
+#define app_conn_idx(x) 0
+#define app_conn_set_idx(x,c) 
+#endif
+
 #ifdef ENABLE_LEAKYBUCKET
   /* Leaky bucket */
   uint64_t bucketup;
