@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -26,16 +27,16 @@ void sys_err(int pri, char *fn, int ln, int en, const char *fmt, ...) {
     int sz;
     
     bvformata(sz, bt, fmt, fmt);
-    
-    if (_options.foreground && _options.debug) {
-      fprintf(stderr, "%s: %d: %d (%s) %s\n", fn, ln, en, en ? strerror(en) : "Debug", bt->data);
-    } else {
-      if (en)
-	syslog(pri, "%s: %d: %d (%s) %s", fn, ln, en, strerror(en), bt->data);
-      else
-	syslog(pri, "%s: %d: %s", fn, ln, bt->data);
+    if (sz == BSTR_OK) {
+      if (_options.foreground && _options.debug) {
+	fprintf(stderr, "%s: %d: %d (%s) %s\n", fn, ln, en, en ? strerror(en) : "Debug", bt->data);
+      } else {
+	if (en)
+	  syslog(pri, "%s: %d: %d (%s) %s", fn, ln, en, strerror(en), bt->data);
+	else
+	  syslog(pri, "%s: %d: %s", fn, ln, bt->data);
+      }
     }
-    
     bdestroy(bt);
   }
 }
@@ -48,26 +49,28 @@ void sys_errpack(int pri, char *fn, int ln, int en, struct sockaddr_in *peer,
   int n;
   
   bvformata(sz, bt, fmt, fmt);
+  if (sz == BSTR_OK) {
 
-  bassignformat(bt2, ". Packet from %s:%u, length: %d, content:",
-		inet_ntoa(peer->sin_addr),
-		ntohs(peer->sin_port),
-		len);
-
-  bconcat(bt, bt2);
-
-  for(n=0; n < len; n++) {
-    bassignformat(bt, " %02hhx", ((unsigned char*)pack)[n]);
+    bassignformat(bt2, ". Packet from %s:%u, length: %d, content:",
+		  inet_ntoa(peer->sin_addr),
+		  ntohs(peer->sin_port),
+		  len);
+    
     bconcat(bt, bt2);
-  }
+    
+    for(n=0; n < len; n++) {
+      bassignformat(bt, " %02hhx", ((unsigned char*)pack)[n]);
+      bconcat(bt, bt2);
+    }
   
-  if (_options.foreground && _options.debug) {
-    fprintf(stderr, "%s: %d: %d (%s) %s", fn, ln, en, strerror(en), bt->data);
-  } else {
-    if (en)
-      syslog(pri, "%s: %d: %d (%s) %s", fn, ln, en, strerror(en), bt->data);
-    else
-      syslog(pri, "%s: %d: %s", fn, ln, bt->data);
+    if (_options.foreground && _options.debug) {
+      fprintf(stderr, "%s: %d: %d (%s) %s", fn, ln, en, strerror(en), bt->data);
+    } else {
+      if (en)
+	syslog(pri, "%s: %d: %d (%s) %s", fn, ln, en, strerror(en), bt->data);
+      else
+	syslog(pri, "%s: %d: %s", fn, ln, bt->data);
+    }
   }
 
   bdestroy(bt);
