@@ -1,9 +1,10 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -30,15 +31,12 @@
 #undef __STRICT_ANSI__
  */
 
-#include <stdarg.h>
-#include <stdio.h>
-#include <string.h>
-#include <time.h>
 #include <ctype.h>
-
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
+#include <stdint.h>
+#include <stdarg.h>
+#include <string.h>
+#include <stdio.h>
+#include <time.h>
 
 #ifdef HAVE_STDDEF_H
 #include <stddef.h>
@@ -52,6 +50,10 @@
 #include <unistd.h>
 #endif
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -76,6 +78,11 @@
 #include <sys/stat.h>
 #endif
 
+#ifdef HAVE_LINUX_SYSINFO_H
+#define _LINUX_KERNEL_H
+#include <linux/sysinfo.h>
+#endif
+
 #ifdef HAVE_SYS_SYSINFO_H
 #include <sys/sysinfo.h>
 #endif
@@ -279,6 +286,7 @@ int safe_recvfrom(int sockfd, void *buf, size_t len, int flags,
 		  struct sockaddr *src_addr, socklen_t *addrlen);
 int safe_sendto(int s, const void *b, size_t blen, int flags,
 		const struct sockaddr *dest_addr, socklen_t addrlen);
+int safe_sendmsg(int sockfd, struct msghdr *msg, int flags);
 int safe_close (int fd);
 pid_t safe_fork();
 
@@ -308,6 +316,12 @@ void copy_mac6(uint8_t *, uint8_t *);
 
 #ifdef HAVE_PATRICIA
 #include "patricia.h"
+#define HAVE_SEARCH 1
+#endif
+
+#ifdef HAVE_AVL
+#include "avl/avl.h"
+#define HAVE_SEARCH 1
 #endif
 
 #endif
