@@ -1,10 +1,11 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* 
+ * Copyright (C) 2007-2012 David Bird (Coova Technologies) <support@coova.com>
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -359,7 +360,6 @@ int tun_addaddr(struct tun_t *this, struct in_addr *addr,
   struct sockaddr_nl local;
   size_t addr_len;
   int fd;
-  int status;
   
   struct sockaddr_nl nladdr;
   struct iovec iov;
@@ -442,7 +442,8 @@ int tun_addaddr(struct tun_t *this, struct in_addr *addr,
   req.n.nlmsg_seq = 0;
   req.n.nlmsg_flags |= NLM_F_ACK;
 
-  status = sendmsg(fd, &msg, 0); 
+  if (sendmsg(fd, &msg, 0) < 0)
+    log_err(errno, "sendmsg()");
 
   dev_set_flags(tuntap(this).devname, IFF_UP | IFF_RUNNING); 
 
@@ -635,6 +636,8 @@ int tuntap_interface(struct _net_interface *netif) {
     }
   }
 #endif
+
+  net_set_mtu(netif, _options.mtu);
   
   return 0;
   
@@ -775,7 +778,7 @@ int tun_free(struct tun_t *tun) {
 }
 
 int tun_set_cb_ind(struct tun_t *this, 
-		   int (*cb_ind) (struct tun_t *tun, void *pack, size_t len, int idx)) {
+		   int (*cb_ind) (struct tun_t *tun, struct pkt_buffer *pb, int idx)) {
   this->cb_ind = cb_ind;
   return 0;
 }
@@ -785,13 +788,16 @@ struct tundecap {
   int idx;
 };
 
-static int tun_decaps_cb(void *ctx, void *packet, size_t length) {
+static int tun_decaps_cb(void *ctx, struct pkt_buffer *pb) {
   struct tundecap *c = (struct tundecap *)ctx;
   struct pkt_iphdr_t *iph;
   int ethsize = 0;
 
   char ethhdr = (tun(c->this, c->idx).flags & NET_ETHHDR) != 0;
 
+  size_t length = pkt_buffer_length(pb);
+  uint8_t *packet = pkt_buffer_head(pb);
+
   if (c->idx) ethhdr = 0;
 
 #if(_debug_ > 1)
@@ -807,7 +813,7 @@ static int tun_decaps_cb(void *ctx, void *packet, size_t length) {
       return -1;
     
     ethsize = PKT_ETH_HLEN;
-    iph = iphdr(packet);
+    iph = pkt_iphdr(packet);
     
   } else {
     
@@ -825,7 +831,7 @@ static int tun_decaps_cb(void *ctx, void *packet, size_t length) {
 
   if (c->idx > 0) {
 #ifdef ENABLE_NETNAT
-    struct pkt_udphdr_t *udph = udphdr(packet);
+    struct pkt_udphdr_t *udph = pkt_udphdr(packet);
     if (iph->daddr == tun(c->this, c->idx).address.s_addr &&
 	ntohs(udph->dst) > 10000 && ntohs(udph->dst) < 30000) {
       if (nat_undo(c->this, c->idx, packet, length))
@@ -866,7 +872,7 @@ static int tun_decaps_cb(void *ctx, void *packet, size_t length) {
     }
   }
 
-  return c->this->cb_ind(c->this, packet, length, c->idx);
+  return c->this->cb_ind(c->this, pb, c->idx);
 }
 
 int tun_decaps(struct tun_t *this, int idx) {
@@ -890,46 +896,60 @@ int tun_decaps(struct tun_t *this, int idx) {
   
 #elif defined (__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
   struct tundecap c;
-  uint8_t buffer[PKT_MAX_LEN];
-  ssize_t status;
+  struct pkt_buffer pb;
+  uint8_t packet[PKT_MAX_LEN];
+  ssize_t length;
+
+  pkt_buffer_init(&pb, packet, sizeof(packet), PKT_BUFFER_IPOFF);
   
   c.this = this;
   c.idx = idx;
   
-  if ((status = safe_read(tun(this, idx).fd, buffer, sizeof(buffer))) <= 0) {
+  if ((length = safe_read(tun(this, idx).fd, 
+			  pkt_buffer_head(&pb), 
+			  pkt_buffer_size(&pb))) <= 0) {
     log_err(errno, "read() failed");
     return -1;
   }
+
+  pb.length = length;
   
   /*
-    log_dbg("tun_decaps(%d) %s",status,tun(tun,idx).devname);
+    log_dbg("tun_decaps(%d) %s",length,tun(tun,idx).devname);
   */
 
   if (this->cb_ind) {
 #if defined (__OpenBSD__)
     /* tun interface adds 4 bytes to front of packet under OpenBSD */
-     return tun_decaps_cb(&c, buffer+4, status-4);
+    pb.length -= 4;
+    pb.offset += 4;
 #else
-     return tun_decaps_cb(&c, buffer, status);
+    return tun_decaps_cb(&c, &pb);
 #endif
   }
 
   return 0;
 
 #elif defined (__sun__)
-  unsigned char buffer[PKT_MAX_LEN];
+  struct pkt_buffer pb;
+  uint8_t packet[PKT_MAX_LEN+4];
+  ssize_t length;
   struct strbuf sbuf;
   int f = 0;
+
+  pkt_buffer_init(&pb, packet, sizeof(packet), 4);
   
-  sbuf.maxlen = PKT_MAX_LEN;      
-  sbuf.buf = buffer;
+  sbuf.maxlen = pkt_buffer_size(&pb);
+  sbuf.buf = pkt_buffer_head(&pb);
   if (getmsg(tun(this, idx).fd, NULL, &sbuf, &f) < 0) {
     log_err(errno, "getmsg() failed");
     return -1;
   }
 
+  pb.length = sbuf.len;
+
   if (this->cb_ind)
-    return this->cb_ind(this, &packet, sbuf.len);
+    return this->cb_ind(this, &pb);
   return 0;
   
 #endif
@@ -957,9 +977,12 @@ int tun_write(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
   if (idx > 0 && tun(tun, idx).flags & NET_PPPHDR) {
     struct sockaddr_ll addr;
     size_t ethlen = sizeofeth(pack);
+#if(_debug_)
+    log_dbg("PPP Header");
+#endif
     memset(&addr,0,sizeof(addr));
     addr.sll_family = AF_PACKET;
-    addr.sll_protocol = ethhdr(pack)->prot;
+    addr.sll_protocol = pkt_ethhdr(pack)->prot;
     addr.sll_ifindex = tun(tun, idx).ifindex;
     pack += ethlen;
     len  -= ethlen;
@@ -983,14 +1006,14 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
   int result;
 
   if (_options.tcpwin)
-    pkt_shape_tcpwin(iphdr(pack), _options.tcpwin);
+    pkt_shape_tcpwin(pkt_iphdr(pack), _options.tcpwin);
 
   if (_options.tcpmss)
     pkt_shape_tcpmss(pack, &len);
 
 #ifdef ENABLE_MULTIROUTE
   if (idx > 0) {
-    struct pkt_iphdr_t *iph = iphdr(pack);
+    struct pkt_iphdr_t *iph = pkt_iphdr(pack);
     if ((iph->daddr & _options.mask.s_addr) == _options.net.s_addr ||
 	iph->daddr == dhcp->uamlisten.s_addr) {
       log_dbg("Using route idx == 0 (tun/tap)");
@@ -999,7 +1022,7 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
   }
 
   if (_options.routeonetone && idx > 0) {
-    struct pkt_iphdr_t *iph = iphdr(pack);
+    struct pkt_iphdr_t *iph = pkt_iphdr(pack);
     if (!tun(tun, idx).nataddress.s_addr)
       tun(tun, idx).nataddress.s_addr = iph->saddr;
     iph->saddr = tun(tun, idx).address.s_addr;
@@ -1017,7 +1040,7 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
 
 #if defined(HAVE_NETFILTER_QUEUE) || defined(HAVE_NETFILTER_COOVA)
   if (_options.uamlisten.s_addr != _options.dhcplisten.s_addr) {
-    struct pkt_iphdr_t *iph = iphdr(pack);
+    struct pkt_iphdr_t *iph = pkt_iphdr(pack);
 
     iph->saddr  = iph->saddr & ~(_options.mask.s_addr);
     iph->saddr |= _options.uamlisten.s_addr & _options.mask.s_addr;
@@ -1052,7 +1075,7 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
 
     copy_mac6(ethh->dst, gwaddr);
 
-#if(_debug_)
+#if(_debug_ > 1)
     log_dbg("writing to tap src=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x "
 	    "dst=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x len=%d",
 	    ethh->src[0],ethh->src[1],ethh->src[2],
@@ -1074,24 +1097,22 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
   result = tun_write(tun, pack, len, idx);
 
   if (result < 0) {
-    log_err(errno, "tun_write(%d)", result);
+    log_err(errno, "tun_write(%d) = %d", len, result);
   }
 
   return result;
 }
 
 int tun_runscript(struct tun_t *tun, char* script, int wait) {
-  char saddr[TUN_ADDRSIZE];
-  char smask[TUN_ADDRSIZE];
-  char b[TUN_ADDRSIZE];
   struct in_addr net;
   pid_t pid;
-
+  char b[56];
+  
   log_dbg("Running %s", script);
 
   net.s_addr = tuntap(tun).address.s_addr & tuntap(tun).netmask.s_addr;
 
-  if ((pid = fork()) < 0) {
+  if ((pid = safe_fork()) < 0) {
     log_err(errno, "fork() returned -1!");
     return 0;
   }
@@ -1108,51 +1129,46 @@ int tun_runscript(struct tun_t *tun, char* script, int wait) {
     return 0;
   }
   
-  if (setenv("DEV", tuntap(tun).devname, 1) != 0) {
-    log_err(errno, "setenv() did not return 0!");
-    exit(0);
+  set_env("DHCPIF", VAL_STRING, _options.dhcpif ? _options.dhcpif : "", 0);
+  set_env("DEV", VAL_STRING, tun(tun, 0).devname, 0);
+  set_env("ADDR", VAL_IN_ADDR, &tuntap(tun).address, 0);
+  set_env("MASK", VAL_IN_ADDR, &tuntap(tun).netmask, 0);
+  set_env("NET", VAL_IN_ADDR, &net, 0);
+
+  set_env("UAMLISTEN", VAL_IN_ADDR, &_options.uamlisten, 0);
+  if (_options.dhcplisten.s_addr && 
+      _options.dhcplisten.s_addr != _options.uamlisten.s_addr) {
+    set_env("DHCPLISTEN", VAL_IN_ADDR, &_options.dhcplisten, 0);
   }
 
-  safe_strncpy(saddr, inet_ntoa(tuntap(tun).address), sizeof(saddr));
-  if (setenv("ADDR", saddr, 1 ) != 0) {
-    log_err(errno, "setenv() did not return 0!");
-    exit(0);
-  }
+  safe_snprintf(b, sizeof(b), "%d", (int)_options.mtu);
+  set_env("MTU", VAL_STRING, b, 0);
 
-  safe_strncpy(smask, inet_ntoa(tuntap(tun).netmask), sizeof(smask));
-  if (setenv("MASK", smask, 1) != 0) {
-    log_err(errno, "setenv() did not return 0!");
-    exit(0);
-  }
-
-  safe_strncpy(b, inet_ntoa(net), sizeof(b));
-  if (setenv("NET", b, 1 ) != 0) {
-    log_err(errno, "setenv() did not return 0!");
-    exit(0);
-  }
-
-  safe_snprintf(b, sizeof(b), "%d", _options.uamport);
-  if (setenv("UAMPORT", b, 1 ) != 0) {
-    log_err(errno, "setenv() did not return 0!");
-    exit(0);
-  }
+  safe_snprintf(b, sizeof(b), "%d", (int)_options.uamport);
+  set_env("UAMPORT", VAL_STRING, b, 0);
 
 #ifdef ENABLE_UAMUIPORT
-  safe_snprintf(b, sizeof(b), "%d", _options.uamuiport);
-  if (setenv("UAMUIPORT", b, 1 ) != 0) {
-    log_err(errno, "setenv() did not return 0!");
-    exit(0);
+  safe_snprintf(b, sizeof(b), "%d", (int)_options.uamuiport);
+  set_env("UAMUIPORT", VAL_STRING, b, 0);
+#endif
+
+#ifdef ENABLE_LAYER3
+  if (_options.layer3) {
+    set_env("LAYER3", VAL_STRING, "1", 0);
   }
 #endif
 
-  if (setenv("DHCPIF", _options.dhcpif ? _options.dhcpif : "", 1 ) != 0) {
-    log_err(errno, "setenv() did not return 0!");
-    exit(0);
-  }
+#ifdef ENABLE_IEEE8021Q
+  if (_options.ieee8021q)
+    set_env("IEEE8021Q", VAL_STRING, "1" , 0);
+  if (_options.ieee8021q_only)
+    set_env("ONLY8021Q", VAL_STRING, "1" , 0);
+#endif
 
-#ifdef ENABLE_LAYER3
-  if (_options.layer3)
-    setenv("LAYER3", "1", 1);
+#ifdef HAVE_NETFILTER_COOVA
+  if (_options.kname) {
+    set_env("KNAME", VAL_STRING, _options.kname, 0);
+  }
 #endif
 
   if (execl(
@@ -1161,8 +1177,8 @@ int tun_runscript(struct tun_t *tun, char* script, int wait) {
 #else
 	    script,
 #endif
-	    script, tuntap(tun).devname, saddr, smask, (char *) 0) != 0) {
-      
+	    script, tuntap(tun).devname, (char *) 0) != 0) {
+    
     log_err(errno, "execl(%s) did not return 0!",script);
     exit(0);
   }
